@@ -69,7 +69,6 @@
 #include "shlwapi.h"
 #include "comctl32.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(commctrl);
 
@@ -136,7 +135,7 @@ BOOL WINAPI RegisterClassNameW(const WCHAR *class)
     while (min <= max)
     {
         int res, pos = (min + max) / 2;
-        if (!(res = strcmpiW(class, classes[pos].nameW)))
+        if (!(res = wcsicmp(class, classes[pos].nameW)))
         {
             classes[pos].fn_register();
             return TRUE;
@@ -1082,6 +1081,9 @@ BOOL WINAPI SetWindowSubclass (HWND hWnd, SUBCLASSPROC pfnSubclass,
 
    TRACE ("(%p, %p, %lx, %lx)\n", hWnd, pfnSubclass, uIDSubclass, dwRef);
 
+   if (!hWnd || !pfnSubclass)
+       return FALSE;
+
    /* Since the window procedure that we set here has two additional arguments,
     * we can't simply set it as the new window procedure of the window. So we
     * set our own window procedure and then calculate the other two arguments
