@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
 #include "d3dx11.h"
 #include "d3dcompiler.h"
 
@@ -65,24 +63,6 @@ HRESULT WINAPI D3DX11CreateTextureFromMemory(ID3D11Device *device, const void *d
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI D3DX11SaveTextureToFileW(ID3D11DeviceContext *context, ID3D11Resource *texture,
-        D3DX11_IMAGE_FILE_FORMAT format, const WCHAR *filename)
-{
-    FIXME("context %p, texture %p, format %u, filename %s stub!\n",
-            context, texture, format, debugstr_w(filename));
-
-    return E_NOTIMPL;
-}
-
-HRESULT WINAPI D3DX11SaveTextureToFileA(ID3D11DeviceContext *context, ID3D11Resource *texture,
-        D3DX11_IMAGE_FILE_FORMAT format, const char *filename)
-{
-    FIXME("context %p, texture %p, format %u, filename %s stub!\n",
-            context, texture, format, debugstr_a(filename));
-
-    return E_NOTIMPL;
-}
-
 HRESULT WINAPI D3DX11SaveTextureToMemory(ID3D11DeviceContext *context, ID3D11Resource *texture,
         D3DX11_IMAGE_FILE_FORMAT format, ID3D10Blob **buffer, UINT flags)
 {
