@@ -65,6 +65,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
  */
 
 unsigned   dbghelp_options = SYMOPT_UNDNAME;
+BOOL       dbghelp_opt_native = FALSE;
 
 static struct process* process_first /* = NULL */;
 
@@ -451,6 +452,44 @@ DWORD WINAPI SymGetOptions(void)
     return dbghelp_options;
 }
 
+/******************************************************************
+ *		SymSetExtendedOption (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymSetExtendedOption(IMAGEHLP_EXTENDED_OPTIONS option, BOOL value)
+{
+    BOOL old = FALSE;
+
+    switch(option)
+    {
+        case SYMOPT_EX_WINE_NATIVE_MODULES:
+            old = dbghelp_opt_native;
+            dbghelp_opt_native = value;
+            break;
+        default:
+            FIXME("Unsupported option %d with value %d\n", option, value);
+    }
+
+    return old;
+}
+
+/******************************************************************
+ *		SymGetExtendedOption (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymGetExtendedOption(IMAGEHLP_EXTENDED_OPTIONS option)
+{
+    switch(option)
+    {
+        case SYMOPT_EX_WINE_NATIVE_MODULES:
+            return dbghelp_opt_native;
+        default:
+            FIXME("Unsupported option %d\n", option);
+    }
+
+    return FALSE;
+}
+
 /******************************************************************
  *		SymSetParentWindow (DBGHELP.@)
  *
