@@ -26,17 +26,22 @@
 #include <stdarg.h>
 #include <string.h>
 #include <stdio.h>
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winbase.h"
+#include "wingdi.h"
 #include "ddrawgdi.h"
 #include "wine/winbase16.h"
 #include "winuser.h"
 #include "winternl.h"
+#include "ddk/d3dkmthk.h"
 
 #include "gdi_private.h"
 #include "wine/unicode.h"
 #include "wine/list.h"
 #include "wine/debug.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(driver);
 
@@ -47,9 +52,25 @@ struct graphics_driver
     const struct gdi_dc_funcs *funcs;
 };
 
+struct d3dkmt_adapter
+{
+    D3DKMT_HANDLE handle;               /* Kernel mode graphics adapter handle */
+    INT ordinal;                        /* Graphics adapter ordinal */
+    struct list entry;                  /* List entry */
+};
+
+struct d3dkmt_device
+{
+    D3DKMT_HANDLE handle;               /* Kernel mode graphics device handle*/
+    struct list entry;                  /* List entry */
+};
+
 static struct list drivers = LIST_INIT( drivers );
 static struct graphics_driver *display_driver;
 
+static struct list d3dkmt_adapters = LIST_INIT( d3dkmt_adapters );
+static struct list d3dkmt_devices = LIST_INIT( d3dkmt_devices );
+
 const struct gdi_dc_funcs *font_driver = NULL;
 
 static CRITICAL_SECTION driver_section;
@@ -114,6 +135,32 @@ static const struct gdi_dc_funcs *get_display_driver(void)
 }
 
 
+/**********************************************************************
+ *	     is_display_device
+ */
+static BOOL is_display_device( LPCWSTR name )
+{
+    static const WCHAR display_deviceW[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y'};
+    const WCHAR *p = name;
+
+    if (strncmpiW( name, display_deviceW, sizeof(display_deviceW) / sizeof(WCHAR) ))
+        return FALSE;
+
+    p += sizeof(display_deviceW) / sizeof(WCHAR);
+
+    if (!isdigitW( *p++ ))
+        return FALSE;
+
+    for (; *p; p++)
+    {
+        if (!isdigitW( *p ))
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
+
 /**********************************************************************
  *	     DRIVER_load_driver
  */
@@ -122,10 +169,9 @@ const struct gdi_dc_funcs *DRIVER_load_driver( LPCWSTR name )
     HMODULE module;
     struct graphics_driver *driver, *new_driver;
     static const WCHAR displayW[] = { 'd','i','s','p','l','a','y',0 };
-    static const WCHAR display1W[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
 
     /* display driver is a special case */
-    if (!strcmpiW( name, displayW ) || !strcmpiW( name, display1W )) return get_display_driver();
+    if (!strcmpiW( name, displayW ) || is_display_device( name )) return get_display_driver();
 
     if ((module = GetModuleHandleW( name )))
     {
@@ -187,126 +233,131 @@ void CDECL __wine_set_display_driver( HMODULE module )
 }
 
 
-static INT nulldrv_AbortDoc( PHYSDEV dev )
+static INT CDECL nulldrv_AbortDoc( PHYSDEV dev )
 {
     return 0;
 }
 
-static BOOL nulldrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                         INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                               INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                           INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                                 INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
+static BOOL CDECL nulldrv_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
 {
     if (!display_driver || !display_driver->funcs->pCreateCompatibleDC) return TRUE;
     return display_driver->funcs->pCreateCompatibleDC( NULL, pdev );
 }
 
-static BOOL nulldrv_CreateDC( PHYSDEV *dev, LPCWSTR driver, LPCWSTR device,
-                              LPCWSTR output, const DEVMODEW *devmode )
+static BOOL CDECL nulldrv_CreateDC( PHYSDEV *dev, LPCWSTR driver, LPCWSTR device,
+                                    LPCWSTR output, const DEVMODEW *devmode )
 {
     assert(0);  /* should never be called */
     return FALSE;
 }
 
-static BOOL nulldrv_DeleteDC( PHYSDEV dev )
+static BOOL CDECL nulldrv_DeleteDC( PHYSDEV dev )
 {
     assert(0);  /* should never be called */
     return TRUE;
 }
 
-static BOOL nulldrv_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
+static BOOL CDECL nulldrv_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
 {
     return TRUE;
 }
 
-static DWORD nulldrv_DeviceCapabilities( LPSTR buffer, LPCSTR device, LPCSTR port,
-                                         WORD cap, LPSTR output, DEVMODEA *devmode )
+static DWORD CDECL nulldrv_DeviceCapabilities( LPSTR buffer, LPCSTR device, LPCSTR port,
+                                               WORD cap, LPSTR output, DEVMODEA *devmode )
 {
     return -1;
 }
 
-static BOOL nulldrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
+static BOOL CDECL nulldrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
 {
     return TRUE;
 }
 
-static INT nulldrv_EndDoc( PHYSDEV dev )
+static INT CDECL nulldrv_EndDoc( PHYSDEV dev )
 {
     return 0;
 }
 
-static INT nulldrv_EndPage( PHYSDEV dev )
+static INT CDECL nulldrv_EndPage( PHYSDEV dev )
 {
     return 0;
 }
 
-static BOOL nulldrv_EnumFonts( PHYSDEV dev, LOGFONTW *logfont, FONTENUMPROCW proc, LPARAM lParam )
+static BOOL CDECL nulldrv_EnumFonts( PHYSDEV dev, LOGFONTW *logfont, FONTENUMPROCW proc, LPARAM lParam )
 {
     return TRUE;
 }
 
-static INT nulldrv_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW func, LPARAM lparam )
+static INT CDECL nulldrv_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW func, LPARAM lparam )
 {
     return -1;
 }
 
-static INT nulldrv_ExtDeviceMode( LPSTR buffer, HWND hwnd, DEVMODEA *output, LPSTR device,
-                                  LPSTR port, DEVMODEA *input, LPSTR profile, DWORD mode )
+static INT CDECL nulldrv_ExtDeviceMode( LPSTR buffer, HWND hwnd, DEVMODEA *output, LPSTR device,
+                                        LPSTR port, DEVMODEA *input, LPSTR profile, DWORD mode )
 {
     return -1;
 }
 
-static INT nulldrv_ExtEscape( PHYSDEV dev, INT escape, INT in_size, const void *in_data,
+static INT CDECL nulldrv_ExtEscape( PHYSDEV dev, INT escape, INT in_size, const void *in_data,
                                     INT out_size, void *out_data )
 {
     return 0;
 }
 
-static BOOL nulldrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
+static BOOL CDECL nulldrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_FontIsLinked( PHYSDEV dev )
+static BOOL CDECL nulldrv_FontIsLinked( PHYSDEV dev )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GdiComment( PHYSDEV dev, UINT size, const BYTE *data )
+static BOOL CDECL nulldrv_GdiComment( PHYSDEV dev, UINT size, const BYTE *data )
 {
     return FALSE;
 }
 
-static UINT nulldrv_GetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+static UINT CDECL nulldrv_GetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
 {
     return DCB_RESET;
 }
 
-static BOOL nulldrv_GetCharABCWidths( PHYSDEV dev, UINT first, UINT last, LPABC abc )
+static BOOL CDECL nulldrv_GetCharABCWidths( PHYSDEV dev, UINT first, UINT last, LPABC abc )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetCharABCWidthsI( PHYSDEV dev, UINT first, UINT count, WORD *indices, LPABC abc )
+static BOOL CDECL nulldrv_GetCharABCWidthsI( PHYSDEV dev, UINT first, UINT count, WORD *indices, LPABC abc )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffer )
+static BOOL CDECL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffer )
 {
     return FALSE;
 }
 
-static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
+static BOOL CDECL nulldrv_GetCharWidthInfo( PHYSDEV dev, void *info )
+{
+    return FALSE;
+}
+
+static INT CDECL nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
 {
     int bpp;
 
@@ -398,75 +449,75 @@ static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
     }
 }
 
-static BOOL nulldrv_GetDeviceGammaRamp( PHYSDEV dev, void *ramp )
+static BOOL CDECL nulldrv_GetDeviceGammaRamp( PHYSDEV dev, void *ramp )
 {
     SetLastError( ERROR_INVALID_PARAMETER );
     return FALSE;
 }
 
-static DWORD nulldrv_GetFontData( PHYSDEV dev, DWORD table, DWORD offset, LPVOID buffer, DWORD length )
+static DWORD CDECL nulldrv_GetFontData( PHYSDEV dev, DWORD table, DWORD offset, LPVOID buffer, DWORD length )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetFontRealizationInfo( PHYSDEV dev, void *info )
+static BOOL CDECL nulldrv_GetFontRealizationInfo( PHYSDEV dev, void *info )
 {
     return FALSE;
 }
 
-static DWORD nulldrv_GetFontUnicodeRanges( PHYSDEV dev, LPGLYPHSET glyphs )
+static DWORD CDECL nulldrv_GetFontUnicodeRanges( PHYSDEV dev, LPGLYPHSET glyphs )
 {
     return 0;
 }
 
-static DWORD nulldrv_GetGlyphIndices( PHYSDEV dev, LPCWSTR str, INT count, LPWORD indices, DWORD flags )
+static DWORD CDECL nulldrv_GetGlyphIndices( PHYSDEV dev, LPCWSTR str, INT count, LPWORD indices, DWORD flags )
 {
     return GDI_ERROR;
 }
 
-static DWORD nulldrv_GetGlyphOutline( PHYSDEV dev, UINT ch, UINT format, LPGLYPHMETRICS metrics,
-                                      DWORD size, LPVOID buffer, const MAT2 *mat )
+static DWORD CDECL nulldrv_GetGlyphOutline( PHYSDEV dev, UINT ch, UINT format, LPGLYPHMETRICS metrics,
+                                            DWORD size, LPVOID buffer, const MAT2 *mat )
 {
     return GDI_ERROR;
 }
 
-static BOOL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
+static BOOL CDECL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
 {
     return FALSE;
 }
 
-static DWORD nulldrv_GetImage( PHYSDEV dev, BITMAPINFO *info, struct gdi_image_bits *bits,
-                               struct bitblt_coords *src )
+static DWORD CDECL nulldrv_GetImage( PHYSDEV dev, BITMAPINFO *info, struct gdi_image_bits *bits,
+                                     struct bitblt_coords *src )
 {
     return ERROR_NOT_SUPPORTED;
 }
 
-static DWORD nulldrv_GetKerningPairs( PHYSDEV dev, DWORD count, LPKERNINGPAIR pairs )
+static DWORD CDECL nulldrv_GetKerningPairs( PHYSDEV dev, DWORD count, LPKERNINGPAIR pairs )
 {
     return 0;
 }
 
-static UINT nulldrv_GetOutlineTextMetrics( PHYSDEV dev, UINT size, LPOUTLINETEXTMETRICW otm )
+static UINT CDECL nulldrv_GetOutlineTextMetrics( PHYSDEV dev, UINT size, LPOUTLINETEXTMETRICW otm )
 {
     return 0;
 }
 
-static UINT nulldrv_GetTextCharsetInfo( PHYSDEV dev, LPFONTSIGNATURE fs, DWORD flags )
+static UINT CDECL nulldrv_GetTextCharsetInfo( PHYSDEV dev, LPFONTSIGNATURE fs, DWORD flags )
 {
     return DEFAULT_CHARSET;
 }
 
-static BOOL nulldrv_GetTextExtentExPoint( PHYSDEV dev, LPCWSTR str, INT count, INT *dx )
+static BOOL CDECL nulldrv_GetTextExtentExPoint( PHYSDEV dev, LPCWSTR str, INT count, INT *dx )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetTextExtentExPointI( PHYSDEV dev, const WORD *indices, INT count, INT *dx )
+static BOOL CDECL nulldrv_GetTextExtentExPointI( PHYSDEV dev, const WORD *indices, INT count, INT *dx )
 {
     return FALSE;
 }
 
-static INT nulldrv_GetTextFace( PHYSDEV dev, INT size, LPWSTR name )
+static INT CDECL nulldrv_GetTextFace( PHYSDEV dev, INT size, LPWSTR name )
 {
     INT ret = 0;
     LOGFONTW font;
@@ -484,55 +535,55 @@ static INT nulldrv_GetTextFace( PHYSDEV dev, INT size, LPWSTR name )
     return ret;
 }
 
-static BOOL nulldrv_GetTextMetrics( PHYSDEV dev, TEXTMETRICW *metrics )
+static BOOL CDECL nulldrv_GetTextMetrics( PHYSDEV dev, TEXTMETRICW *metrics )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_LineTo( PHYSDEV dev, INT x, INT y )
+static BOOL CDECL nulldrv_LineTo( PHYSDEV dev, INT x, INT y )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_MoveTo( PHYSDEV dev, INT x, INT y )
+static BOOL CDECL nulldrv_MoveTo( PHYSDEV dev, INT x, INT y )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PaintRgn( PHYSDEV dev, HRGN rgn )
+static BOOL CDECL nulldrv_PaintRgn( PHYSDEV dev, HRGN rgn )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop )
+static BOOL CDECL nulldrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                         INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                               INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PolyPolygon( PHYSDEV dev, const POINT *points, const INT *counts, UINT polygons )
+static BOOL CDECL nulldrv_PolyPolygon( PHYSDEV dev, const POINT *points, const INT *counts, UINT polygons )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PolyPolyline( PHYSDEV dev, const POINT *points, const DWORD *counts, DWORD lines )
+static BOOL CDECL nulldrv_PolyPolyline( PHYSDEV dev, const POINT *points, const DWORD *counts, DWORD lines )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Polygon( PHYSDEV dev, const POINT *points, INT count )
+static BOOL CDECL nulldrv_Polygon( PHYSDEV dev, const POINT *points, INT count )
 {
     INT counts[1] = { count };
 
     return PolyPolygon( dev->hdc, points, counts, 1 );
 }
 
-static BOOL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
+static BOOL CDECL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
 {
     DWORD counts[1] = { count };
 
@@ -540,175 +591,175 @@ static BOOL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
     return PolyPolyline( dev->hdc, points, counts, 1 );
 }
 
-static DWORD nulldrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
-                               const struct gdi_image_bits *bits, struct bitblt_coords *src,
-                               struct bitblt_coords *dst, DWORD rop )
+static DWORD CDECL nulldrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
+                                     const struct gdi_image_bits *bits, struct bitblt_coords *src,
+                                     struct bitblt_coords *dst, DWORD rop )
 {
     return ERROR_SUCCESS;
 }
 
-static UINT nulldrv_RealizeDefaultPalette( PHYSDEV dev )
+static UINT CDECL nulldrv_RealizeDefaultPalette( PHYSDEV dev )
 {
     return 0;
 }
 
-static UINT nulldrv_RealizePalette( PHYSDEV dev, HPALETTE palette, BOOL primary )
+static UINT CDECL nulldrv_RealizePalette( PHYSDEV dev, HPALETTE palette, BOOL primary )
 {
     return 0;
 }
 
-static BOOL nulldrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
+static BOOL CDECL nulldrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
 {
     return TRUE;
 }
 
-static HDC nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
+static HDC CDECL nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
 {
     return 0;
 }
 
-static BOOL nulldrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                               INT ell_width, INT ell_height )
+static BOOL CDECL nulldrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                                     INT ell_width, INT ell_height )
 {
     return TRUE;
 }
 
-static HBITMAP nulldrv_SelectBitmap( PHYSDEV dev, HBITMAP bitmap )
+static HBITMAP CDECL nulldrv_SelectBitmap( PHYSDEV dev, HBITMAP bitmap )
 {
     return bitmap;
 }
 
-static HBRUSH nulldrv_SelectBrush( PHYSDEV dev, HBRUSH brush, const struct brush_pattern *pattern )
+static HBRUSH CDECL nulldrv_SelectBrush( PHYSDEV dev, HBRUSH brush, const struct brush_pattern *pattern )
 {
     return brush;
 }
 
-static HPALETTE nulldrv_SelectPalette( PHYSDEV dev, HPALETTE palette, BOOL bkgnd )
+static HPALETTE CDECL nulldrv_SelectPalette( PHYSDEV dev, HPALETTE palette, BOOL bkgnd )
 {
     return palette;
 }
 
-static HPEN nulldrv_SelectPen( PHYSDEV dev, HPEN pen, const struct brush_pattern *pattern )
+static HPEN CDECL nulldrv_SelectPen( PHYSDEV dev, HPEN pen, const struct brush_pattern *pattern )
 {
     return pen;
 }
 
-static INT nulldrv_SetArcDirection( PHYSDEV dev, INT dir )
+static INT CDECL nulldrv_SetArcDirection( PHYSDEV dev, INT dir )
 {
     return dir;
 }
 
-static COLORREF nulldrv_SetBkColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetBkColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static INT nulldrv_SetBkMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetBkMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static UINT nulldrv_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+static UINT CDECL nulldrv_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
 {
     return DCB_RESET;
 }
 
-static COLORREF nulldrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static COLORREF nulldrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static void nulldrv_SetDeviceClipping( PHYSDEV dev, HRGN rgn )
+static void CDECL nulldrv_SetDeviceClipping( PHYSDEV dev, HRGN rgn )
 {
 }
 
-static DWORD nulldrv_SetLayout( PHYSDEV dev, DWORD layout )
+static DWORD CDECL nulldrv_SetLayout( PHYSDEV dev, DWORD layout )
 {
     return layout;
 }
 
-static BOOL nulldrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
+static BOOL CDECL nulldrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
 {
     SetLastError( ERROR_INVALID_PARAMETER );
     return FALSE;
 }
 
-static DWORD nulldrv_SetMapperFlags( PHYSDEV dev, DWORD flags )
+static DWORD CDECL nulldrv_SetMapperFlags( PHYSDEV dev, DWORD flags )
 {
     return flags;
 }
 
-static COLORREF nulldrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
+static COLORREF CDECL nulldrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
 {
     return color;
 }
 
-static INT nulldrv_SetPolyFillMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetPolyFillMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static INT nulldrv_SetROP2( PHYSDEV dev, INT rop )
+static INT CDECL nulldrv_SetROP2( PHYSDEV dev, INT rop )
 {
     return rop;
 }
 
-static INT nulldrv_SetRelAbs( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetRelAbs( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static INT nulldrv_SetStretchBltMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetStretchBltMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static UINT nulldrv_SetTextAlign( PHYSDEV dev, UINT align )
+static UINT CDECL nulldrv_SetTextAlign( PHYSDEV dev, UINT align )
 {
     return align;
 }
 
-static INT nulldrv_SetTextCharacterExtra( PHYSDEV dev, INT extra )
+static INT CDECL nulldrv_SetTextCharacterExtra( PHYSDEV dev, INT extra )
 {
     return extra;
 }
 
-static COLORREF nulldrv_SetTextColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetTextColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static BOOL nulldrv_SetTextJustification( PHYSDEV dev, INT extra, INT breaks )
+static BOOL CDECL nulldrv_SetTextJustification( PHYSDEV dev, INT extra, INT breaks )
 {
     return TRUE;
 }
 
-static INT nulldrv_StartDoc( PHYSDEV dev, const DOCINFOW *info )
+static INT CDECL nulldrv_StartDoc( PHYSDEV dev, const DOCINFOW *info )
 {
     return 0;
 }
 
-static INT nulldrv_StartPage( PHYSDEV dev )
+static INT CDECL nulldrv_StartPage( PHYSDEV dev )
 {
     return 1;
 }
 
-static BOOL nulldrv_UnrealizePalette( HPALETTE palette )
+static BOOL CDECL nulldrv_UnrealizePalette( HPALETTE palette )
 {
     return FALSE;
 }
 
-static struct opengl_funcs *nulldrv_wine_get_wgl_driver( PHYSDEV dev, UINT version )
+static struct opengl_funcs * CDECL nulldrv_wine_get_wgl_driver( PHYSDEV dev, UINT version )
 {
     return (void *)-1;
 }
 
-static const struct vulkan_funcs *nulldrv_wine_get_vulkan_driver( PHYSDEV dev, UINT version )
+static const struct vulkan_funcs * CDECL nulldrv_wine_get_vulkan_driver( PHYSDEV dev, UINT version )
 {
     return NULL;
 }
@@ -752,6 +803,7 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_GetCharABCWidths,           /* pGetCharABCWidths */
     nulldrv_GetCharABCWidthsI,          /* pGetCharABCWidthsI */
     nulldrv_GetCharWidth,               /* pGetCharWidth */
+    nulldrv_GetCharWidthInfo,           /* pGetCharWidthInfo */
     nulldrv_GetDeviceCaps,              /* pGetDeviceCaps */
     nulldrv_GetDeviceGammaRamp,         /* pGetDeviceGammaRamp */
     nulldrv_GetFontData,                /* pGetFontData */
@@ -856,13 +908,12 @@ BOOL DRIVER_GetDriverName( LPCWSTR device, LPWSTR driver, DWORD size )
 {
     static const WCHAR displayW[] = { 'd','i','s','p','l','a','y',0 };
     static const WCHAR devicesW[] = { 'd','e','v','i','c','e','s',0 };
-    static const WCHAR display1W[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
     static const WCHAR empty_strW[] = { 0 };
     WCHAR *p;
 
     /* display is a special case */
     if (!strcmpiW( device, displayW ) ||
-        !strcmpiW( device, display1W ))
+        is_display_device( device ))
     {
         lstrcpynW( driver, displayW, size );
         return TRUE;
@@ -1253,3 +1304,146 @@ NTSTATUS WINAPI D3DKMTEscape( const void *pData )
     FIXME("(%p): stub\n", pData);
     return STATUS_NO_MEMORY;
 }
+
+/******************************************************************************
+ *		D3DKMTCloseAdapter [GDI32.@]
+ */
+NTSTATUS WINAPI D3DKMTCloseAdapter( const D3DKMT_CLOSEADAPTER *desc )
+{
+    NTSTATUS status = STATUS_INVALID_PARAMETER;
+    struct d3dkmt_adapter *adapter;
+
+    TRACE("(%p)\n", desc);
+
+    if (!desc || !desc->hAdapter)
+        return STATUS_INVALID_PARAMETER;
+
+    EnterCriticalSection( &driver_section );
+    LIST_FOR_EACH_ENTRY( adapter, &d3dkmt_adapters, struct d3dkmt_adapter, entry )
+    {
+        if (adapter->handle == desc->hAdapter)
+        {
+            list_remove( &adapter->entry );
+            heap_free( adapter );
+            status = STATUS_SUCCESS;
+            break;
+        }
+    }
+    LeaveCriticalSection( &driver_section );
+
+    return status;
+}
+
+/******************************************************************************
+ *		D3DKMTOpenAdapterFromGdiDisplayName [GDI32.@]
+ */
+NTSTATUS WINAPI D3DKMTOpenAdapterFromGdiDisplayName( D3DKMT_OPENADAPTERFROMGDIDISPLAYNAME *desc )
+{
+    static const WCHAR display1W[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
+    static D3DKMT_HANDLE handle_start = 0;
+    struct d3dkmt_adapter *adapter;
+
+    TRACE("(%p) semi-stub\n", desc);
+
+    if (!desc)
+        return STATUS_UNSUCCESSFUL;
+
+    /* FIXME: Support multiple monitors */
+    if (lstrcmpiW( desc->DeviceName, display1W ))
+    {
+        FIXME("%s is unsupported\n", wine_dbgstr_w( desc->DeviceName ));
+        return STATUS_UNSUCCESSFUL;
+    }
+
+    adapter = heap_alloc( sizeof( *adapter ) );
+    if (!adapter)
+        return STATUS_NO_MEMORY;
+
+    EnterCriticalSection( &driver_section );
+    /* D3DKMT_HANDLE is UINT, so we can't use pointer as handle */
+    adapter->handle = ++handle_start;
+    adapter->ordinal = 0;
+    list_add_tail( &d3dkmt_adapters, &adapter->entry );
+    LeaveCriticalSection( &driver_section );
+
+    desc->hAdapter = handle_start;
+    /* FIXME: Support AdapterLuid */
+    desc->AdapterLuid.LowPart = 0;
+    desc->AdapterLuid.HighPart = 0;
+    desc->VidPnSourceId = 0;
+    return STATUS_SUCCESS;
+}
+
+/******************************************************************************
+ *		D3DKMTCreateDevice [GDI32.@]
+ */
+NTSTATUS WINAPI D3DKMTCreateDevice( D3DKMT_CREATEDEVICE *desc )
+{
+    static D3DKMT_HANDLE handle_start = 0;
+    struct d3dkmt_adapter *adapter;
+    struct d3dkmt_device *device;
+    BOOL found = FALSE;
+
+    TRACE("(%p)\n", desc);
+
+    if (!desc)
+        return STATUS_INVALID_PARAMETER;
+
+    EnterCriticalSection( &driver_section );
+    LIST_FOR_EACH_ENTRY( adapter, &d3dkmt_adapters, struct d3dkmt_adapter, entry )
+    {
+        if (adapter->handle == desc->hAdapter)
+        {
+            found = TRUE;
+            break;
+        }
+    }
+    LeaveCriticalSection( &driver_section );
+
+    if (!found)
+        return STATUS_INVALID_PARAMETER;
+
+    if (desc->Flags.LegacyMode || desc->Flags.RequestVSync || desc->Flags.DisableGpuTimeout)
+        FIXME("Flags unsupported.\n");
+
+    device = heap_alloc_zero( sizeof( *device ) );
+    if (!device)
+        return STATUS_NO_MEMORY;
+
+    EnterCriticalSection( &driver_section );
+    device->handle = ++handle_start;
+    list_add_tail( &d3dkmt_devices, &device->entry );
+    LeaveCriticalSection( &driver_section );
+
+    desc->hDevice = device->handle;
+    return STATUS_SUCCESS;
+}
+
+/******************************************************************************
+ *		D3DKMTDestroyDevice [GDI32.@]
+ */
+NTSTATUS WINAPI D3DKMTDestroyDevice( const D3DKMT_DESTROYDEVICE *desc )
+{
+    NTSTATUS status = STATUS_INVALID_PARAMETER;
+    struct d3dkmt_device *device;
+
+    TRACE("(%p)\n", desc);
+
+    if (!desc || !desc->hDevice)
+        return STATUS_INVALID_PARAMETER;
+
+    EnterCriticalSection( &driver_section );
+    LIST_FOR_EACH_ENTRY( device, &d3dkmt_devices, struct d3dkmt_device, entry )
+    {
+        if (device->handle == desc->hDevice)
+        {
+            list_remove( &device->entry );
+            heap_free( device );
+            status = STATUS_SUCCESS;
+            break;
+        }
+    }
+    LeaveCriticalSection( &driver_section );
+
+    return status;
+}
