@@ -69,7 +69,7 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     static const WCHAR regexpW[] = {'R','e','g','E','x','p',0};
     static const WCHAR stringW[] = {'S','t','r','i','n','g',0};
     /* Keep in sync with jsclass_t enum */
-    static const WCHAR *names[] = {NULL, arrayW, booleanW, dateW, errorW,
+    static const WCHAR *names[] = {NULL, arrayW, booleanW, dateW, objectW, errorW,
         functionW, NULL, mathW, numberW, objectW, regexpW, stringW, objectW, objectW, objectW};
 
     TRACE("\n");
@@ -89,11 +89,11 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
         jsstr_t *ret;
         WCHAR *ptr;
 
-        ret = jsstr_alloc_buf(9+strlenW(str), &ptr);
+        ret = jsstr_alloc_buf(9+lstrlenW(str), &ptr);
         if(!ret)
             return E_OUTOFMEMORY;
 
-        sprintfW(ptr, formatW, str);
+        swprintf(ptr, 9 + lstrlenW(str), formatW, str);
         *r = jsval_string(ret);
     }
 
