@@ -72,7 +72,6 @@ static const struct alloccanonicalize_test alloccanonicalize_tests[] =
     {"\\\\?C:a", "\\\\?C:a", 0, S_OK},
 
     /* No . */
-    {"*", "*", 0, S_OK},
     {"", "\\", 0, S_OK},
     {"C:", "C:", 0, S_OK},
     {"C:\\", "C:\\", 0, S_OK},
@@ -84,41 +83,19 @@ static const struct alloccanonicalize_test alloccanonicalize_tests[] =
     {"..", "\\", 0, S_OK},
     {"...", "\\", 0, S_OK},
     {"*.", "*.", 0, S_OK},
-    {"*.\\", "*.\\", 0, S_OK},
-    {"*.\\", "*.\\", 0, S_OK},
     {"*..", "*.", 0, S_OK},
-    {"*..\\", "*..\\", 0, S_OK},
     {"*...", "*.", 0, S_OK},
-    {"*...\\", "*...\\", 0, S_OK},
-    {"*....", "*.", 0, S_OK},
-    {"*....\\", "*....\\", 0, S_OK},
-    {".a", ".a", 0, S_OK},
-    {".a\\", ".a\\", 0, S_OK},
     {"a.", "a", 0, S_OK},
-    {"a.\\", "a.\\", 0, S_OK},
-    {".a.", ".a", 0, S_OK},
     {"a.b", "a.b", 0, S_OK},
-    {".a.b.", ".a.b", 0, S_OK},
     {"a\\.", "a", 0, S_OK},
     {"a\\.\\b", "a\\b", 0, S_OK},
-    {"a\\.b", "a\\.b", 0, S_OK},
-    {"a\\.b\\", "a\\.b\\", 0, S_OK},
-    {":.", ":", 0, S_OK},
-    {"::.", "::\\", 0, S_OK},
-    {":::.", ":::", 0, S_OK},
     {"C:.", "C:\\", 0, S_OK},
-    {"C:.\\", "C:.\\", 0, S_OK},
-    {"C:.\\.", "C:\\", 0, S_OK},
     {"C:\\.", "C:\\", 0, S_OK},
     {"C:\\.\\", "C:\\", 0, S_OK},
     {"C:\\a.", "C:\\a", 0, S_OK},
-    {"C:\\a.\\", "C:\\a.\\", 0, S_OK},
-    {"C:\\.a", "C:\\.a", 0, S_OK},
-    {"C:\\.a\\", "C:\\.a\\", 0, S_OK},
     {"C:\\a\\.", "C:\\a", 0, S_OK},
     {"C:\\a\\\\.", "C:\\a\\", 0, S_OK},
     {"C:\\a\\\\\\.", "C:\\a\\\\", 0, S_OK},
-    {".\\", "\\", 0, S_OK},
     {"\\.", "\\", 0, S_OK},
     {"\\\\.", "\\\\", 0, S_OK},
     {"\\\\.\\", "\\\\", 0, S_OK},
@@ -138,53 +115,21 @@ static const struct alloccanonicalize_test alloccanonicalize_tests[] =
       "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}\\", 0, S_OK},
 
     /* .. */
-    {"..a", "..a", 0, S_OK},
-    {"..a\\", "..a\\", 0, S_OK},
-    {"...a", "...a", 0, S_OK},
-    {"...a\\", "...a\\", 0, S_OK},
-    {"....a", "....a", 0, S_OK},
     {"a..", "a", 0, S_OK},
-    {"a..\\", "a..\\", 0, S_OK},
-    {"a...", "a", 0, S_OK},
-    {"a...\\", "a...\\", 0, S_OK},
-    {"a....", "a", 0, S_OK},
-    {"a....\\", "a....\\", 0, S_OK},
-    {"..a..", "..a", 0, S_OK},
     {"a..b", "a..b", 0, S_OK},
-    {"..a..b..", "..a..b", 0, S_OK},
     {"a\\..", "\\", 0, S_OK},
     {"a\\..\\", "\\", 0, S_OK},
     {"a\\..\\b", "\\b", 0, S_OK},
-    {":..", ":", 0, S_OK},
-    {"::..", "::\\", 0, S_OK},
-    {":::..", ":::", 0, S_OK},
     {"C:..", "C:\\", 0, S_OK},
-    {"C:...", "C:\\", 0, S_OK},
-    {"C:..\\", "C:..\\", 0, S_OK},
-    {"C:..\\\\", "C:..\\\\", 0, S_OK},
-    {"C:...\\", "C:...\\", 0, S_OK},
     {"C:\\..", "C:\\", 0, S_OK},
-    {"C:\\..a", "C:\\..a", 0, S_OK},
-    {"C:\\..a\\", "C:\\..a\\", 0, S_OK},
-    {"C:\\...a", "C:\\...a", 0, S_OK},
-    {"C:\\...a\\", "C:\\...a\\", 0, S_OK},
-    {"C:\\....a", "C:\\....a", 0, S_OK},
-    {"C:\\....a\\", "C:\\....a\\", 0, S_OK},
-    {"C:\\a..", "C:\\a", 0, S_OK},
-    {"C:\\a..\\", "C:\\a..\\", 0, S_OK},
     {"C:\\\\..", "C:\\", 0, S_OK},
     {"C:\\..\\", "C:\\", 0, S_OK},
-    {"C:\\...\\", "C:\\...\\", 0, S_OK},
     {"C:\\a\\..", "C:\\", 0, S_OK},
-    {"C:\\a\\b..", "C:\\a\\b", 0, S_OK},
     {"C:\\a\\\\..", "C:\\a", 0, S_OK},
     {"C:\\a\\\\\\..", "C:\\a\\", 0, S_OK},
     {"C:\\a\\..\\b", "C:\\b", 0, S_OK},
     {"C:\\a\\..\\\\b", "C:\\\\b", 0, S_OK},
-    {"..\\", "\\", 0, S_OK},
-    {"...\\", "...\\", 0, S_OK},
     {"\\..", "\\", 0, S_OK},
-    {"\\...", "\\", 0, S_OK},
     {"\\\\..", "\\\\", 0, S_OK},
     {"\\\\\\..", "\\", 0, S_OK},
     {"\\\\..\\", "\\\\", 0, S_OK},
@@ -304,8 +249,6 @@ static const struct alloccanonicalize_test alloccanonicalize_tests[] =
 
     /* PATHCCH_DO_NOT_NORMALIZE_SEGMENTS */
     /* No effect for spaces */
-    {"a ", "a ", 0, S_OK},
-    {"C:\\a ", "C:\\a ", 0, S_OK},
     {"C:\\a \\", "C:\\a \\", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:\\a\\ ", "C:\\a\\ ", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:\\a ", "C:\\a ", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
@@ -320,15 +263,8 @@ static const struct alloccanonicalize_test alloccanonicalize_tests[] =
     {"*..", "*..", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {".", "\\", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"..", "\\", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {":.", ":.", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {"::.", "::.", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {":::.", ":::.", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {":..", ":..", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {"::..", "::..", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {":::..", ":::..", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:.", "C:.", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:..", "C:..", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
-    {"C:...", "C:...", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:\\a\\.", "C:\\a", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:\\a\\..", "C:\\", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
     {"C:\\a.", "C:\\a.", PATHCCH_DO_NOT_NORMALIZE_SEGMENTS, S_OK},
@@ -1099,6 +1035,7 @@ static void test_PathCchCanonicalize(void)
         ok(hr == E_INVALIDARG, "expect hr %#x, got %#x\n", E_INVALIDARG, hr);
     }
 
+    path_inW[0] = 0;
     hr = pPathCchCanonicalize(path_outW, 0, path_inW);
     ok(hr == E_INVALIDARG, "expect hr %#x, got %#x\n", E_INVALIDARG, hr);
 
