@@ -192,7 +192,7 @@ static struct named_item *host_get_named_item(ScriptHost *host, const WCHAR *nam
     struct named_item *item;
 
     LIST_FOR_EACH_ENTRY(item, &host->named_items, struct named_item, entry) {
-        if (!lstrcmpW(item->name, nameW))
+        if (!wcscmp(item->name, nameW))
             return item;
     }
 
@@ -792,8 +792,10 @@ static HRESULT WINAPI ScriptControl_put_State(IScriptControl *iface, ScriptContr
 static HRESULT WINAPI ScriptControl_put_SitehWnd(IScriptControl *iface, LONG hwnd)
 {
     ScriptControl *This = impl_from_IScriptControl(iface);
+
     FIXME("(%p)->(%x)\n", This, hwnd);
-    return E_NOTIMPL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptControl_get_SitehWnd(IScriptControl *iface, LONG *p)
