@@ -352,15 +352,18 @@ static void* WINAPI call_catch_block(EXCEPTION_RECORD *rec)
     const cxx_function_descr *descr = (void*)rec->ExceptionInformation[2];
     EXCEPTION_RECORD *prev_rec = (void*)rec->ExceptionInformation[4];
     EXCEPTION_RECORD *untrans_rec = (void*)rec->ExceptionInformation[6];
+    CONTEXT *context = (void*)rec->ExceptionInformation[7];
     void* (__cdecl *handler)(ULONG64 unk, ULONG64 rbp) = (void*)rec->ExceptionInformation[5];
     int *unwind_help = rva_to_ptr(descr->unwind_help, frame);
+    EXCEPTION_POINTERS ep = { prev_rec, context };
     cxx_catch_ctx ctx;
     void *ret_addr = NULL;
 
     TRACE("calling handler %p\n", handler);
 
     ctx.rethrow = FALSE;
-    __CxxRegisterExceptionObject(&prev_rec, &ctx.frame_info);
+    __CxxRegisterExceptionObject(&ep, &ctx.frame_info);
+    msvcrt_get_thread_data()->processing_throw--;
     __TRY
     {
         __TRY
@@ -395,11 +398,12 @@ static void* WINAPI call_catch_block(EXCEPTION_RECORD *rec)
 
 static inline BOOL cxx_is_consolidate(const EXCEPTION_RECORD *rec)
 {
-    return rec->ExceptionCode==STATUS_UNWIND_CONSOLIDATE && rec->NumberParameters==7 &&
+    return rec->ExceptionCode==STATUS_UNWIND_CONSOLIDATE && rec->NumberParameters==8 &&
            rec->ExceptionInformation[0]==(ULONG_PTR)call_catch_block;
 }
 
-static inline void find_catch_block(EXCEPTION_RECORD *rec, EXCEPTION_RECORD *untrans_rec,
+static inline void find_catch_block(EXCEPTION_RECORD *rec, CONTEXT *context,
+                                    EXCEPTION_RECORD *untrans_rec,
                                     ULONG64 frame, DISPATCHER_CONTEXT *dispatch,
                                     const cxx_function_descr *descr,
                                     cxx_exception_type *info, ULONG64 orig_frame)
@@ -407,12 +411,14 @@ static inline void find_catch_block(EXCEPTION_RECORD *rec, EXCEPTION_RECORD *unt
     ULONG64 exc_base = (rec->NumberParameters == 4 ? rec->ExceptionInformation[3] : 0);
     int trylevel = ip_to_state(rva_to_ptr(descr->ipmap, dispatch->ImageBase),
             descr->ipmap_count, dispatch->ControlPc-dispatch->ImageBase);
+    thread_data_t *data = msvcrt_get_thread_data();
     const tryblock_info *in_catch;
     EXCEPTION_RECORD catch_record;
-    CONTEXT context;
+    CONTEXT ctx;
     UINT i, j;
     INT *unwind_help;
 
+    data->processing_throw++;
     for (i=descr->tryblock_count; i>0; i--)
     {
         in_catch = rva_to_ptr(descr->tryblock, dispatch->ImageBase);
@@ -476,7 +482,7 @@ static inline void find_catch_block(EXCEPTION_RECORD *rec, EXCEPTION_RECORD *unt
             memset(&catch_record, 0, sizeof(catch_record));
             catch_record.ExceptionCode = STATUS_UNWIND_CONSOLIDATE;
             catch_record.ExceptionFlags = EXCEPTION_NONCONTINUABLE;
-            catch_record.NumberParameters = 7;
+            catch_record.NumberParameters = 8;
             catch_record.ExceptionInformation[0] = (ULONG_PTR)call_catch_block;
             catch_record.ExceptionInformation[1] = orig_frame;
             catch_record.ExceptionInformation[2] = (ULONG_PTR)descr;
@@ -485,11 +491,13 @@ static inline void find_catch_block(EXCEPTION_RECORD *rec, EXCEPTION_RECORD *unt
             catch_record.ExceptionInformation[5] =
                 (ULONG_PTR)rva_to_ptr(catchblock->handler, dispatch->ImageBase);
             catch_record.ExceptionInformation[6] = (ULONG_PTR)untrans_rec;
-            RtlUnwindEx((void*)frame, (void*)dispatch->ControlPc, &catch_record, NULL, &context, NULL);
+            catch_record.ExceptionInformation[7] = (ULONG_PTR)context;
+            RtlUnwindEx((void*)frame, (void*)dispatch->ControlPc, &catch_record, NULL, &ctx, NULL);
         }
     }
 
     TRACE("no matching catch block found\n");
+    data->processing_throw--;
 }
 
 static LONG CALLBACK se_translation_filter(EXCEPTION_POINTERS *ep, void *c)
@@ -505,7 +513,7 @@ static LONG CALLBACK se_translation_filter(EXCEPTION_POINTERS *ep, void *c)
     }
 
     exc_type = (cxx_exception_type *)rec->ExceptionInformation[2];
-    find_catch_block(rec, ctx->seh_rec, ctx->dest_frame, ctx->dispatch,
+    find_catch_block(rec, ep->ContextRecord, ctx->seh_rec, ctx->dest_frame, ctx->dispatch,
                      ctx->descr, exc_type, ctx->orig_frame);
 
     __DestructExceptionObject(rec);
@@ -618,7 +626,7 @@ static DWORD cxx_frame_handler(EXCEPTION_RECORD *rec, ULONG64 frame,
         }
     }
 
-    find_catch_block(rec, NULL, frame, dispatch, descr, exc_type, orig_frame);
+    find_catch_block(rec, context, NULL, frame, dispatch, descr, exc_type, orig_frame);
     return ExceptionContinueSearch;
 }
 
@@ -692,63 +700,7 @@ unsigned int CDECL __CxxQueryExceptionSize(void)
  *		_setjmp (MSVCRT.@)
  */
 __ASM_GLOBAL_FUNC( MSVCRT__setjmp,
-                   "jmp " __ASM_NAME("MSVCRT__setjmpex") );
-
-/*******************************************************************
- *		_setjmpex (MSVCRT.@)
- */
-__ASM_GLOBAL_FUNC( MSVCRT__setjmpex,
-                   "movq %rdx,(%rcx)\n\t"          /* jmp_buf->Frame */
-                   "movq %rbx,0x8(%rcx)\n\t"       /* jmp_buf->Rbx */
-                   "leaq 0x8(%rsp),%rax\n\t"
-                   "movq %rax,0x10(%rcx)\n\t"      /* jmp_buf->Rsp */
-                   "movq %rbp,0x18(%rcx)\n\t"      /* jmp_buf->Rbp */
-                   "movq %rsi,0x20(%rcx)\n\t"      /* jmp_buf->Rsi */
-                   "movq %rdi,0x28(%rcx)\n\t"      /* jmp_buf->Rdi */
-                   "movq %r12,0x30(%rcx)\n\t"      /* jmp_buf->R12 */
-                   "movq %r13,0x38(%rcx)\n\t"      /* jmp_buf->R13 */
-                   "movq %r14,0x40(%rcx)\n\t"      /* jmp_buf->R14 */
-                   "movq %r15,0x48(%rcx)\n\t"      /* jmp_buf->R15 */
-                   "movq (%rsp),%rax\n\t"
-                   "movq %rax,0x50(%rcx)\n\t"      /* jmp_buf->Rip */
-                   "movdqa %xmm6,0x60(%rcx)\n\t"   /* jmp_buf->Xmm6 */
-                   "movdqa %xmm7,0x70(%rcx)\n\t"   /* jmp_buf->Xmm7 */
-                   "movdqa %xmm8,0x80(%rcx)\n\t"   /* jmp_buf->Xmm8 */
-                   "movdqa %xmm9,0x90(%rcx)\n\t"   /* jmp_buf->Xmm9 */
-                   "movdqa %xmm10,0xa0(%rcx)\n\t"  /* jmp_buf->Xmm10 */
-                   "movdqa %xmm11,0xb0(%rcx)\n\t"  /* jmp_buf->Xmm11 */
-                   "movdqa %xmm12,0xc0(%rcx)\n\t"  /* jmp_buf->Xmm12 */
-                   "movdqa %xmm13,0xd0(%rcx)\n\t"  /* jmp_buf->Xmm13 */
-                   "movdqa %xmm14,0xe0(%rcx)\n\t"  /* jmp_buf->Xmm14 */
-                   "movdqa %xmm15,0xf0(%rcx)\n\t"  /* jmp_buf->Xmm15 */
-                   "xorq %rax,%rax\n\t"
-                   "retq" );
-
-
-extern void DECLSPEC_NORETURN CDECL longjmp_set_regs( struct MSVCRT___JUMP_BUFFER *jmp, int retval );
-__ASM_GLOBAL_FUNC( longjmp_set_regs,
-                   "movq %rdx,%rax\n\t"            /* retval */
-                   "movq 0x8(%rcx),%rbx\n\t"       /* jmp_buf->Rbx */
-                   "movq 0x18(%rcx),%rbp\n\t"      /* jmp_buf->Rbp */
-                   "movq 0x20(%rcx),%rsi\n\t"      /* jmp_buf->Rsi */
-                   "movq 0x28(%rcx),%rdi\n\t"      /* jmp_buf->Rdi */
-                   "movq 0x30(%rcx),%r12\n\t"      /* jmp_buf->R12 */
-                   "movq 0x38(%rcx),%r13\n\t"      /* jmp_buf->R13 */
-                   "movq 0x40(%rcx),%r14\n\t"      /* jmp_buf->R14 */
-                   "movq 0x48(%rcx),%r15\n\t"      /* jmp_buf->R15 */
-                   "movdqa 0x60(%rcx),%xmm6\n\t"   /* jmp_buf->Xmm6 */
-                   "movdqa 0x70(%rcx),%xmm7\n\t"   /* jmp_buf->Xmm7 */
-                   "movdqa 0x80(%rcx),%xmm8\n\t"   /* jmp_buf->Xmm8 */
-                   "movdqa 0x90(%rcx),%xmm9\n\t"   /* jmp_buf->Xmm9 */
-                   "movdqa 0xa0(%rcx),%xmm10\n\t"  /* jmp_buf->Xmm10 */
-                   "movdqa 0xb0(%rcx),%xmm11\n\t"  /* jmp_buf->Xmm11 */
-                   "movdqa 0xc0(%rcx),%xmm12\n\t"  /* jmp_buf->Xmm12 */
-                   "movdqa 0xd0(%rcx),%xmm13\n\t"  /* jmp_buf->Xmm13 */
-                   "movdqa 0xe0(%rcx),%xmm14\n\t"  /* jmp_buf->Xmm14 */
-                   "movdqa 0xf0(%rcx),%xmm15\n\t"  /* jmp_buf->Xmm15 */
-                   "movq 0x50(%rcx),%rdx\n\t"      /* jmp_buf->Rip */
-                   "movq 0x10(%rcx),%rsp\n\t"      /* jmp_buf->Rsp */
-                   "jmp *%rdx" );
+                   "jmp " __ASM_NAME("__wine_setjmpex") );
 
 /*******************************************************************
  *		longjmp (MSVCRT.@)
@@ -768,7 +720,7 @@ void __cdecl MSVCRT_longjmp( struct MSVCRT___JUMP_BUFFER *jmp, int retval )
         rec.ExceptionInformation[0] = (DWORD_PTR)jmp;
         RtlUnwind( (void *)jmp->Frame, (void *)jmp->Rip, &rec, IntToPtr(retval) );
     }
-    longjmp_set_regs( jmp, retval );
+    __wine_longjmp( (__wine_jmp_buf *)jmp, retval );
 }
 
 /*******************************************************************
@@ -790,4 +742,42 @@ int __cdecl _fpieee_flt(ULONG exception_code, EXCEPTION_POINTERS *ep,
     return EXCEPTION_CONTINUE_SEARCH;
 }
 
+#if _MSVCR_VER>=110 && _MSVCR_VER<=120
+/*********************************************************************
+ *  __crtCapturePreviousContext (MSVCR110.@)
+ */
+void __cdecl get_prev_context(CONTEXT *ctx, DWORD64 rip)
+{
+    ULONG64 frame, image_base;
+    RUNTIME_FUNCTION *rf;
+    void *data;
+
+    TRACE("(%p)\n", ctx);
+
+    ctx->Rip = rip;
+    ctx->Rsp += 3*8; /* Rip, Rcx, return address */
+
+    rf = RtlLookupFunctionEntry(ctx->Rip, &image_base, NULL);
+    if(!rf) {
+        FIXME("RtlLookupFunctionEntry failed\n");
+        return;
+    }
+
+    RtlVirtualUnwind(UNW_FLAG_NHANDLER, image_base, ctx->Rip,
+            rf, ctx, &data, &frame, NULL);
+}
+
+__ASM_GLOBAL_FUNC( __crtCapturePreviousContext,
+        "pushq (%rsp)\n\t" /* save Rip */
+        __ASM_CFI(".cfi_adjust_cfa_offset 8\n\t")
+        "pushq %rcx\n\t"
+        __ASM_CFI(".cfi_adjust_cfa_offset 8\n\t")
+        "call " __ASM_NAME("RtlCaptureContext") "\n\t"
+        "popq %rcx\n\t"
+        __ASM_CFI(".cfi_adjust_cfa_offset -8\n\t")
+        "popq %rdx\n\t"
+        __ASM_CFI(".cfi_adjust_cfa_offset -8\n\t")
+        "jmp " __ASM_NAME("get_prev_context") );
+#endif
+
 #endif  /* __x86_64__ */
