@@ -67,8 +67,6 @@ struct startup_info
 
 static PEB *peb;
 static PEB_LDR_DATA ldr;
-static RTL_USER_PROCESS_PARAMETERS params;  /* default parameters if no parent */
-static WCHAR current_dir[MAX_PATH];
 static RTL_BITMAP tls_bitmap;
 static RTL_BITMAP tls_expansion_bitmap;
 static RTL_BITMAP fls_bitmap;
@@ -83,121 +81,6 @@ static RTL_CRITICAL_SECTION_DEBUG critsect_debug =
 };
 static RTL_CRITICAL_SECTION peb_lock = { &critsect_debug, -1, 0, 0, 0, 0 };
 
-/***********************************************************************
- *           get_unicode_string
- *
- * Copy a unicode string from the startup info.
- */
-static inline void get_unicode_string( UNICODE_STRING *str, WCHAR **src, WCHAR **dst, UINT len )
-{
-    str->Buffer = *dst;
-    str->Length = len;
-    str->MaximumLength = len + sizeof(WCHAR);
-    memcpy( str->Buffer, *src, len );
-    str->Buffer[len / sizeof(WCHAR)] = 0;
-    *src += len / sizeof(WCHAR);
-    *dst += len / sizeof(WCHAR) + 1;
-}
-
-/***********************************************************************
- *           init_user_process_params
- *
- * Fill the RTL_USER_PROCESS_PARAMETERS structure from the server.
- */
-static NTSTATUS init_user_process_params( SIZE_T data_size )
-{
-    void *ptr;
-    WCHAR *src, *dst;
-    SIZE_T info_size, env_size, size, alloc_size;
-    NTSTATUS status;
-    startup_info_t *info;
-    RTL_USER_PROCESS_PARAMETERS *params = NULL;
-
-    if (!(info = RtlAllocateHeap( GetProcessHeap(), 0, data_size )))
-        return STATUS_NO_MEMORY;
-
-    SERVER_START_REQ( get_startup_info )
-    {
-        wine_server_set_reply( req, info, data_size );
-        if (!(status = wine_server_call( req )))
-        {
-            data_size = wine_server_reply_size( reply );
-            info_size = reply->info_size;
-            env_size  = data_size - info_size;
-        }
-    }
-    SERVER_END_REQ;
-    if (status != STATUS_SUCCESS) goto done;
-
-    size = sizeof(*params);
-    size += sizeof(current_dir);
-    size += info->dllpath_len + sizeof(WCHAR);
-    size += info->imagepath_len + sizeof(WCHAR);
-    size += info->cmdline_len + sizeof(WCHAR);
-    size += info->title_len + sizeof(WCHAR);
-    size += info->desktop_len + sizeof(WCHAR);
-    size += info->shellinfo_len + sizeof(WCHAR);
-    size += info->runtime_len + sizeof(WCHAR);
-
-    alloc_size = size;
-    status = NtAllocateVirtualMemory( NtCurrentProcess(), (void **)&params, 0, &alloc_size,
-                                      MEM_COMMIT, PAGE_READWRITE );
-    if (status != STATUS_SUCCESS) goto done;
-
-    NtCurrentTeb()->Peb->ProcessParameters = params;
-    params->AllocationSize  = alloc_size;
-    params->Size            = size;
-    params->Flags           = PROCESS_PARAMS_FLAG_NORMALIZED;
-    params->DebugFlags      = info->debug_flags;
-    params->ConsoleHandle   = wine_server_ptr_handle( info->console );
-    params->ConsoleFlags    = info->console_flags;
-    params->hStdInput       = wine_server_ptr_handle( info->hstdin );
-    params->hStdOutput      = wine_server_ptr_handle( info->hstdout );
-    params->hStdError       = wine_server_ptr_handle( info->hstderr );
-    params->dwX             = info->x;
-    params->dwY             = info->y;
-    params->dwXSize         = info->xsize;
-    params->dwYSize         = info->ysize;
-    params->dwXCountChars   = info->xchars;
-    params->dwYCountChars   = info->ychars;
-    params->dwFillAttribute = info->attribute;
-    params->dwFlags         = info->flags;
-    params->wShowWindow     = info->show;
-
-    src = (WCHAR *)(info + 1);
-    dst = (WCHAR *)(params + 1);
-
-    /* current directory needs more space */
-    get_unicode_string( &params->CurrentDirectory.DosPath, &src, &dst, info->curdir_len );
-    params->CurrentDirectory.DosPath.MaximumLength = sizeof(current_dir);
-    dst = (WCHAR *)(params + 1) + ARRAY_SIZE(current_dir);
-
-    get_unicode_string( &params->DllPath, &src, &dst, info->dllpath_len );
-    get_unicode_string( &params->ImagePathName, &src, &dst, info->imagepath_len );
-    get_unicode_string( &params->CommandLine, &src, &dst, info->cmdline_len );
-    get_unicode_string( &params->WindowTitle, &src, &dst, info->title_len );
-    get_unicode_string( &params->Desktop, &src, &dst, info->desktop_len );
-    get_unicode_string( &params->ShellInfo, &src, &dst, info->shellinfo_len );
-
-    /* runtime info isn't a real string */
-    params->RuntimeInfo.Buffer = dst;
-    params->RuntimeInfo.Length = params->RuntimeInfo.MaximumLength = info->runtime_len;
-    memcpy( dst, src, info->runtime_len );
-
-    /* environment needs to be a separate memory block */
-    ptr = NULL;
-    alloc_size = max( 1, env_size );
-    status = NtAllocateVirtualMemory( NtCurrentProcess(), &ptr, 0, &alloc_size,
-                                      MEM_COMMIT, PAGE_READWRITE );
-    if (status != STATUS_SUCCESS) goto done;
-    memcpy( ptr, (char *)info + info_size, env_size );
-    params->Environment = ptr;
-
-done:
-    RtlFreeHeap( GetProcessHeap(), 0, info );
-    return status;
-}
-
 #ifdef __linux__
 
 #ifdef HAVE_ELF_H
@@ -280,7 +163,6 @@ void thread_init(void)
     LARGE_INTEGER now;
     NTSTATUS status;
     struct ntdll_thread_data *thread_data;
-    static struct debug_info debug_info;  /* debug info for initial thread */
 
     virtual_init();
 
@@ -302,12 +184,10 @@ void thread_init(void)
 
     addr = NULL;
     size = sizeof(*peb);
-    NtAllocateVirtualMemory( NtCurrentProcess(), &addr, 1, &size,
-                             MEM_COMMIT | MEM_TOP_DOWN, PAGE_READWRITE );
+    virtual_alloc_aligned( &addr, 0, &size, MEM_COMMIT | MEM_TOP_DOWN, PAGE_READWRITE, 1 );
     peb = addr;
 
     peb->FastPebLock        = &peb_lock;
-    peb->ProcessParameters  = &params;
     peb->TlsBitmap          = &tls_bitmap;
     peb->TlsExpansionBitmap = &tls_expansion_bitmap;
     peb->FlsBitmap          = &fls_bitmap;
@@ -316,9 +196,6 @@ void thread_init(void)
     peb->OSMinorVersion     = 1;
     peb->OSBuildNumber      = 0xA28;
     peb->OSPlatformId       = VER_PLATFORM_WIN32_NT;
-    params.CurrentDirectory.DosPath.Buffer = current_dir;
-    params.CurrentDirectory.DosPath.MaximumLength = sizeof(current_dir);
-    params.wShowWindow = 1; /* SW_SHOWNORMAL */
     ldr.Length = sizeof(ldr);
     ldr.Initialized = TRUE;
     RtlInitializeBitMap( &tls_bitmap, peb->TlsBitmapBits, sizeof(peb->TlsBitmapBits) * 8 );
@@ -352,13 +229,9 @@ void thread_init(void)
     thread_data->reply_fd   = -1;
     thread_data->wait_fd[0] = -1;
     thread_data->wait_fd[1] = -1;
-    thread_data->debug_info = &debug_info;
 
     signal_init_thread( teb );
     virtual_init_threading();
-
-    debug_info.str_pos = debug_info.strings;
-    debug_info.out_pos = debug_info.output;
     debug_init();
 
     /* setup the server connection */
@@ -372,22 +245,8 @@ void thread_init(void)
         exit(1);
     }
 
-    /* allocate user parameters */
-    if (info_size)
-    {
-        init_user_process_params( info_size );
-    }
-    else
-    {
-        if (isatty(0) || isatty(1) || isatty(2))
-            params.ConsoleHandle = (HANDLE)2; /* see kernel32/kernel_private.h */
-        if (!isatty(0))
-            wine_server_fd_to_handle( 0, GENERIC_READ|SYNCHRONIZE,  OBJ_INHERIT, &params.hStdInput );
-        if (!isatty(1))
-            wine_server_fd_to_handle( 1, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, &params.hStdOutput );
-        if (!isatty(2))
-            wine_server_fd_to_handle( 2, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, &params.hStdError );
-    }
+    init_directories();
+    init_user_process_params( info_size );
 
     /* initialize time values in user_shared_data */
     NtQuerySystemTime( &now );
@@ -432,7 +291,7 @@ static void free_thread_data( TEB *teb )
 void abort_thread( int status )
 {
     pthread_sigmask( SIG_BLOCK, &server_block_set, NULL );
-    if (interlocked_xchg_add( &nb_threads, -1 ) <= 1) _exit( status );
+    if (interlocked_xchg_add( &nb_threads, -1 ) <= 1) _exit( get_unix_exit_code( status ));
     signal_exit_thread( status );
 }
 
@@ -473,7 +332,7 @@ void WINAPI RtlExitUserThread( ULONG status )
     {
         LdrShutdownProcess();
         pthread_sigmask( SIG_BLOCK, &server_block_set, NULL );
-        signal_exit_process( status );
+        signal_exit_process( get_unix_exit_code( status ));
     }
 
     LdrShutdownThread();
@@ -508,8 +367,7 @@ static void start_thread( struct startup_info *info )
     struct ntdll_thread_data *thread_data = (struct ntdll_thread_data *)&teb->GdiTebBatch;
     struct debug_info debug_info;
 
-    debug_info.str_pos = debug_info.strings;
-    debug_info.out_pos = debug_info.output;
+    debug_info.str_pos = debug_info.out_pos = 0;
     thread_data->debug_info = &debug_info;
     thread_data->pthread_id = pthread_self();
 
@@ -558,6 +416,7 @@ NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, SECURITY_DESCRIPTOR *descr,
     SIZE_T extra_stack = PTHREAD_STACK_MIN;
     data_size_t len = 0;
     struct object_attributes *objattr = NULL;
+    INITIAL_TEB stack;
 
     if (process != NtCurrentProcess())
     {
@@ -649,9 +508,13 @@ NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, SECURITY_DESCRIPTOR *descr,
     info->entry_point = start;
     info->entry_arg   = param;
 
-    if ((status = virtual_alloc_thread_stack( teb, stack_reserve, stack_commit, &extra_stack )))
+    if ((status = virtual_alloc_thread_stack( &stack, stack_reserve, stack_commit, &extra_stack )))
         goto error;
 
+    teb->Tib.StackBase = stack.StackBase;
+    teb->Tib.StackLimit = stack.StackLimit;
+    teb->DeallocationStack = stack.DeallocationStack;
+
     thread_data = (struct ntdll_thread_data *)&teb->GdiTebBatch;
     thread_data->request_fd  = request_pipe[1];
     thread_data->reply_fd    = -1;
