@@ -29,7 +29,7 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "winnls.h"
-#include "wine/unicode.h"
+#include "sqlext.h"
 #include "wine/debug.h"
 #include "wine/heap.h"
 
@@ -316,7 +316,7 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
         {
             WCHAR *divider, *value;
 
-            name = heap_alloc( (strlenW(args) + 1) * sizeof(WCHAR));
+            name = heap_alloc( (lstrlenW(args) + 1) * sizeof(WCHAR));
             if(!name)
             {
                 push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
@@ -324,7 +324,7 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
             }
             lstrcpyW(name, args);
 
-            divider = strchrW(name,'=');
+            divider = wcschr(name,'=');
             if(!divider)
             {
                 push_error(ODBC_ERROR_INVALID_KEYWORD_VALUE, odbc_error_invalid_keyword);
@@ -336,7 +336,7 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
 
             TRACE("Write pair: %s = %s\n", debugstr_w(name), debugstr_w(value));
             if(RegSetValueExW(hkeydriver, name, 0, REG_SZ, (BYTE*)value,
-                               (strlenW(value)+1) * sizeof(WCHAR)) != ERROR_SUCCESS)
+                               (lstrlenW(value)+1) * sizeof(WCHAR)) != ERROR_SUCCESS)
                 ERR("Failed to write registry installed key\n");
             heap_free(name);
 
@@ -859,7 +859,7 @@ static void write_registry_values(const WCHAR *regkey, const WCHAR *driver, cons
 
             for (; *p; p += lstrlenW(p) + 1)
             {
-                WCHAR *divider = strchrW(p,'=');
+                WCHAR *divider = wcschr(p,'=');
 
                 if (divider)
                 {
@@ -1514,34 +1514,44 @@ BOOL WINAPI SQLSetConfigMode(UWORD wConfigMode)
 
 BOOL WINAPI SQLValidDSNW(LPCWSTR lpszDSN)
 {
+    static const WCHAR invalid[] = {'[',']','{','}','(',')',',',';','?','*','=','!','@','\\',0};
     clear_errors();
-    FIXME("%s\n", debugstr_w(lpszDSN));
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return FALSE;
+    TRACE("%s\n", debugstr_w(lpszDSN));
+
+    if(lstrlenW(lpszDSN) > SQL_MAX_DSN_LENGTH || wcspbrk(lpszDSN, invalid) != NULL)
+    {
+        return FALSE;
+    }
+
+    return TRUE;
 }
 
 BOOL WINAPI SQLValidDSN(LPCSTR lpszDSN)
 {
+    static const char *invalid = "[]{}(),;?*=!@\\";
     clear_errors();
-    FIXME("%s\n", debugstr_a(lpszDSN));
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return FALSE;
+    TRACE("%s\n", debugstr_a(lpszDSN));
+
+    if(strlen(lpszDSN) > SQL_MAX_DSN_LENGTH || strpbrk(lpszDSN, invalid) != NULL)
+    {
+        return FALSE;
+    }
+
+    return TRUE;
 }
 
 BOOL WINAPI SQLWriteDSNToIniW(LPCWSTR lpszDSN, LPCWSTR lpszDriver)
 {
     clear_errors();
     FIXME("%s %s\n", debugstr_w(lpszDSN), debugstr_w(lpszDriver));
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return FALSE;
+    return TRUE;
 }
 
 BOOL WINAPI SQLWriteDSNToIni(LPCSTR lpszDSN, LPCSTR lpszDriver)
 {
     clear_errors();
     FIXME("%s %s\n", debugstr_a(lpszDSN), debugstr_a(lpszDriver));
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return FALSE;
+    return TRUE;
 }
 
 BOOL WINAPI SQLWriteFileDSNW(LPCWSTR lpszFileName, LPCWSTR lpszAppName,
