@@ -37,8 +37,8 @@ typedef struct {
     BaseFilter filter;
     IPersistPropertyBag IPersistPropertyBag_iface;
 
-    BaseInputPin *in;
-    BaseOutputPin *out;
+    BaseInputPin sink;
+    BaseOutputPin source;
 
     DWORD fcc_handler;
     HIC hic;
@@ -117,58 +117,6 @@ static HRESULT fill_format_info(AVICompressor *This, VIDEOINFOHEADER *src_videoi
     return S_OK;
 }
 
-static HRESULT WINAPI AVICompressor_QueryInterface(IBaseFilter *iface, REFIID riid, void **ppv)
-{
-    AVICompressor *This = impl_from_IBaseFilter(iface);
-
-    if(IsEqualIID(riid, &IID_IUnknown)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    }else if(IsEqualIID(riid, &IID_IPersist)) {
-        TRACE("(%p)->(IID_IPersist %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    }else if(IsEqualIID(riid, &IID_IMediaFilter)) {
-        TRACE("(%p)->(IID_IMediaFilter %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    }else if(IsEqualIID(riid, &IID_IBaseFilter)) {
-        TRACE("(%p)->(IID_IBaseFilter %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    }else if(IsEqualIID(riid, &IID_IPersistPropertyBag)) {
-        TRACE("(%p)->(IID_IPersistPropertyBag %p)\n", This, ppv);
-        *ppv = &This->IPersistPropertyBag_iface;
-    }else {
-        FIXME("no interface for %s\n", debugstr_guid(riid));
-        *ppv = NULL;
-        return E_NOINTERFACE;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-
-}
-
-static ULONG WINAPI AVICompressor_Release(IBaseFilter *iface)
-{
-    AVICompressor *This = impl_from_IBaseFilter(iface);
-    ULONG ref = InterlockedDecrement(&This->filter.refCount);
-
-    TRACE("(%p) ref=%d\n", This, ref);
-
-    if(!ref) {
-        if(This->hic)
-            ICClose(This->hic);
-        heap_free(This->videoinfo);
-        if(This->in)
-            BaseInputPinImpl_Release(&This->in->pin.IPin_iface);
-        if(This->out)
-            BaseOutputPinImpl_Release(&This->out->pin.IPin_iface);
-        strmbase_filter_cleanup(&This->filter);
-        heap_free(This);
-    }
-
-    return ref;
-}
-
 static HRESULT WINAPI AVICompressor_Stop(IBaseFilter *iface)
 {
     AVICompressor *This = impl_from_IBaseFilter(iface);
@@ -200,7 +148,7 @@ static HRESULT WINAPI AVICompressor_Run(IBaseFilter *iface, REFERENCE_TIME tStar
     if(This->filter.state == State_Running)
         return S_OK;
 
-    hres = IMemAllocator_Commit(This->out->pAllocator);
+    hres = IMemAllocator_Commit(This->source.pAllocator);
     if(FAILED(hres)) {
         FIXME("Commit failed: %08x\n", hres);
         return hres;
@@ -212,24 +160,10 @@ static HRESULT WINAPI AVICompressor_Run(IBaseFilter *iface, REFERENCE_TIME tStar
     return S_OK;
 }
 
-static HRESULT WINAPI AVICompressor_QueryFilterInfo(IBaseFilter *iface, FILTER_INFO *pInfo)
-{
-    AVICompressor *This = impl_from_IBaseFilter(iface);
-    FIXME("(%p)->(%p)\n", This, pInfo);
-    return E_NOTIMPL;
-}
-
-static HRESULT WINAPI AVICompressor_QueryVendorInfo(IBaseFilter *iface, LPWSTR *pVendorInfo)
-{
-    AVICompressor *This = impl_from_IBaseFilter(iface);
-    FIXME("(%p)->(%p)\n", This, pVendorInfo);
-    return E_NOTIMPL;
-}
-
 static const IBaseFilterVtbl AVICompressorVtbl = {
-    AVICompressor_QueryInterface,
+    BaseFilterImpl_QueryInterface,
     BaseFilterImpl_AddRef,
-    AVICompressor_Release,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     AVICompressor_Stop,
     AVICompressor_Pause,
@@ -239,29 +173,52 @@ static const IBaseFilterVtbl AVICompressorVtbl = {
     BaseFilterImpl_GetSyncSource,
     BaseFilterImpl_EnumPins,
     BaseFilterImpl_FindPin,
-    AVICompressor_QueryFilterInfo,
+    BaseFilterImpl_QueryFilterInfo,
     BaseFilterImpl_JoinFilterGraph,
-    AVICompressor_QueryVendorInfo
+    BaseFilterImpl_QueryVendorInfo,
 };
 
 static IPin *avi_compressor_get_pin(BaseFilter *iface, unsigned int index)
 {
-    AVICompressor *This = impl_from_BaseFilter(iface);
-    IPin *ret;
+    AVICompressor *filter = impl_from_BaseFilter(iface);
 
     if (index == 0)
-        ret = &This->in->pin.IPin_iface;
+        return &filter->sink.pin.IPin_iface;
     else if (index == 1)
-        ret = &This->out->pin.IPin_iface;
-    else
-        return NULL;
+        return &filter->source.pin.IPin_iface;
+    return NULL;
+}
 
-    IPin_AddRef(ret);
-    return ret;
+static void avi_compressor_destroy(BaseFilter *iface)
+{
+    AVICompressor *filter = impl_from_BaseFilter(iface);
+
+    if (filter->hic)
+        ICClose(filter->hic);
+    heap_free(filter->videoinfo);
+    strmbase_sink_cleanup(&filter->sink);
+    strmbase_source_cleanup(&filter->source);
+    strmbase_filter_cleanup(&filter->filter);
+    heap_free(filter);
+}
+
+static HRESULT avi_compressor_query_interface(BaseFilter *iface, REFIID iid, void **out)
+{
+    AVICompressor *filter = impl_from_BaseFilter(iface);
+
+    if (IsEqualGUID(iid, &IID_IPersistPropertyBag))
+        *out = &filter->IPersistPropertyBag_iface;
+    else
+        return E_NOINTERFACE;
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
 }
 
 static const BaseFilterFuncTable filter_func_table = {
     .filter_get_pin = avi_compressor_get_pin,
+    .filter_destroy = avi_compressor_destroy,
+    .filter_query_interface = avi_compressor_query_interface,
 };
 
 static AVICompressor *impl_from_IPersistPropertyBag(IPersistPropertyBag *iface)
@@ -366,18 +323,6 @@ static HRESULT WINAPI AVICompressorIn_QueryInterface(IPin *iface, REFIID riid, v
     return BaseInputPinImpl_QueryInterface(iface, riid, ppv);
 }
 
-static ULONG WINAPI AVICompressorIn_AddRef(IPin *iface)
-{
-    AVICompressor *This = impl_from_IPin(iface);
-    return IBaseFilter_AddRef(&This->filter.IBaseFilter_iface);
-}
-
-static ULONG WINAPI AVICompressorIn_Release(IPin *iface)
-{
-    AVICompressor *This = impl_from_IPin(iface);
-    return IBaseFilter_Release(&This->filter.IBaseFilter_iface);
-}
-
 static HRESULT WINAPI AVICompressorIn_ReceiveConnection(IPin *iface,
         IPin *pConnector, const AM_MEDIA_TYPE *pmt)
 {
@@ -415,8 +360,8 @@ static HRESULT WINAPI AVICompressorIn_Disconnect(IPin *iface)
 
 static const IPinVtbl AVICompressorInputPinVtbl = {
     AVICompressorIn_QueryInterface,
-    AVICompressorIn_AddRef,
-    AVICompressorIn_Release,
+    BasePinImpl_AddRef,
+    BasePinImpl_Release,
     BaseInputPinImpl_Connect,
     AVICompressorIn_ReceiveConnection,
     AVICompressorIn_Disconnect,
@@ -514,7 +459,7 @@ static HRESULT WINAPI AVICompressorIn_Receive(BaseInputPin *base, IMediaSample *
         return hres;
     }
 
-    hres = BaseOutputPinImpl_GetDeliveryBuffer(This->out, &out_sample, &start, &stop, 0);
+    hres = BaseOutputPinImpl_GetDeliveryBuffer(&This->source, &out_sample, &start, &stop, 0);
     if(FAILED(hres))
         return hres;
 
@@ -525,7 +470,7 @@ static HRESULT WINAPI AVICompressorIn_Receive(BaseInputPin *base, IMediaSample *
     if((This->driver_flags & VIDCF_TEMPORAL) && !(This->driver_flags & VIDCF_FASTTEMPORALC))
         FIXME("Unsupported temporal compression\n");
 
-    src_videoinfo = (VIDEOINFOHEADER*)This->in->pin.mtCurrent.pbFormat;
+    src_videoinfo = (VIDEOINFOHEADER *)This->sink.pin.mtCurrent.pbFormat;
     This->videoinfo->bmiHeader.biSizeImage = This->max_frame_size;
     res = ICCompress(This->hic, sync_point ? ICCOMPRESS_KEYFRAME : 0, &This->videoinfo->bmiHeader, buf,
             &src_videoinfo->bmiHeader, ptr, 0, &comp_flags, This->frame_cnt, 0, 0, NULL, NULL);
@@ -545,7 +490,7 @@ static HRESULT WINAPI AVICompressorIn_Receive(BaseInputPin *base, IMediaSample *
     else
         IMediaSample_SetMediaTime(out_sample, NULL, NULL);
 
-    hres = BaseOutputPinImpl_Deliver(This->out, out_sample);
+    hres = BaseOutputPinImpl_Deliver(&This->source, out_sample);
     if(FAILED(hres))
         WARN("Deliver failed: %08x\n", hres);
 
@@ -567,22 +512,10 @@ static HRESULT WINAPI AVICompressorOut_QueryInterface(IPin *iface, REFIID riid,
     return BaseInputPinImpl_QueryInterface(iface, riid, ppv);
 }
 
-static ULONG WINAPI AVICompressorOut_AddRef(IPin *iface)
-{
-    AVICompressor *This = impl_from_IPin(iface);
-    return IBaseFilter_AddRef(&This->filter.IBaseFilter_iface);
-}
-
-static ULONG WINAPI AVICompressorOut_Release(IPin *iface)
-{
-    AVICompressor *This = impl_from_IPin(iface);
-    return IBaseFilter_Release(&This->filter.IBaseFilter_iface);
-}
-
 static const IPinVtbl AVICompressorOutputPinVtbl = {
     AVICompressorOut_QueryInterface,
-    AVICompressorOut_AddRef,
-    AVICompressorOut_Release,
+    BasePinImpl_AddRef,
+    BasePinImpl_Release,
     BaseOutputPinImpl_Connect,
     BaseOutputPinImpl_ReceiveConnection,
     BaseOutputPinImpl_Disconnect,
@@ -613,7 +546,7 @@ static HRESULT WINAPI AVICompressorOut_GetMediaType(BasePin *base, int iPosition
     amt->subtype = MEDIASUBTYPE_PCM;
     amt->bFixedSizeSamples = FALSE;
     amt->bTemporalCompression = (This->driver_flags & VIDCF_TEMPORAL) != 0;
-    amt->lSampleSize = This->in->pin.mtCurrent.lSampleSize;
+    amt->lSampleSize = This->sink.pin.mtCurrent.lSampleSize;
     amt->formattype = FORMAT_VideoInfo;
     amt->pUnk = NULL;
     amt->cbFormat = This->videoinfo_size;
@@ -660,9 +593,6 @@ IUnknown* WINAPI QCAP_createAVICompressor(IUnknown *outer, HRESULT *phr)
     PIN_INFO in_pin_info  = {NULL, PINDIR_INPUT,  {'I','n',0}};
     PIN_INFO out_pin_info = {NULL, PINDIR_OUTPUT, {'O','u','t',0}};
     AVICompressor *compressor;
-    HRESULT hres;
-
-    TRACE("\n");
 
     compressor = heap_alloc_zero(sizeof(*compressor));
     if(!compressor) {
@@ -670,29 +600,19 @@ IUnknown* WINAPI QCAP_createAVICompressor(IUnknown *outer, HRESULT *phr)
         return NULL;
     }
 
-    BaseFilter_Init(&compressor->filter, &AVICompressorVtbl, &CLSID_AVICo,
+    strmbase_filter_init(&compressor->filter, &AVICompressorVtbl, outer, &CLSID_AVICo,
             (DWORD_PTR)(__FILE__ ": AVICompressor.csFilter"), &filter_func_table);
 
     compressor->IPersistPropertyBag_iface.lpVtbl = &PersistPropertyBagVtbl;
 
     in_pin_info.pFilter = &compressor->filter.IBaseFilter_iface;
-    hres = BaseInputPin_Construct(&AVICompressorInputPinVtbl, sizeof(BaseInputPin), &in_pin_info,
-            &AVICompressorBaseInputPinVtbl, &compressor->filter.csFilter, NULL, (IPin**)&compressor->in);
-    if(FAILED(hres)) {
-        strmbase_filter_cleanup(&compressor->filter);
-        *phr = hres;
-        return NULL;
-    }
+    strmbase_sink_init(&compressor->sink, &AVICompressorInputPinVtbl, &in_pin_info,
+            &AVICompressorBaseInputPinVtbl, &compressor->filter.csFilter, NULL);
 
     out_pin_info.pFilter = &compressor->filter.IBaseFilter_iface;
-    hres = BaseOutputPin_Construct(&AVICompressorOutputPinVtbl, sizeof(BaseOutputPin), &out_pin_info,
-            &AVICompressorBaseOutputPinVtbl, &compressor->filter.csFilter, (IPin**)&compressor->out);
-    if(FAILED(hres)) {
-        strmbase_filter_cleanup(&compressor->filter);
-        *phr = hres;
-        return NULL;
-    }
+    strmbase_source_init(&compressor->source, &AVICompressorOutputPinVtbl, &out_pin_info,
+            &AVICompressorBaseOutputPinVtbl, &compressor->filter.csFilter);
 
     *phr = S_OK;
-    return (IUnknown*)&compressor->filter.IBaseFilter_iface;
+    return &compressor->filter.IUnknown_inner;
 }
