@@ -435,9 +435,9 @@ static void test_enum_pins(void)
     hr = IEnumPins_Next(enum1, 1, pins, NULL);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ref = get_refcount(filter);
-    todo_wine ok(ref == 3, "Got unexpected refcount %d.\n", ref);
+    ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(pins[0]);
-    todo_wine ok(ref == 3, "Got unexpected refcount %d.\n", ref);
+    ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(enum1);
     ok(ref == 1, "Got unexpected refcount %d.\n", ref);
     IPin_Release(pins[0]);
@@ -663,6 +663,122 @@ static void test_pin_info(void)
     ok(!ref, "Got outstanding refcount %d.\n", ref);
 }
 
+static void test_media_types(void)
+{
+    WCHAR sink_id[] = {'V','M','R',' ','I','n','p','u','t','0',0};
+    IBaseFilter *filter = create_vmr9(0);
+    AM_MEDIA_TYPE *mt, req_mt = {{0}};
+    VIDEOINFOHEADER vih =
+    {
+        {0}, {0}, 0, 0, 0,
+        {sizeof(BITMAPINFOHEADER), 32, 24, 1, 0, BI_RGB}
+    };
+    IEnumMediaTypes *enummt;
+    unsigned int i;
+    HRESULT hr;
+    ULONG ref;
+    IPin *pin;
+
+    static const GUID *subtype_tests[] =
+    {
+        &MEDIASUBTYPE_RGB565,
+        &MEDIASUBTYPE_RGB24,
+        &MEDIASUBTYPE_RGB32,
+    };
+
+    IBaseFilter_FindPin(filter, sink_id, &pin);
+
+    hr = IPin_EnumMediaTypes(pin, &enummt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enummt, 1, &mt, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    IEnumMediaTypes_Release(enummt);
+
+    req_mt.majortype = MEDIATYPE_Video;
+    req_mt.formattype = FORMAT_VideoInfo;
+    req_mt.cbFormat = sizeof(VIDEOINFOHEADER);
+    req_mt.pbFormat = (BYTE *)&vih;
+
+    for (i = 0; i < ARRAY_SIZE(subtype_tests); ++i)
+    {
+        req_mt.subtype = *subtype_tests[i];
+        hr = IPin_QueryAccept(pin, &req_mt);
+        ok(hr == S_OK, "Got hr %#x for subtype %s.\n", hr, wine_dbgstr_guid(subtype_tests[i]));
+    }
+
+    req_mt.subtype = MEDIASUBTYPE_RGB8;
+    hr = IPin_QueryAccept(pin, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.subtype = MEDIASUBTYPE_NULL;
+    hr = IPin_QueryAccept(pin, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.subtype = MEDIASUBTYPE_RGB24;
+
+    req_mt.majortype = MEDIATYPE_NULL;
+    hr = IPin_QueryAccept(pin, &req_mt);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.majortype = MEDIATYPE_Video;
+
+    req_mt.formattype = FORMAT_None;
+    hr = IPin_QueryAccept(pin, &req_mt);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    req_mt.formattype = GUID_NULL;
+    hr = IPin_QueryAccept(pin, &req_mt);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    IPin_Release(pin);
+    ref = IBaseFilter_Release(filter);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+}
+
+static void test_enum_media_types(void)
+{
+    WCHAR sink_id[] = {'V','M','R',' ','I','n','p','u','t','0',0};
+    IBaseFilter *filter = create_vmr9(0);
+    IEnumMediaTypes *enum1, *enum2;
+    AM_MEDIA_TYPE *mts[2];
+    ULONG ref, count;
+    HRESULT hr;
+    IPin *pin;
+
+    IBaseFilter_FindPin(filter, sink_id, &pin);
+
+    hr = IPin_EnumMediaTypes(pin, &enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, &count);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(!count, "Got count %u.\n", count);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Clone(enum1, &enum2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Skip(enum1, 1);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum2, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    IEnumMediaTypes_Release(enum1);
+    IEnumMediaTypes_Release(enum2);
+    IPin_Release(pin);
+
+    ref = IBaseFilter_Release(filter);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+}
+
 START_TEST(vmr9)
 {
     IBaseFilter *filter;
@@ -684,6 +800,8 @@ START_TEST(vmr9)
     test_enum_pins();
     test_find_pin();
     test_pin_info();
+    test_media_types();
+    test_enum_media_types();
 
     CoUninitialize();
 }
