@@ -103,6 +103,9 @@ void ME_CheckCharOffsets(ME_TextEditor *editor)
   ME_DisplayItem *p = editor->pBuffer->pFirst;
   int ofs = 0, ofsp = 0;
 
+  if (!TRACE_ON(richedit_check))
+    return;
+
   TRACE_(richedit_check)("Checking begin\n");
   if(TRACE_ON(richedit_lists))
   {
@@ -224,7 +227,7 @@ void ME_JoinRuns(ME_TextEditor *editor, ME_DisplayItem *p)
   int i;
   assert(p->type == diRun && pNext->type == diRun);
   assert(p->member.run.nCharOfs != -1);
-  ME_GetParagraph(p)->member.para.nFlags |= MEPF_REWRAP;
+  mark_para_rewrap(editor, ME_GetParagraph(p));
 
   /* Update all cursors so that they don't contain the soon deleted run */
   for (i=0; i<editor->nCursors; i++) {
@@ -238,8 +241,7 @@ void ME_JoinRuns(ME_TextEditor *editor, ME_DisplayItem *p)
   ME_Remove(pNext);
   ME_DestroyDisplayItem(pNext);
   ME_UpdateRunFlags(editor, &p->member.run);
-  if(TRACE_ON(richedit_check))
-    ME_CheckCharOffsets(editor);
+  ME_CheckCharOffsets(editor);
 }
 
 /******************************************************************************
@@ -277,7 +279,7 @@ ME_DisplayItem *ME_SplitRunSimple(ME_TextEditor *editor, ME_Cursor *cursor)
       editor->pCursors[i].nOffset -= nOffset;
     }
   }
-  cursor->pPara->member.para.nFlags |= MEPF_REWRAP;
+  mark_para_rewrap(editor, cursor->pPara);
   return run;
 }
 
@@ -345,7 +347,7 @@ ME_InsertRunAtCursor(ME_TextEditor *editor, ME_Cursor *cursor, ME_Style *style,
   ME_InsertBefore( insert_before, pDI );
   TRACE("Shift length:%d\n", len);
   ME_PropagateCharOffset( insert_before, len );
-  insert_before->member.run.para->nFlags |= MEPF_REWRAP;
+  mark_para_rewrap(editor, get_di_from_para(insert_before->member.run.para));
 
   /* Move any cursors that were at the end of the previous run to the end of the inserted run */
   prev = ME_FindItemBack( pDI, diRun );
@@ -610,8 +612,8 @@ int ME_PointFromChar(ME_TextEditor *editor, ME_Run *pRun, int nOffset, BOOL visu
 SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run *run, int nLen,
                          int startx, int *pAscent, int *pDescent)
 {
+  static const WCHAR spaceW[] = {' ',0};
   SIZE size;
-  WCHAR spaceW[] = {' ',0};
 
   nLen = min( nLen, run->len );
 
@@ -766,7 +768,7 @@ void ME_SetCharFormat(ME_TextEditor *editor, ME_Cursor *start, ME_Cursor *end, C
       ME_ReleaseStyle(para->para_num.style);
       para->para_num.style = NULL;
     }
-    para->nFlags |= MEPF_REWRAP;
+    mark_para_rewrap(editor, get_di_from_para(para));
   }
 }
 
@@ -860,7 +862,7 @@ void ME_GetCharFormat(ME_TextEditor *editor, const ME_Cursor *from,
     {
       if (!(tmp.dwMask & CFM_FACE))
         pFmt->dwMask &= ~CFM_FACE;
-      else if (lstrcmpW(pFmt->szFaceName, tmp.szFaceName) ||
+      else if (wcscmp(pFmt->szFaceName, tmp.szFaceName) ||
           pFmt->bPitchAndFamily != tmp.bPitchAndFamily)
         pFmt->dwMask &= ~CFM_FACE;
     }
