@@ -55,266 +55,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(shell);
 
-/*************************************************************************
- * CommandLineToArgvW            [SHELL32.@]
- *
- * We must interpret the quotes in the command line to rebuild the argv
- * array correctly:
- * - arguments are separated by spaces or tabs
- * - quotes serve as optional argument delimiters
- *   '"a b"'   -> 'a b'
- * - escaped quotes must be converted back to '"'
- *   '\"'      -> '"'
- * - consecutive backslashes preceding a quote see their number halved with
- *   the remainder escaping the quote:
- *   2n   backslashes + quote -> n backslashes + quote as an argument delimiter
- *   2n+1 backslashes + quote -> n backslashes + literal quote
- * - backslashes that are not followed by a quote are copied literally:
- *   'a\b'     -> 'a\b'
- *   'a\\b'    -> 'a\\b'
- * - in quoted strings, consecutive quotes see their number divided by three
- *   with the remainder modulo 3 deciding whether to close the string or not.
- *   Note that the opening quote must be counted in the consecutive quotes,
- *   that's the (1+) below:
- *   (1+) 3n   quotes -> n quotes
- *   (1+) 3n+1 quotes -> n quotes plus closes the quoted string
- *   (1+) 3n+2 quotes -> n+1 quotes plus closes the quoted string
- * - in unquoted strings, the first quote opens the quoted string and the
- *   remaining consecutive quotes follow the above rule.
- */
-LPWSTR* WINAPI CommandLineToArgvW(LPCWSTR lpCmdline, int* numargs)
-{
-    DWORD argc;
-    LPWSTR  *argv;
-    LPCWSTR s;
-    LPWSTR d;
-    LPWSTR cmdline;
-    int qcount,bcount;
-
-    if(!numargs)
-    {
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return NULL;
-    }
-
-    if (*lpCmdline==0)
-    {
-        /* Return the path to the executable */
-        DWORD len, deslen=MAX_PATH, size;
-
-        size = sizeof(LPWSTR)*2 + deslen*sizeof(WCHAR);
-        for (;;)
-        {
-            if (!(argv = LocalAlloc(LMEM_FIXED, size))) return NULL;
-            len = GetModuleFileNameW(0, (LPWSTR)(argv+2), deslen);
-            if (!len)
-            {
-                LocalFree(argv);
-                return NULL;
-            }
-            if (len < deslen) break;
-            deslen*=2;
-            size = sizeof(LPWSTR)*2 + deslen*sizeof(WCHAR);
-            LocalFree( argv );
-        }
-        argv[0]=(LPWSTR)(argv+2);
-        argv[1]=NULL;
-        *numargs=1;
-
-        return argv;
-    }
-
-    /* --- First count the arguments */
-    argc=1;
-    s=lpCmdline;
-    /* The first argument, the executable path, follows special rules */
-    if (*s=='"')
-    {
-        /* The executable path ends at the next quote, no matter what */
-        s++;
-        while (*s)
-            if (*s++=='"')
-                break;
-    }
-    else
-    {
-        /* The executable path ends at the next space, no matter what */
-        while (*s && *s!=' ' && *s!='\t')
-            s++;
-    }
-    /* skip to the first argument, if any */
-    while (*s==' ' || *s=='\t')
-        s++;
-    if (*s)
-        argc++;
-
-    /* Analyze the remaining arguments */
-    qcount=bcount=0;
-    while (*s)
-    {
-        if ((*s==' ' || *s=='\t') && qcount==0)
-        {
-            /* skip to the next argument and count it if any */
-            while (*s==' ' || *s=='\t')
-                s++;
-            if (*s)
-                argc++;
-            bcount=0;
-        }
-        else if (*s=='\\')
-        {
-            /* '\', count them */
-            bcount++;
-            s++;
-        }
-        else if (*s=='"')
-        {
-            /* '"' */
-            if ((bcount & 1)==0)
-                qcount++; /* unescaped '"' */
-            s++;
-            bcount=0;
-            /* consecutive quotes, see comment in copying code below */
-            while (*s=='"')
-            {
-                qcount++;
-                s++;
-            }
-            qcount=qcount % 3;
-            if (qcount==2)
-                qcount=0;
-        }
-        else
-        {
-            /* a regular character */
-            bcount=0;
-            s++;
-        }
-    }
-
-    /* Allocate in a single lump, the string array, and the strings that go
-     * with it. This way the caller can make a single LocalFree() call to free
-     * both, as per MSDN.
-     */
-    argv=LocalAlloc(LMEM_FIXED, (argc+1)*sizeof(LPWSTR)+(strlenW(lpCmdline)+1)*sizeof(WCHAR));
-    if (!argv)
-        return NULL;
-    cmdline=(LPWSTR)(argv+argc+1);
-    strcpyW(cmdline, lpCmdline);
-
-    /* --- Then split and copy the arguments */
-    argv[0]=d=cmdline;
-    argc=1;
-    /* The first argument, the executable path, follows special rules */
-    if (*d=='"')
-    {
-        /* The executable path ends at the next quote, no matter what */
-        s=d+1;
-        while (*s)
-        {
-            if (*s=='"')
-            {
-                s++;
-                break;
-            }
-            *d++=*s++;
-        }
-    }
-    else
-    {
-        /* The executable path ends at the next space, no matter what */
-        while (*d && *d!=' ' && *d!='\t')
-            d++;
-        s=d;
-        if (*s)
-            s++;
-    }
-    /* close the executable path */
-    *d++=0;
-    /* skip to the first argument and initialize it if any */
-    while (*s==' ' || *s=='\t')
-        s++;
-    if (!*s)
-    {
-        /* There are no parameters so we are all done */
-        argv[argc]=NULL;
-        *numargs=argc;
-        return argv;
-    }
-
-    /* Split and copy the remaining arguments */
-    argv[argc++]=d;
-    qcount=bcount=0;
-    while (*s)
-    {
-        if ((*s==' ' || *s=='\t') && qcount==0)
-        {
-            /* close the argument */
-            *d++=0;
-            bcount=0;
-
-            /* skip to the next one and initialize it if any */
-            do {
-                s++;
-            } while (*s==' ' || *s=='\t');
-            if (*s)
-                argv[argc++]=d;
-        }
-        else if (*s=='\\')
-        {
-            *d++=*s++;
-            bcount++;
-        }
-        else if (*s=='"')
-        {
-            if ((bcount & 1)==0)
-            {
-                /* Preceded by an even number of '\', this is half that
-                 * number of '\', plus a quote which we erase.
-                 */
-                d-=bcount/2;
-                qcount++;
-            }
-            else
-            {
-                /* Preceded by an odd number of '\', this is half that
-                 * number of '\' followed by a '"'
-                 */
-                d=d-bcount/2-1;
-                *d++='"';
-            }
-            s++;
-            bcount=0;
-            /* Now count the number of consecutive quotes. Note that qcount
-             * already takes into account the opening quote if any, as well as
-             * the quote that lead us here.
-             */
-            while (*s=='"')
-            {
-                if (++qcount==3)
-                {
-                    *d++='"';
-                    qcount=0;
-                }
-                s++;
-            }
-            if (qcount==2)
-                qcount=0;
-        }
-        else
-        {
-            /* a regular character */
-            *d++=*s++;
-            bcount=0;
-        }
-    }
-    *d='\0';
-    argv[argc]=NULL;
-    *numargs=argc;
-
-    return argv;
-}
-
 static DWORD shgfi_get_exe_type(LPCWSTR szFullPath)
 {
     BOOL status = FALSE;
@@ -723,8 +463,8 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
 
                     while ((hr = SIC_get_location( psfi->iIcon, file, &size, &icon_idx )) == E_NOT_SUFFICIENT_BUFFER)
                     {
-                        if (file == buf) file = HeapAlloc( GetProcessHeap(), 0, size );
-                        else file = HeapReAlloc( GetProcessHeap(), 0, file, size );
+                        if (file == buf) file = heap_alloc( size );
+                        else file = heap_realloc( file, size );
                         if (!file) break;
                     }
                     if (SUCCEEDED(hr))
@@ -732,7 +472,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                         ret = PrivateExtractIconsW( file, icon_idx, width, height, &psfi->hIcon, 0, 1, 0);
                         if (ret == 0 || ret == (UINT)-1) hr = E_FAIL;
                     }
-                    if (file != buf) HeapFree( GetProcessHeap(), 0, file );
+                    if (file != buf) heap_free( file );
                 }
             }
         }
@@ -783,7 +523,7 @@ DWORD_PTR WINAPI SHGetFileInfoA(LPCSTR path,DWORD dwFileAttributes,
     else
     {
         len = MultiByteToWideChar(CP_ACP, 0, path, -1, NULL, 0);
-        temppath = HeapAlloc(GetProcessHeap(), 0, len*sizeof(WCHAR));
+        temppath = heap_alloc(len*sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, path, -1, temppath, len);
         pathW = temppath;
     }
@@ -816,7 +556,7 @@ DWORD_PTR WINAPI SHGetFileInfoA(LPCSTR path,DWORD dwFileAttributes,
         }
     }
 
-    HeapFree(GetProcessHeap(), 0, temppath);
+    heap_free(temppath);
 
     return ret;
 }
@@ -855,7 +595,7 @@ HICON WINAPI ExtractIconA(HINSTANCE hInstance, const char *file, UINT nIconIndex
 
     fileW = strdupAtoW(file);
     ret = ExtractIconW(hInstance, fileW, nIconIndex);
-    HeapFree(GetProcessHeap(), 0, fileW);
+    heap_free(fileW);
 
     return ret;
 }
@@ -1037,7 +777,7 @@ static void add_authors( HWND list )
 
     if (!strA) return;
     sizeW = MultiByteToWideChar( CP_UTF8, 0, strA, sizeA, NULL, 0 ) + 1;
-    if (!(strW = HeapAlloc( GetProcessHeap(), 0, sizeW * sizeof(WCHAR) ))) return;
+    if (!(strW = heap_alloc( sizeW * sizeof(WCHAR) ))) return;
     MultiByteToWideChar( CP_UTF8, 0, strA, sizeA, strW, sizeW );
     strW[sizeW - 1] = 0;
 
@@ -1051,7 +791,7 @@ static void add_authors( HWND list )
         SendMessageW( list, LB_ADDSTRING, -1, (LPARAM)start );
         start = end;
     }
-    HeapFree( GetProcessHeap(), 0, strW );
+    heap_free( strW );
 }
 
 /*************************************************************************
@@ -1070,22 +810,26 @@ static INT_PTR CALLBACK AboutDlgProc( HWND hWnd, UINT msg, WPARAM wParam,
         {
             ABOUT_INFO *info = (ABOUT_INFO *)lParam;
             WCHAR template[512], buffer[512], version[64];
-            extern const char *wine_get_build_id(void);
+            const char *(CDECL *wine_get_build_id)(void);
 
             if (info)
             {
+                wine_get_build_id = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"),
+                                                            "wine_get_build_id");
                 SendDlgItemMessageW(hWnd, stc1, STM_SETICON,(WPARAM)info->hIcon, 0);
-                GetWindowTextW( hWnd, template, sizeof(template)/sizeof(WCHAR) );
+                GetWindowTextW( hWnd, template, ARRAY_SIZE(template) );
                 sprintfW( buffer, template, info->szApp );
                 SetWindowTextW( hWnd, buffer );
                 SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT1), info->szApp );
                 SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT2), info->szOtherStuff );
                 GetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT3),
-                                template, sizeof(template)/sizeof(WCHAR) );
-                MultiByteToWideChar( CP_UTF8, 0, wine_get_build_id(), -1,
-                                     version, sizeof(version)/sizeof(WCHAR) );
-                sprintfW( buffer, template, version );
-                SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT3), buffer );
+                                template, ARRAY_SIZE(template) );
+                if (wine_get_build_id)
+                {
+                    MultiByteToWideChar( CP_UTF8, 0, wine_get_build_id(), -1, version, ARRAY_SIZE(version) );
+                    sprintfW( buffer, template, version );
+                    SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT3), buffer );
+                }
                 hWndCtl = GetDlgItem(hWnd, IDC_ABOUT_LISTBOX);
                 SendMessageW( hWndCtl, WM_SETREDRAW, 0, 0 );
                 SendMessageW( hWndCtl, WM_SETFONT, (WPARAM)info->hFont, 0 );
@@ -1148,20 +892,20 @@ BOOL WINAPI ShellAboutA( HWND hWnd, LPCSTR szApp, LPCSTR szOtherStuff, HICON hIc
     if (szApp)
     {
         len = MultiByteToWideChar(CP_ACP, 0, szApp, -1, NULL, 0);
-        appW = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+        appW = heap_alloc( len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, szApp, -1, appW, len);
     }
     if (szOtherStuff)
     {
         len = MultiByteToWideChar(CP_ACP, 0, szOtherStuff, -1, NULL, 0);
-        otherW = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+        otherW = heap_alloc( len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, szOtherStuff, -1, otherW, len);
     }
 
     ret = ShellAboutW(hWnd, appW, otherW, hIcon);
 
-    HeapFree(GetProcessHeap(), 0, otherW);
-    HeapFree(GetProcessHeap(), 0, appW);
+    heap_free(otherW);
+    heap_free(appW);
     return ret;
 }
 
@@ -1377,25 +1121,6 @@ HRESULT WINAPI SHGetLocalizedName(LPCWSTR path, LPWSTR module, UINT size, INT *r
     return E_NOTIMPL;
 }
 
-/***********************************************************************
- *              SetCurrentProcessExplicitAppUserModelID (SHELL32.@)
- */
-HRESULT WINAPI SetCurrentProcessExplicitAppUserModelID(PCWSTR appid)
-{
-    FIXME("%s: stub\n", debugstr_w(appid));
-    return E_NOTIMPL;
-}
-
-/***********************************************************************
- *              GetCurrentProcessExplicitAppUserModelID (SHELL32.@)
- */
-HRESULT WINAPI GetCurrentProcessExplicitAppUserModelID(PWSTR *appid)
-{
-    FIXME("%p: stub\n", appid);
-    *appid = NULL;
-    return E_NOTIMPL;
-}
-
 /***********************************************************************
  *              SHSetUnreadMailCountW (SHELL32.@)
  */
