@@ -22,11 +22,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(strmbase);
 
-static inline BaseInputPin *impl_BaseInputPin_from_IPin( IPin *iface )
-{
-    return CONTAINING_RECORD(iface, BaseInputPin, pin.IPin_iface);
-}
-
 static inline BaseRenderer *impl_from_IBaseFilter(IBaseFilter *iface)
 {
     return CONTAINING_RECORD(iface, BaseRenderer, filter.IBaseFilter_iface);
@@ -45,59 +40,60 @@ static const IQualityControlVtbl Renderer_QualityControl_Vtbl = {
     QualityControlImpl_SetSink
 };
 
-static HRESULT WINAPI BaseRenderer_InputPin_ReceiveConnection(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
+static inline BaseRenderer *impl_from_IPin(IPin *iface)
 {
-    BaseInputPin *This = impl_BaseInputPin_from_IPin(iface);
-    BaseRenderer *renderer = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
+    return CONTAINING_RECORD(iface, BaseRenderer, sink.pin.IPin_iface);
+}
+
+static HRESULT WINAPI BaseRenderer_InputPin_ReceiveConnection(IPin *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
+{
+    BaseRenderer *filter = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p/%p)->(%p, %p)\n", This, renderer, pReceivePin, pmt);
+    TRACE("iface %p, peer %p, mt %p.\n", iface, peer, mt);
 
-    EnterCriticalSection(This->pin.pCritSec);
-    hr = BaseInputPinImpl_ReceiveConnection(iface, pReceivePin, pmt);
+    EnterCriticalSection(filter->sink.pin.pCritSec);
+    hr = BaseInputPinImpl_ReceiveConnection(iface, peer, mt);
     if (SUCCEEDED(hr))
     {
-        if (renderer->pFuncsTable->pfnCompleteConnect)
-            hr = renderer->pFuncsTable->pfnCompleteConnect(renderer, pReceivePin);
+        if (filter->pFuncsTable->pfnCompleteConnect)
+            hr = filter->pFuncsTable->pfnCompleteConnect(filter, peer);
     }
-    LeaveCriticalSection(This->pin.pCritSec);
+    LeaveCriticalSection(filter->sink.pin.pCritSec);
 
     return hr;
 }
 
 static HRESULT WINAPI BaseRenderer_InputPin_Disconnect(IPin * iface)
 {
-    BaseInputPin *This = impl_BaseInputPin_from_IPin(iface);
-    BaseRenderer *renderer = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
+    BaseRenderer *filter = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p/%p)\n", This, renderer);
+    TRACE("iface %p.\n", iface);
 
-    EnterCriticalSection(This->pin.pCritSec);
+    EnterCriticalSection(filter->sink.pin.pCritSec);
     hr = BasePinImpl_Disconnect(iface);
     if (SUCCEEDED(hr))
     {
-        if (renderer->pFuncsTable->pfnBreakConnect)
-            hr = renderer->pFuncsTable->pfnBreakConnect(renderer);
+        if (filter->pFuncsTable->pfnBreakConnect)
+            hr = filter->pFuncsTable->pfnBreakConnect(filter);
     }
-    BaseRendererImpl_ClearPendingSample(renderer);
-    LeaveCriticalSection(This->pin.pCritSec);
+    BaseRendererImpl_ClearPendingSample(filter);
+    LeaveCriticalSection(filter->sink.pin.pCritSec);
 
     return hr;
 }
 
 static HRESULT WINAPI BaseRenderer_InputPin_EndOfStream(IPin * iface)
 {
+    BaseRenderer *pFilter = impl_from_IPin(iface);
     HRESULT hr;
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    BaseRenderer *pFilter = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
 
-    TRACE("(%p/%p)->()\n", This, pFilter);
+    TRACE("iface %p.\n", iface);
 
     EnterCriticalSection(&pFilter->csRenderLock);
     EnterCriticalSection(&pFilter->filter.csFilter);
     hr = BaseInputPinImpl_EndOfStream(iface);
-    EnterCriticalSection(This->pin.pCritSec);
     if (SUCCEEDED(hr))
     {
         if (pFilter->pFuncsTable->pfnEndOfStream)
@@ -105,7 +101,6 @@ static HRESULT WINAPI BaseRenderer_InputPin_EndOfStream(IPin * iface)
         else
             hr = BaseRendererImpl_EndOfStream(pFilter);
     }
-    LeaveCriticalSection(This->pin.pCritSec);
     LeaveCriticalSection(&pFilter->filter.csFilter);
     LeaveCriticalSection(&pFilter->csRenderLock);
     return hr;
@@ -113,24 +108,16 @@ static HRESULT WINAPI BaseRenderer_InputPin_EndOfStream(IPin * iface)
 
 static HRESULT WINAPI BaseRenderer_InputPin_BeginFlush(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    BaseRenderer *pFilter = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
+    BaseRenderer *pFilter = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p/%p)->()\n", This, iface);
+    TRACE("iface %p.\n", iface);
 
     EnterCriticalSection(&pFilter->csRenderLock);
     EnterCriticalSection(&pFilter->filter.csFilter);
-    EnterCriticalSection(This->pin.pCritSec);
     hr = BaseInputPinImpl_BeginFlush(iface);
     if (SUCCEEDED(hr))
-    {
-        if (pFilter->pFuncsTable->pfnBeginFlush)
-            hr = pFilter->pFuncsTable->pfnBeginFlush(pFilter);
-        else
-            hr = BaseRendererImpl_BeginFlush(pFilter);
-    }
-    LeaveCriticalSection(This->pin.pCritSec);
+        hr = BaseRendererImpl_BeginFlush(pFilter);
     LeaveCriticalSection(&pFilter->filter.csFilter);
     LeaveCriticalSection(&pFilter->csRenderLock);
     return hr;
@@ -138,15 +125,13 @@ static HRESULT WINAPI BaseRenderer_InputPin_BeginFlush(IPin * iface)
 
 static HRESULT WINAPI BaseRenderer_InputPin_EndFlush(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    BaseRenderer *pFilter = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
+    BaseRenderer *pFilter = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p/%p)->()\n", This, pFilter);
+    TRACE("iface %p.\n", iface);
 
     EnterCriticalSection(&pFilter->csRenderLock);
     EnterCriticalSection(&pFilter->filter.csFilter);
-    EnterCriticalSection(This->pin.pCritSec);
     hr = BaseInputPinImpl_EndFlush(iface);
     if (SUCCEEDED(hr))
     {
@@ -155,7 +140,6 @@ static HRESULT WINAPI BaseRenderer_InputPin_EndFlush(IPin * iface)
         else
             hr = BaseRendererImpl_EndFlush(pFilter);
     }
-    LeaveCriticalSection(This->pin.pCritSec);
     LeaveCriticalSection(&pFilter->filter.csFilter);
     LeaveCriticalSection(&pFilter->csRenderLock);
     return hr;
@@ -165,7 +149,7 @@ static const IPinVtbl BaseRenderer_InputPin_Vtbl =
 {
     BaseInputPinImpl_QueryInterface,
     BasePinImpl_AddRef,
-    BaseInputPinImpl_Release,
+    BasePinImpl_Release,
     BaseInputPinImpl_Connect,
     BaseRenderer_InputPin_ReceiveConnection,
     BaseRenderer_InputPin_Disconnect,
@@ -185,31 +169,59 @@ static const IPinVtbl BaseRenderer_InputPin_Vtbl =
 
 static IPin *renderer_get_pin(BaseFilter *iface, unsigned int index)
 {
-    BaseRenderer *This = impl_from_BaseFilter(iface);
+    BaseRenderer *filter = impl_from_BaseFilter(iface);
 
-    if (index >= 1)
-        return NULL;
-
-    IPin_AddRef(&This->pInputPin->pin.IPin_iface);
-    return &This->pInputPin->pin.IPin_iface;
+    if (index == 0)
+        return &filter->sink.pin.IPin_iface;
+    return NULL;
 }
 
-static HRESULT WINAPI BaseRenderer_Input_CheckMediaType(BasePin *pin, const AM_MEDIA_TYPE * pmt)
+static void renderer_destroy(BaseFilter *iface)
 {
-    BaseRenderer *This = impl_from_IBaseFilter(pin->pinInfo.pFilter);
-    return This->pFuncsTable->pfnCheckMediaType(This, pmt);
+    BaseRenderer *filter = impl_from_BaseFilter(iface);
+    filter->pFuncsTable->renderer_destroy(filter);
 }
 
-static HRESULT WINAPI BaseRenderer_Receive(BaseInputPin *pin, IMediaSample * pSample)
+static HRESULT renderer_query_interface(BaseFilter *iface, REFIID iid, void **out)
 {
-    BaseRenderer *This = impl_from_IBaseFilter(pin->pin.pinInfo.pFilter);
-    return BaseRendererImpl_Receive(This, pSample);
+    BaseRenderer *filter = impl_from_BaseFilter(iface);
+    HRESULT hr;
+
+    if (filter->pFuncsTable->renderer_query_interface
+            && SUCCEEDED(hr = filter->pFuncsTable->renderer_query_interface(filter, iid, out)))
+    {
+        return hr;
+    }
+
+    if (IsEqualIID(iid, &IID_IMediaSeeking) || IsEqualIID(iid, &IID_IMediaPosition))
+        return IUnknown_QueryInterface(filter->pPosition, iid, out);
+    else if (IsEqualIID(iid, &IID_IQualityControl))
+    {
+        *out = &filter->qcimpl->IQualityControl_iface;
+        IUnknown_AddRef((IUnknown *)*out);
+        return S_OK;
+    }
+    return E_NOINTERFACE;
 }
 
 static const BaseFilterFuncTable RendererBaseFilterFuncTable = {
     .filter_get_pin = renderer_get_pin,
+    .filter_destroy = renderer_destroy,
+    .filter_query_interface = renderer_query_interface,
 };
 
+static HRESULT WINAPI BaseRenderer_Input_CheckMediaType(BasePin *pin, const AM_MEDIA_TYPE *mt)
+{
+    BaseRenderer *filter = impl_from_IPin(&pin->IPin_iface);
+    return filter->pFuncsTable->pfnCheckMediaType(filter, mt);
+}
+
+static HRESULT WINAPI BaseRenderer_Receive(BaseInputPin *pin, IMediaSample *sample)
+{
+    BaseRenderer *filter = impl_from_IPin(&pin->pin.IPin_iface);
+    return BaseRendererImpl_Receive(filter, sample);
+}
+
 static const BaseInputPinFuncTable input_BaseInputFuncTable = {
     {
         BaseRenderer_Input_CheckMediaType,
@@ -219,73 +231,53 @@ static const BaseInputPinFuncTable input_BaseInputFuncTable = {
 };
 
 
-HRESULT WINAPI strmbase_renderer_init(BaseRenderer *This, const IBaseFilterVtbl *Vtbl,
-        IUnknown *pUnkOuter, const CLSID *pClsid, const WCHAR *sink_name, DWORD_PTR DebugInfo,
+HRESULT WINAPI strmbase_renderer_init(BaseRenderer *filter, const IBaseFilterVtbl *vtbl,
+        IUnknown *outer, const CLSID *clsid, const WCHAR *sink_name, DWORD_PTR debug_info,
         const BaseRendererFuncTable *pBaseFuncsTable)
 {
     PIN_INFO piInput;
     HRESULT hr;
 
-    BaseFilter_Init(&This->filter, Vtbl, pClsid, DebugInfo, &RendererBaseFilterFuncTable);
+    strmbase_filter_init(&filter->filter, vtbl, outer, clsid, debug_info, &RendererBaseFilterFuncTable);
 
-    This->pFuncsTable = pBaseFuncsTable;
+    filter->pFuncsTable = pBaseFuncsTable;
 
     /* construct input pin */
     piInput.dir = PINDIR_INPUT;
-    piInput.pFilter = &This->filter.IBaseFilter_iface;
+    piInput.pFilter = &filter->filter.IBaseFilter_iface;
     lstrcpynW(piInput.achName, sink_name, ARRAY_SIZE(piInput.achName));
 
-    hr = BaseInputPin_Construct(&BaseRenderer_InputPin_Vtbl, sizeof(BaseInputPin), &piInput,
-            &input_BaseInputFuncTable, &This->filter.csFilter, NULL, (IPin **)&This->pInputPin);
+    strmbase_sink_init(&filter->sink, &BaseRenderer_InputPin_Vtbl, &piInput,
+            &input_BaseInputFuncTable, &filter->filter.csFilter, NULL);
 
-    if (SUCCEEDED(hr))
+    hr = CreatePosPassThru(outer ? outer : (IUnknown *)&filter->filter.IBaseFilter_iface, TRUE,
+            &filter->sink.pin.IPin_iface, &filter->pPosition);
+    if (FAILED(hr))
     {
-        hr = CreatePosPassThru(pUnkOuter ? pUnkOuter: (IUnknown *)&This->filter.IBaseFilter_iface, TRUE,
-                &This->pInputPin->pin.IPin_iface, &This->pPosition);
-        if (FAILED(hr))
-            return hr;
-
-        InitializeCriticalSection(&This->csRenderLock);
-        This->csRenderLock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__": BaseRenderer.csRenderLock");
-        This->evComplete = CreateEventW(NULL, TRUE, TRUE, NULL);
-        This->ThreadSignal = CreateEventW(NULL, TRUE, TRUE, NULL);
-        This->RenderEvent = CreateEventW(NULL, FALSE, FALSE, NULL);
-        This->pMediaSample = NULL;
-
-        QualityControlImpl_Create(&This->pInputPin->pin.IPin_iface, &This->filter.IBaseFilter_iface, &This->qcimpl);
-        This->qcimpl->IQualityControl_iface.lpVtbl = &Renderer_QualityControl_Vtbl;
+        strmbase_sink_cleanup(&filter->sink);
+        strmbase_filter_cleanup(&filter->filter);
+        return hr;
     }
 
-    return hr;
-}
+    InitializeCriticalSection(&filter->csRenderLock);
+    filter->csRenderLock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__": BaseRenderer.csRenderLock");
+    filter->state_event = CreateEventW(NULL, TRUE, TRUE, NULL);
+    filter->advise_event = CreateEventW(NULL, FALSE, FALSE, NULL);
+    filter->flush_event = CreateEventW(NULL, TRUE, TRUE, NULL);
+    filter->pMediaSample = NULL;
 
-HRESULT WINAPI BaseRendererImpl_QueryInterface(IBaseFilter* iface, REFIID riid, LPVOID * ppv)
-{
-    BaseRenderer *This = impl_from_IBaseFilter(iface);
+    QualityControlImpl_Create(&filter->sink.pin.IPin_iface, &filter->filter.IBaseFilter_iface, &filter->qcimpl);
+    filter->qcimpl->IQualityControl_iface.lpVtbl = &Renderer_QualityControl_Vtbl;
 
-    if (IsEqualIID(riid, &IID_IMediaSeeking) || IsEqualIID(riid, &IID_IMediaPosition))
-        return IUnknown_QueryInterface(This->pPosition, riid, ppv);
-    else if (IsEqualIID(riid, &IID_IQualityControl))
-    {
-        *ppv = &This->qcimpl->IQualityControl_iface;
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-    else
-        return BaseFilterImpl_QueryInterface(iface, riid, ppv);
+    return S_OK;
 }
 
 void strmbase_renderer_cleanup(BaseRenderer *filter)
 {
-    IPin *peer;
-
-    if (SUCCEEDED(IPin_ConnectedTo(&filter->pInputPin->pin.IPin_iface, &peer)))
-    {
-        IPin_Disconnect(peer);
-        IPin_Release(peer);
-    }
-    IPin_Disconnect(&filter->pInputPin->pin.IPin_iface);
-    IPin_Release(&filter->pInputPin->pin.IPin_iface);
+    if (filter->sink.pin.pConnectedTo)
+        IPin_Disconnect(filter->sink.pin.pConnectedTo);
+    IPin_Disconnect(&filter->sink.pin.IPin_iface);
+    strmbase_sink_cleanup(&filter->sink);
 
     if (filter->pPosition)
         IUnknown_Release(filter->pPosition);
@@ -294,9 +286,9 @@ void strmbase_renderer_cleanup(BaseRenderer *filter)
     DeleteCriticalSection(&filter->csRenderLock);
 
     BaseRendererImpl_ClearPendingSample(filter);
-    CloseHandle(filter->evComplete);
-    CloseHandle(filter->ThreadSignal);
-    CloseHandle(filter->RenderEvent);
+    CloseHandle(filter->state_event);
+    CloseHandle(filter->advise_event);
+    CloseHandle(filter->flush_event);
     QualityControlImpl_Destroy(filter->qcimpl);
     strmbase_filter_cleanup(&filter->filter);
 }
@@ -309,7 +301,7 @@ HRESULT WINAPI BaseRendererImpl_Receive(BaseRenderer *This, IMediaSample * pSamp
 
     TRACE("(%p)->%p\n", This, pSample);
 
-    if (This->pInputPin->end_of_stream || This->pInputPin->flushing)
+    if (This->sink.end_of_stream || This->sink.flushing)
         return S_FALSE;
 
     if (This->filter.state == State_Stopped)
@@ -337,20 +329,12 @@ HRESULT WINAPI BaseRendererImpl_Receive(BaseRenderer *This, IMediaSample * pSamp
             return hr;
     }
 
-    if (This->pFuncsTable->pfnPrepareRender)
-        This->pFuncsTable->pfnPrepareRender(This);
-
     EnterCriticalSection(&This->csRenderLock);
-    if ( This->filter.state == State_Paused )
-    {
-        if (This->pFuncsTable->pfnOnReceiveFirstSample)
-            This->pFuncsTable->pfnOnReceiveFirstSample(This, pSample);
-
-        SetEvent(This->evComplete);
-    }
+    if (This->filter.state == State_Paused)
+        SetEvent(This->state_event);
 
     /* Wait for render Time */
-    if (SUCCEEDED(IMediaSample_GetTime(pSample, &start, &stop)))
+    if (This->filter.pClock && SUCCEEDED(IMediaSample_GetTime(pSample, &start, &stop)))
     {
         hr = S_FALSE;
         RendererPosPassThru_RegisterMediaTime(This->pPosition, start);
@@ -361,15 +345,25 @@ HRESULT WINAPI BaseRendererImpl_Receive(BaseRenderer *This, IMediaSample * pSamp
             ;/* Do not wait: drop through */
         else if (hr == S_FALSE)
         {
-            if (This->pFuncsTable->pfnOnWaitStart)
-                This->pFuncsTable->pfnOnWaitStart(This);
+            REFERENCE_TIME now;
+            DWORD_PTR cookie;
 
-            LeaveCriticalSection(&This->csRenderLock);
-            hr = QualityControlRender_WaitFor(This->qcimpl, pSample, This->RenderEvent);
-            EnterCriticalSection(&This->csRenderLock);
+            IReferenceClock_GetTime(This->filter.pClock, &now);
 
-            if (This->pFuncsTable->pfnOnWaitEnd)
-                This->pFuncsTable->pfnOnWaitEnd(This);
+            if (now - This->filter.rtStreamStart - start <= -10000)
+            {
+                HANDLE handles[2] = {This->advise_event, This->flush_event};
+
+                IReferenceClock_AdviseTime(This->filter.pClock, This->filter.rtStreamStart,
+                        start, (HEVENT)This->advise_event, &cookie);
+
+                LeaveCriticalSection(&This->csRenderLock);
+
+                WaitForMultipleObjects(2, handles, FALSE, INFINITE);
+                IReferenceClock_Unadvise(This->filter.pClock, cookie);
+
+                EnterCriticalSection(&This->csRenderLock);
+            }
         }
         else
         {
@@ -378,10 +372,12 @@ HRESULT WINAPI BaseRendererImpl_Receive(BaseRenderer *This, IMediaSample * pSamp
             return S_OK;
         }
     }
+    else
+        start = stop = -1;
 
     if (SUCCEEDED(hr))
     {
-        QualityControlRender_BeginRender(This->qcimpl);
+        QualityControlRender_BeginRender(This->qcimpl, start, stop);
         hr = This->pFuncsTable->pfnDoRenderSample(This, pSample);
         QualityControlRender_EndRender(This->qcimpl);
     }
@@ -403,12 +399,11 @@ HRESULT WINAPI BaseRendererImpl_Stop(IBaseFilter * iface)
     EnterCriticalSection(&This->csRenderLock);
     {
         RendererPosPassThru_ResetMediaTime(This->pPosition);
-        if (This->pFuncsTable->pfnOnStopStreaming)
-            This->pFuncsTable->pfnOnStopStreaming(This);
+        if (This->pFuncsTable->renderer_stop_stream)
+            This->pFuncsTable->renderer_stop_stream(This);
         This->filter.state = State_Stopped;
-        SetEvent(This->evComplete);
-        SetEvent(This->ThreadSignal);
-        SetEvent(This->RenderEvent);
+        SetEvent(This->state_event);
+        SetEvent(This->flush_event);
     }
     LeaveCriticalSection(&This->csRenderLock);
 
@@ -426,12 +421,11 @@ HRESULT WINAPI BaseRendererImpl_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
     if (This->filter.state == State_Running)
         goto out;
 
-    SetEvent(This->evComplete);
-    ResetEvent(This->ThreadSignal);
+    SetEvent(This->state_event);
 
-    if (This->pInputPin->pin.pConnectedTo)
+    if (This->sink.pin.pConnectedTo)
     {
-        This->pInputPin->end_of_stream = FALSE;
+        This->sink.end_of_stream = FALSE;
     }
     else if (This->filter.filterInfo.pGraph)
     {
@@ -447,11 +441,10 @@ HRESULT WINAPI BaseRendererImpl_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
     if (SUCCEEDED(hr))
     {
         QualityControlRender_Start(This->qcimpl, This->filter.rtStreamStart);
-        if (This->pFuncsTable->pfnOnStartStreaming)
-            This->pFuncsTable->pfnOnStartStreaming(This);
+        if (This->pFuncsTable->renderer_start_stream)
+            This->pFuncsTable->renderer_start_stream(This);
         if (This->filter.state == State_Stopped)
             BaseRendererImpl_ClearPendingSample(This);
-        SetEvent(This->RenderEvent);
         This->filter.state = State_Running;
     }
 out:
@@ -472,20 +465,19 @@ HRESULT WINAPI BaseRendererImpl_Pause(IBaseFilter * iface)
         {
             if (This->filter.state == State_Stopped)
             {
-                if (This->pInputPin->pin.pConnectedTo)
-                    ResetEvent(This->evComplete);
-                This->pInputPin->end_of_stream = FALSE;
+                if (This->sink.pin.pConnectedTo)
+                    ResetEvent(This->state_event);
+                This->sink.end_of_stream = FALSE;
             }
-            else if (This->pFuncsTable->pfnOnStopStreaming)
-                This->pFuncsTable->pfnOnStopStreaming(This);
+            else if (This->pFuncsTable->renderer_stop_stream)
+                This->pFuncsTable->renderer_stop_stream(This);
 
             if (This->filter.state == State_Stopped)
                 BaseRendererImpl_ClearPendingSample(This);
-            ResetEvent(This->RenderEvent);
+            ResetEvent(This->flush_event);
             This->filter.state = State_Paused;
         }
     }
-    ResetEvent(This->ThreadSignal);
     LeaveCriticalSection(&This->csRenderLock);
 
     return S_OK;
@@ -511,7 +503,7 @@ HRESULT WINAPI BaseRendererImpl_GetState(IBaseFilter * iface, DWORD dwMilliSecsT
 
     TRACE("(%p)->(%d, %p)\n", This, dwMilliSecsTimeout, pState);
 
-    if (WaitForSingleObject(This->evComplete, dwMilliSecsTimeout) == WAIT_TIMEOUT)
+    if (WaitForSingleObject(This->state_event, dwMilliSecsTimeout) == WAIT_TIMEOUT)
         hr = VFW_S_STATE_INTERMEDIATE;
     else
         hr = S_OK;
@@ -539,7 +531,7 @@ HRESULT WINAPI BaseRendererImpl_EndOfStream(BaseRenderer* iface)
         }
     }
     RendererPosPassThru_EOS(iface->pPosition);
-    SetEvent(iface->evComplete);
+    SetEvent(iface->state_event);
 
     return hr;
 }
@@ -548,8 +540,7 @@ HRESULT WINAPI BaseRendererImpl_BeginFlush(BaseRenderer* iface)
 {
     TRACE("(%p)\n", iface);
     BaseRendererImpl_ClearPendingSample(iface);
-    SetEvent(iface->ThreadSignal);
-    SetEvent(iface->RenderEvent);
+    SetEvent(iface->flush_event);
     return S_OK;
 }
 
@@ -558,8 +549,7 @@ HRESULT WINAPI BaseRendererImpl_EndFlush(BaseRenderer* iface)
     TRACE("(%p)\n", iface);
     QualityControlRender_Start(iface->qcimpl, iface->filter.rtStreamStart);
     RendererPosPassThru_ResetMediaTime(iface->pPosition);
-    ResetEvent(iface->ThreadSignal);
-    ResetEvent(iface->RenderEvent);
+    ResetEvent(iface->flush_event);
     return S_OK;
 }
 
