@@ -30,21 +30,6 @@ static const IPinVtbl TransformFilter_InputPin_Vtbl;
 static const IPinVtbl TransformFilter_OutputPin_Vtbl;
 static const IQualityControlVtbl TransformFilter_QualityControl_Vtbl;
 
-static inline BaseInputPin *impl_BaseInputPin_from_BasePin( BasePin *iface )
-{
-    return CONTAINING_RECORD(iface, BaseInputPin, pin);
-}
-
-static inline BaseInputPin *impl_BaseInputPin_from_IPin( IPin *iface )
-{
-    return CONTAINING_RECORD(iface, BaseInputPin, pin.IPin_iface);
-}
-
-static inline BaseOutputPin *impl_BaseOutputPin_from_IPin( IPin *iface )
-{
-    return CONTAINING_RECORD(iface, BaseOutputPin, pin.IPin_iface);
-}
-
 static inline TransformFilter *impl_from_IBaseFilter( IBaseFilter *iface )
 {
     return CONTAINING_RECORD(iface, TransformFilter, filter.IBaseFilter_iface);
@@ -55,13 +40,16 @@ static inline TransformFilter *impl_from_BaseFilter( BaseFilter *iface )
     return CONTAINING_RECORD(iface, TransformFilter, filter);
 }
 
+static inline TransformFilter *impl_from_sink_IPin(IPin *iface)
+{
+    return CONTAINING_RECORD(iface, TransformFilter, sink.pin.IPin_iface);
+}
+
 static HRESULT WINAPI TransformFilter_Input_CheckMediaType(BasePin *iface, const AM_MEDIA_TYPE * pmt)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_BasePin(iface);
-    TransformFilter * pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(&iface->IPin_iface);
 
     TRACE("%p\n", iface);
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
 
     if (pTransform->pFuncsTable->pfnCheckInputType)
         return pTransform->pFuncsTable->pfnCheckInputType(pTransform, pmt);
@@ -72,10 +60,10 @@ static HRESULT WINAPI TransformFilter_Input_CheckMediaType(BasePin *iface, const
 
 static HRESULT WINAPI TransformFilter_Input_Receive(BaseInputPin *This, IMediaSample *pInSample)
 {
+    TransformFilter *pTransform = impl_from_sink_IPin(&This->pin.IPin_iface);
     HRESULT hr;
-    TransformFilter * pTransform;
+
     TRACE("%p\n", This);
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
 
     EnterCriticalSection(&pTransform->csReceive);
     if (pTransform->filter.state == State_Stopped)
@@ -99,9 +87,14 @@ static HRESULT WINAPI TransformFilter_Input_Receive(BaseInputPin *This, IMediaSa
     return hr;
 }
 
+static inline TransformFilter *impl_from_source_IPin(IPin *iface)
+{
+    return CONTAINING_RECORD(iface, TransformFilter, source.pin.IPin_iface);
+}
+
 static HRESULT WINAPI TransformFilter_Output_CheckMediaType(BasePin *This, const AM_MEDIA_TYPE *pmt)
 {
-    TransformFilter *pTransformFilter = impl_from_IBaseFilter(This->pinInfo.pFilter);
+    TransformFilter *pTransformFilter = impl_from_source_IPin(&This->IPin_iface);
     AM_MEDIA_TYPE* outpmt = &pTransformFilter->pmt;
 
     if (IsEqualIID(&pmt->majortype, &outpmt->majortype)
@@ -112,13 +105,13 @@ static HRESULT WINAPI TransformFilter_Output_CheckMediaType(BasePin *This, const
 
 static HRESULT WINAPI TransformFilter_Output_DecideBufferSize(BaseOutputPin *This, IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest)
 {
-    TransformFilter *pTransformFilter = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
+    TransformFilter *pTransformFilter = impl_from_source_IPin(&This->pin.IPin_iface);
     return pTransformFilter->pFuncsTable->pfnDecideBufferSize(pTransformFilter, pAlloc, ppropInputRequest);
 }
 
 static HRESULT WINAPI TransformFilter_Output_GetMediaType(BasePin *This, int iPosition, AM_MEDIA_TYPE *pmt)
 {
-    TransformFilter *pTransform = impl_from_IBaseFilter(This->pinInfo.pFilter);
+    TransformFilter *pTransform = impl_from_source_IPin(&This->IPin_iface);
 
     if (iPosition < 0)
         return E_INVALIDARG;
@@ -132,15 +125,40 @@ static IPin *transform_get_pin(BaseFilter *iface, unsigned int index)
 {
     TransformFilter *filter = impl_from_BaseFilter(iface);
 
-    if (index >= 2)
-        return NULL;
+    if (index == 0)
+        return &filter->sink.pin.IPin_iface;
+    else if (index == 1)
+        return &filter->source.pin.IPin_iface;
+    return NULL;
+}
 
-    IPin_AddRef(filter->ppPins[index]);
-    return filter->ppPins[index];
+static void transform_destroy(BaseFilter *iface)
+{
+    TransformFilter *filter = impl_from_BaseFilter(iface);
+
+    if (filter->sink.pin.pConnectedTo)
+        IPin_Disconnect(filter->sink.pin.pConnectedTo);
+    IPin_Disconnect(&filter->sink.pin.IPin_iface);
+
+    if (filter->source.pin.pConnectedTo)
+        IPin_Disconnect(filter->source.pin.pConnectedTo);
+    IPin_Disconnect(&filter->source.pin.IPin_iface);
+
+    strmbase_sink_cleanup(&filter->sink);
+    strmbase_source_cleanup(&filter->source);
+
+    filter->csReceive.DebugInfo->Spare[0] = 0;
+    DeleteCriticalSection(&filter->csReceive);
+    FreeMediaType(&filter->pmt);
+    QualityControlImpl_Destroy(filter->qcimpl);
+    IUnknown_Release(filter->seekthru_unk);
+    strmbase_filter_cleanup(&filter->filter);
+    CoTaskMemFree(filter);
 }
 
 static const BaseFilterFuncTable tfBaseFuncTable = {
     .filter_get_pin = transform_get_pin,
+    .filter_destroy = transform_destroy,
 };
 
 static const BaseInputPinFuncTable tf_input_BaseInputFuncTable = {
@@ -161,45 +179,6 @@ static const BaseOutputPinFuncTable tf_output_BaseOutputFuncTable = {
     BaseOutputPinImpl_DecideAllocator,
 };
 
-static ULONG WINAPI TransformFilterImpl_Release(IBaseFilter * iface)
-{
-    TransformFilter *This = impl_from_IBaseFilter(iface);
-    ULONG refCount = InterlockedDecrement(&This->filter.refCount);
-
-    TRACE("(%p/%p)->() Release from %d\n", This, iface, refCount + 1);
-
-    if (!refCount)
-    {
-        ULONG i;
-
-        for (i = 0; i < 2; i++)
-        {
-            IPin *pConnectedTo;
-
-            if (SUCCEEDED(IPin_ConnectedTo(This->ppPins[i], &pConnectedTo)))
-            {
-                IPin_Disconnect(pConnectedTo);
-                IPin_Release(pConnectedTo);
-            }
-            IPin_Disconnect(This->ppPins[i]);
-
-            IPin_Release(This->ppPins[i]);
-        }
-
-        CoTaskMemFree(This->ppPins);
-
-        TRACE("Destroying transform filter\n");
-        This->csReceive.DebugInfo->Spare[0] = 0;
-        DeleteCriticalSection(&This->csReceive);
-        FreeMediaType(&This->pmt);
-        QualityControlImpl_Destroy(This->qcimpl);
-        IUnknown_Release(This->seekthru_unk);
-        strmbase_filter_cleanup(&This->filter);
-        CoTaskMemFree(This);
-    }
-    return refCount;
-}
-
 static HRESULT WINAPI TransformFilterImpl_Stop(IBaseFilter *iface)
 {
     TransformFilter *This = impl_from_IBaseFilter(iface);
@@ -213,7 +192,7 @@ static HRESULT WINAPI TransformFilterImpl_Stop(IBaseFilter *iface)
         if (This->pFuncsTable->pfnStopStreaming)
             hr = This->pFuncsTable->pfnStopStreaming(This);
         if (SUCCEEDED(hr))
-            hr = BaseOutputPinImpl_Inactive(impl_BaseOutputPin_from_IPin(This->ppPins[1]));
+            hr = BaseOutputPinImpl_Inactive(&This->source);
     }
     LeaveCriticalSection(&This->csReceive);
 
@@ -253,11 +232,11 @@ static HRESULT WINAPI TransformFilterImpl_Run(IBaseFilter *iface, REFERENCE_TIME
     {
         if (This->filter.state == State_Stopped)
         {
-            impl_BaseInputPin_from_IPin(This->ppPins[0])->end_of_stream = FALSE;
+            This->sink.end_of_stream = FALSE;
             if (This->pFuncsTable->pfnStartStreaming)
                 hr = This->pFuncsTable->pfnStartStreaming(This);
             if (SUCCEEDED(hr))
-                hr = BaseOutputPinImpl_Active(impl_BaseOutputPin_from_IPin(This->ppPins[1]));
+                hr = BaseOutputPinImpl_Active(&This->source);
         }
 
         if (SUCCEEDED(hr))
@@ -275,7 +254,7 @@ static const IBaseFilterVtbl transform_vtbl =
 {
     BaseFilterImpl_QueryInterface,
     BaseFilterImpl_AddRef,
-    TransformFilterImpl_Release,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     TransformFilterImpl_Stop,
     TransformFilterImpl_Pause,
@@ -290,14 +269,15 @@ static const IBaseFilterVtbl transform_vtbl =
     BaseFilterImpl_QueryVendorInfo
 };
 
-static HRESULT strmbase_transform_init(const CLSID *clsid,
+static HRESULT strmbase_transform_init(IUnknown *outer, const CLSID *clsid,
         const TransformFilterFuncTable *func_table, TransformFilter *filter)
 {
+    ISeekingPassThru *passthru;
     HRESULT hr;
     PIN_INFO piInput;
     PIN_INFO piOutput;
 
-    BaseFilter_Init(&filter->filter, &transform_vtbl, clsid,
+    strmbase_filter_init(&filter->filter, &transform_vtbl, outer, clsid,
             (DWORD_PTR)(__FILE__ ": TransformFilter.csFilter"), &tfBaseFuncTable);
 
     InitializeCriticalSection(&filter->csReceive);
@@ -307,8 +287,6 @@ static HRESULT strmbase_transform_init(const CLSID *clsid,
     filter->pFuncsTable = func_table;
     ZeroMemory(&filter->pmt, sizeof(filter->pmt));
 
-    filter->ppPins = CoTaskMemAlloc(2 * sizeof(IPin *));
-
     /* construct input pin */
     piInput.dir = PINDIR_INPUT;
     piInput.pFilter = &filter->filter.IBaseFilter_iface;
@@ -317,48 +295,37 @@ static HRESULT strmbase_transform_init(const CLSID *clsid,
     piOutput.pFilter = &filter->filter.IBaseFilter_iface;
     lstrcpynW(piOutput.achName, wcsOutputPinName, ARRAY_SIZE(piOutput.achName));
 
-    hr = BaseInputPin_Construct(&TransformFilter_InputPin_Vtbl, sizeof(BaseInputPin), &piInput,
-            &tf_input_BaseInputFuncTable, &filter->filter.csFilter, NULL, &filter->ppPins[0]);
+    strmbase_sink_init(&filter->sink, &TransformFilter_InputPin_Vtbl, &piInput,
+            &tf_input_BaseInputFuncTable, &filter->filter.csFilter, NULL);
 
+    strmbase_source_init(&filter->source, &TransformFilter_OutputPin_Vtbl,
+            &piOutput, &tf_output_BaseOutputFuncTable, &filter->filter.csFilter);
+
+    QualityControlImpl_Create(&filter->sink.pin.IPin_iface,
+            &filter->filter.IBaseFilter_iface, &filter->qcimpl);
+    filter->qcimpl->IQualityControl_iface.lpVtbl = &TransformFilter_QualityControl_Vtbl;
+
+    filter->seekthru_unk = NULL;
+    hr = CoCreateInstance(&CLSID_SeekingPassThru, (IUnknown *)&filter->filter.IBaseFilter_iface,
+            CLSCTX_INPROC_SERVER, &IID_IUnknown, (void **)&filter->seekthru_unk);
     if (SUCCEEDED(hr))
     {
-        hr = BaseOutputPin_Construct(&TransformFilter_OutputPin_Vtbl, sizeof(BaseOutputPin),
-                &piOutput, &tf_output_BaseOutputFuncTable, &filter->filter.csFilter,
-                &filter->ppPins[1]);
-
-        if (FAILED(hr))
-            ERR("Cannot create output pin (%x)\n", hr);
-        else {
-            QualityControlImpl_Create(filter->ppPins[0],
-                    &filter->filter.IBaseFilter_iface, &filter->qcimpl);
-            filter->qcimpl->IQualityControl_iface.lpVtbl = &TransformFilter_QualityControl_Vtbl;
-        }
-    }
-
-    if (SUCCEEDED(hr))
-    {
-        ISeekingPassThru *passthru;
-        filter->seekthru_unk = NULL;
-        hr = CoCreateInstance(&CLSID_SeekingPassThru, (IUnknown *)&filter->filter.IBaseFilter_iface,
-                CLSCTX_INPROC_SERVER, &IID_IUnknown, (void **)&filter->seekthru_unk);
-        if (SUCCEEDED(hr))
-        {
-            IUnknown_QueryInterface(filter->seekthru_unk, &IID_ISeekingPassThru, (void **)&passthru);
-            ISeekingPassThru_Init(passthru, FALSE, filter->ppPins[0]);
-            ISeekingPassThru_Release(passthru);
-        }
+        IUnknown_QueryInterface(filter->seekthru_unk, &IID_ISeekingPassThru, (void **)&passthru);
+        ISeekingPassThru_Init(passthru, FALSE, &filter->sink.pin.IPin_iface);
+        ISeekingPassThru_Release(passthru);
     }
 
     if (FAILED(hr))
     {
-        CoTaskMemFree(filter->ppPins);
+        strmbase_sink_cleanup(&filter->sink);
+        strmbase_source_cleanup(&filter->source);
         strmbase_filter_cleanup(&filter->filter);
     }
 
     return hr;
 }
 
-HRESULT strmbase_transform_create(LONG filter_size, const CLSID *pClsid,
+HRESULT strmbase_transform_create(LONG filter_size, IUnknown *outer, const CLSID *pClsid,
         const TransformFilterFuncTable *pFuncsTable, IBaseFilter **ppTransformFilter)
 {
     TransformFilter* pTf;
@@ -374,7 +341,7 @@ HRESULT strmbase_transform_create(LONG filter_size, const CLSID *pClsid,
 
     ZeroMemory(pTf, filter_size);
 
-    if (SUCCEEDED(strmbase_transform_init(pClsid, pFuncsTable, pTf)))
+    if (SUCCEEDED(strmbase_transform_init(outer, pClsid, pFuncsTable, pTf)))
     {
         *ppTransformFilter = &pTf->filter.IBaseFilter_iface;
         return S_OK;
@@ -389,44 +356,24 @@ HRESULT WINAPI TransformFilterImpl_Notify(TransformFilter *iface, IBaseFilter *s
     return QualityControlImpl_Notify((IQualityControl*)iface->qcimpl, sender, qm);
 }
 
-/** IBaseFilter implementation **/
-
 static HRESULT WINAPI TransformFilter_InputPin_EndOfStream(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
-    IPin* ppin;
-    HRESULT hr;
+    TransformFilter *filter = impl_from_sink_IPin(iface);
 
-    TRACE("(%p)->()\n", iface);
+    TRACE("iface %p.\n", iface);
 
-    /* Since we process samples synchronously, just forward notification downstream */
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
-    if (!pTransform)
-        hr = E_FAIL;
-    else
-        hr = IPin_ConnectedTo(pTransform->ppPins[1], &ppin);
-    if (SUCCEEDED(hr))
-    {
-        hr = IPin_EndOfStream(ppin);
-        IPin_Release(ppin);
-    }
-
-    if (FAILED(hr))
-        ERR("%x\n", hr);
-    return hr;
+    if (filter->source.pin.pConnectedTo)
+        return IPin_EndOfStream(filter->source.pin.pConnectedTo);
+    return VFW_E_NOT_CONNECTED;
 }
 
 static HRESULT WINAPI TransformFilter_InputPin_ReceiveConnection(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(iface);
     HRESULT hr = S_OK;
 
     TRACE("(%p)->(%p, %p)\n", iface, pReceivePin, pmt);
 
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
-
     if (pTransform->pFuncsTable->pfnSetMediaType)
         hr = pTransform->pFuncsTable->pfnSetMediaType(pTransform, PINDIR_INPUT, pmt);
 
@@ -445,12 +392,10 @@ static HRESULT WINAPI TransformFilter_InputPin_ReceiveConnection(IPin * iface, I
 
 static HRESULT WINAPI TransformFilter_InputPin_Disconnect(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(iface);
 
     TRACE("(%p)->()\n", iface);
 
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
     if (pTransform->pFuncsTable->pfnBreakConnect)
         pTransform->pFuncsTable->pfnBreakConnect(pTransform, PINDIR_INPUT);
 
@@ -459,13 +404,11 @@ static HRESULT WINAPI TransformFilter_InputPin_Disconnect(IPin * iface)
 
 static HRESULT WINAPI TransformFilter_InputPin_BeginFlush(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(iface);
     HRESULT hr = S_OK;
 
     TRACE("(%p)->()\n", iface);
 
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
     EnterCriticalSection(&pTransform->filter.csFilter);
     if (pTransform->pFuncsTable->pfnBeginFlush)
         hr = pTransform->pFuncsTable->pfnBeginFlush(pTransform);
@@ -477,13 +420,11 @@ static HRESULT WINAPI TransformFilter_InputPin_BeginFlush(IPin * iface)
 
 static HRESULT WINAPI TransformFilter_InputPin_EndFlush(IPin * iface)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(iface);
     HRESULT hr = S_OK;
 
     TRACE("(%p)->()\n", iface);
 
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
     EnterCriticalSection(&pTransform->filter.csFilter);
     if (pTransform->pFuncsTable->pfnEndFlush)
         hr = pTransform->pFuncsTable->pfnEndFlush(pTransform);
@@ -495,13 +436,11 @@ static HRESULT WINAPI TransformFilter_InputPin_EndFlush(IPin * iface)
 
 static HRESULT WINAPI TransformFilter_InputPin_NewSegment(IPin * iface, REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
 {
-    BaseInputPin* This = impl_BaseInputPin_from_IPin(iface);
-    TransformFilter* pTransform;
+    TransformFilter *pTransform = impl_from_sink_IPin(iface);
     HRESULT hr = S_OK;
 
     TRACE("(%p)->(%s %s %e)\n", iface, wine_dbgstr_longlong(tStart), wine_dbgstr_longlong(tStop), dRate);
 
-    pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
     EnterCriticalSection(&pTransform->filter.csFilter);
     if (pTransform->pFuncsTable->pfnNewSegment)
         hr = pTransform->pFuncsTable->pfnNewSegment(pTransform, tStart, tStop, dRate);
@@ -515,7 +454,7 @@ static const IPinVtbl TransformFilter_InputPin_Vtbl =
 {
     BaseInputPinImpl_QueryInterface,
     BasePinImpl_AddRef,
-    BaseInputPinImpl_Release,
+    BasePinImpl_Release,
     BaseInputPinImpl_Connect,
     TransformFilter_InputPin_ReceiveConnection,
     TransformFilter_InputPin_Disconnect,
@@ -535,7 +474,8 @@ static const IPinVtbl TransformFilter_InputPin_Vtbl =
 
 static HRESULT WINAPI transform_source_QueryInterface(IPin *iface, REFIID iid, void **out)
 {
-    TransformFilter *filter = impl_from_IBaseFilter(impl_BaseOutputPin_from_IPin(iface)->pin.pinInfo.pFilter);
+    TransformFilter *filter = impl_from_source_IPin(iface);
+
     if (IsEqualGUID(iid, &IID_IUnknown) || IsEqualGUID(iid, &IID_IPin))
         *out = iface;
     else if (IsEqualGUID(iid, &IID_IQualityControl))
@@ -557,7 +497,7 @@ static const IPinVtbl TransformFilter_OutputPin_Vtbl =
 {
     transform_source_QueryInterface,
     BasePinImpl_AddRef,
-    BaseOutputPinImpl_Release,
+    BasePinImpl_Release,
     BaseOutputPinImpl_Connect,
     BaseOutputPinImpl_ReceiveConnection,
     BaseOutputPinImpl_Disconnect,
