@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <math.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -66,20 +67,6 @@ DEFINE_EXPECT(invalid_parameter_handler);
 #define UCRTBASE_PRINTF_LEGACY_MSVCRT_COMPATIBILITY      (0x0008)
 #define UCRTBASE_PRINTF_LEGACY_THREE_DIGIT_EXPONENTS     (0x0010)
 
-static inline float __port_infinity(void)
-{
-    static const unsigned __inf_bytes = 0x7f800000;
-    return *(const float *)&__inf_bytes;
-}
-#define INFINITY __port_infinity()
-
-static inline float __port_nan(void)
-{
-    static const unsigned __nan_bytes = 0x7fc00000;
-    return *(const float *)&__nan_bytes;
-}
-#define NAN __port_nan()
-
 static inline float __port_ind(void)
 {
     static const unsigned __ind_bytes = 0xffc00000;
@@ -171,7 +158,7 @@ static void test_snprintf (void)
     unsigned int i;
 
     /* Legacy _snprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const char *fmt  = tests[i];
         const int expect = strlen(fmt) > bufsiz ? -1 : strlen(fmt);
         const int n      = vsprintf_wrapper (UCRTBASE_PRINTF_LEGACY_VSPRINTF_NULL_TERMINATION, buffer, bufsiz, fmt);
@@ -184,7 +171,7 @@ static void test_snprintf (void)
     }
 
     /* C99 snprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const char *fmt  = tests[i];
         const int expect = strlen(fmt);
         const int n      = vsprintf_wrapper (UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR, buffer, bufsiz, fmt);
@@ -199,7 +186,7 @@ static void test_snprintf (void)
     }
 
     /* swprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const char *fmt  = tests[i];
         const int expect = strlen(fmt) >= bufsiz ? -2 : strlen(fmt);
         const int n      = vsprintf_wrapper (0, buffer, bufsiz, fmt);
@@ -215,6 +202,10 @@ static void test_snprintf (void)
 
     ok (vsprintf_wrapper (UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR, NULL, 0, "abcd") == 4,
         "Failure to snprintf to NULL\n");
+    ok (vsprintf_wrapper (UCRTBASE_PRINTF_LEGACY_VSPRINTF_NULL_TERMINATION, NULL, 0, "abcd") == 4,
+        "Failure to snprintf to NULL\n");
+    ok (vsprintf_wrapper (0, NULL, 0, "abcd") == 4,
+        "Failure to snprintf to NULL\n");
 }
 
 static int WINAPIV vswprintf_wrapper(unsigned __int64 options, wchar_t *str,
@@ -238,11 +229,11 @@ static void test_swprintf (void)
 
     wchar_t buffer[8];
     char narrow[8], narrow_fmt[16];
-    const int bufsiz = sizeof buffer / sizeof buffer[0];
+    const int bufsiz = ARRAY_SIZE(buffer);
     unsigned int i;
 
     /* Legacy _snprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const wchar_t *fmt = tests[i];
         const int expect   = wcslen(fmt) > bufsiz ? -1 : wcslen(fmt);
         const int n        = vswprintf_wrapper (UCRTBASE_PRINTF_LEGACY_VSPRINTF_NULL_TERMINATION, buffer, bufsiz, fmt);
@@ -257,7 +248,7 @@ static void test_swprintf (void)
     }
 
     /* C99 snprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const wchar_t *fmt = tests[i];
         const int expect   = wcslen(fmt);
         const int n        = vswprintf_wrapper (UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR, buffer, bufsiz, fmt);
@@ -274,7 +265,7 @@ static void test_swprintf (void)
     }
 
     /* swprintf style termination */
-    for (i = 0; i < sizeof tests / sizeof tests[0]; i++) {
+    for (i = 0; i < ARRAY_SIZE(tests); i++) {
         const wchar_t *fmt = tests[i];
         const int expect   = wcslen(fmt) >= bufsiz ? -2 : wcslen(fmt);
         const int n        = vswprintf_wrapper (0, buffer, bufsiz, fmt);
@@ -289,6 +280,13 @@ static void test_swprintf (void)
         ok (buffer[valid] == '\0',
             "\"%s\": Missing null termination (ret %d) - is %d\n", narrow_fmt, n, buffer[valid]);
     }
+
+    ok (vswprintf_wrapper (UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR, NULL, 0, str_short) == 5,
+        "Failure to swprintf to NULL\n");
+    ok (vswprintf_wrapper (UCRTBASE_PRINTF_LEGACY_VSPRINTF_NULL_TERMINATION, NULL, 0, str_short) == 5,
+        "Failure to swprintf to NULL\n");
+    ok (vswprintf_wrapper (0, NULL, 0, str_short) == 5,
+        "Failure to swprintf to NULL\n");
 }
 
 static int WINAPIV vfprintf_wrapper(FILE *file,
@@ -400,12 +398,12 @@ static void test_fwprintf(void)
     p_fclose(fp);
 
     fp = p_fopen(file_name, "rb");
-    p_fgetws(bufw, sizeof(bufw)/sizeof(bufw[0]), fp);
+    p_fgetws(bufw, ARRAY_SIZE(bufw), fp);
     ret = p_ftell(fp);
     ok(ret == 24, "ftell returned %d\n", ret);
     ok(!wcscmp(bufw, simple), "buf = %s\n", wine_dbgstr_w(bufw));
 
-    p_fgetws(bufw, sizeof(bufw)/sizeof(bufw[0]), fp);
+    p_fgetws(bufw, ARRAY_SIZE(bufw), fp);
     ret = p_ftell(fp);
     ok(ret == 52, "ret = %d\n", ret);
     ok(!memcmp(bufw, cont, 28), "buf = %s\n", wine_dbgstr_w(bufw));
