@@ -1008,12 +1008,12 @@ void wined3d_cs_emit_draw_indirect(struct wined3d_cs *cs, GLenum primitive_type,
 
 static void wined3d_cs_exec_flush(struct wined3d_cs *cs, const void *data)
 {
-    struct wined3d_context *context;
+    struct wined3d_context_gl *context_gl;
 
-    context = context_acquire(cs->device, NULL, 0);
-    if (context->valid)
-        context->gl_info->gl_ops.gl.p_glFlush();
-    context_release(context);
+    context_gl = wined3d_context_gl(context_acquire(cs->device, NULL, 0));
+    if (context_gl->valid)
+        context_gl->c.gl_info->gl_ops.gl.p_glFlush();
+    context_release(&context_gl->c);
 }
 
 void wined3d_cs_emit_flush(struct wined3d_cs *cs)
@@ -2228,7 +2228,8 @@ static void wined3d_cs_exec_blt_sub_resource(struct wined3d_cs *cs, const void *
                 && update_h == wined3d_texture_get_level_height(dst_texture, level)
                 && update_d == wined3d_texture_get_level_depth(dst_texture, level))
         {
-            wined3d_texture_prepare_texture(dst_texture, context, FALSE);
+            wined3d_texture_prepare_location(dst_texture, op->dst_sub_resource_idx,
+                    context, WINED3D_LOCATION_TEXTURE_RGB);
         }
         else if (!wined3d_texture_load_location(dst_texture, op->dst_sub_resource_idx,
                 context, WINED3D_LOCATION_TEXTURE_RGB))
@@ -2242,10 +2243,10 @@ static void wined3d_cs_exec_blt_sub_resource(struct wined3d_cs *cs, const void *
         wined3d_texture_get_pitch(src_texture, op->src_sub_resource_idx % src_texture->level_count,
                 &row_pitch, &slice_pitch);
 
-        wined3d_texture_gl_bind_and_dirtify(wined3d_texture_gl(dst_texture), wined3d_context_gl(context), FALSE);
-        wined3d_texture_upload_data(dst_texture, op->dst_sub_resource_idx, context,
-                dst_texture->resource.format, &op->src_box, wined3d_const_bo_address(&addr),
-                row_pitch, slice_pitch, op->dst_box.left, op->dst_box.top, op->dst_box.front, FALSE);
+        dst_texture->texture_ops->texture_upload_data(context, wined3d_const_bo_address(&addr),
+                dst_texture->resource.format, &op->src_box, row_pitch, slice_pitch, dst_texture,
+                op->dst_sub_resource_idx, WINED3D_LOCATION_TEXTURE_RGB,
+                op->dst_box.left, op->dst_box.top, op->dst_box.front);
         wined3d_texture_validate_location(dst_texture, op->dst_sub_resource_idx, WINED3D_LOCATION_TEXTURE_RGB);
         wined3d_texture_invalidate_location(dst_texture, op->dst_sub_resource_idx, ~WINED3D_LOCATION_TEXTURE_RGB);
 
@@ -2337,14 +2338,14 @@ static void wined3d_cs_exec_update_sub_resource(struct wined3d_cs *cs, const voi
     /* Only load the sub-resource for partial updates. */
     if (!box->left && !box->top && !box->front
             && box->right == width && box->bottom == height && box->back == depth)
-        wined3d_texture_prepare_texture(texture, context, FALSE);
+        wined3d_texture_prepare_location(texture, op->sub_resource_idx, context, WINED3D_LOCATION_TEXTURE_RGB);
     else
         wined3d_texture_load_location(texture, op->sub_resource_idx, context, WINED3D_LOCATION_TEXTURE_RGB);
-    wined3d_texture_gl_bind_and_dirtify(wined3d_texture_gl(texture), wined3d_context_gl(context), FALSE);
 
     wined3d_box_set(&src_box, 0, 0, box->right - box->left, box->bottom - box->top, 0, box->back - box->front);
-    wined3d_texture_upload_data(texture, op->sub_resource_idx, context, texture->resource.format, &src_box,
-            &addr, op->data.row_pitch, op->data.slice_pitch, box->left, box->top, box->front, FALSE);
+    texture->texture_ops->texture_upload_data(context, &addr, texture->resource.format, &src_box,
+            op->data.row_pitch, op->data.slice_pitch, texture, op->sub_resource_idx,
+            WINED3D_LOCATION_TEXTURE_RGB, box->left, box->top, box->front);
 
     wined3d_texture_validate_location(texture, op->sub_resource_idx, WINED3D_LOCATION_TEXTURE_RGB);
     wined3d_texture_invalidate_location(texture, op->sub_resource_idx, ~WINED3D_LOCATION_TEXTURE_RGB);
