@@ -28,9 +28,6 @@
 
 #define GL_COMPRESSED_LUMINANCE_ALPHA_3DC_ATI 0x8837  /* not in the gl spec */
 
-void (WINE_GLAPI *glDisableWINE)(GLenum cap) DECLSPEC_HIDDEN;
-void (WINE_GLAPI *glEnableWINE)(GLenum cap) DECLSPEC_HIDDEN;
-
 /* OpenGL extensions. */
 enum wined3d_gl_extension
 {
@@ -157,14 +154,15 @@ enum wined3d_gl_extension
     EXT_BLEND_FUNC_SEPARATE,
     EXT_BLEND_MINMAX,
     EXT_BLEND_SUBTRACT,
-    EXT_DRAW_BUFFERS2,
     EXT_DEPTH_BOUNDS_TEST,
+    EXT_DRAW_BUFFERS2,
     EXT_FOG_COORD,
     EXT_FRAMEBUFFER_BLIT,
     EXT_FRAMEBUFFER_MULTISAMPLE,
     EXT_FRAMEBUFFER_OBJECT,
     EXT_GPU_PROGRAM_PARAMETERS,
     EXT_GPU_SHADER4,
+    EXT_MEMORY_OBJECT,
     EXT_PACKED_DEPTH_STENCIL,
     EXT_PACKED_FLOAT,
     EXT_POINT_PARAMETERS,
@@ -185,7 +183,6 @@ enum wined3d_gl_extension
     EXT_TEXTURE_SNORM,
     EXT_TEXTURE_SRGB,
     EXT_TEXTURE_SRGB_DECODE,
-    EXT_VERTEX_ARRAY_BGRA,
     /* NVIDIA */
     NV_FENCE,
     NV_FOG_DISTANCE,
