@@ -484,11 +484,29 @@ static void init_visuals( Display *display, int screen )
     int count;
     XVisualInfo *info;
 
+    argb_visual.screen     = screen;
+    argb_visual.class      = TrueColor;
+    argb_visual.depth      = 32;
+    argb_visual.red_mask   = 0xff0000;
+    argb_visual.green_mask = 0x00ff00;
+    argb_visual.blue_mask  = 0x0000ff;
+
+    if ((info = XGetVisualInfo( display, VisualScreenMask | VisualDepthMask | VisualClassMask |
+                                VisualRedMaskMask | VisualGreenMaskMask | VisualBlueMaskMask,
+                                &argb_visual, &count )))
+    {
+        argb_visual = *info;
+        XFree( info );
+    }
+
     default_visual.screen = screen;
     if (default_visual.depth)  /* depth specified */
     {
-        info = XGetVisualInfo( display, VisualScreenMask | VisualDepthMask, &default_visual, &count );
-        if (info)
+        if (default_visual.depth == 32 && argb_visual.visual)
+        {
+            default_visual = argb_visual;
+        }
+        else if ((info = XGetVisualInfo( display, VisualScreenMask | VisualDepthMask, &default_visual, &count )))
         {
             default_visual = *info;
             XFree( info );
@@ -510,20 +528,6 @@ static void init_visuals( Display *display, int screen )
     }
     default_colormap = XCreateColormap( display, root_window, default_visual.visual, AllocNone );
 
-    argb_visual.screen     = screen;
-    argb_visual.class      = TrueColor;
-    argb_visual.depth      = 32;
-    argb_visual.red_mask   = 0xff0000;
-    argb_visual.green_mask = 0x00ff00;
-    argb_visual.blue_mask  = 0x0000ff;
-
-    if ((info = XGetVisualInfo( display, VisualScreenMask | VisualDepthMask | VisualClassMask |
-                                VisualRedMaskMask | VisualGreenMaskMask | VisualBlueMaskMask,
-                                &argb_visual, &count )))
-    {
-        argb_visual = *info;
-        XFree( info );
-    }
     TRACE( "default visual %lx class %u argb %lx\n",
            default_visual.visualid, default_visual.class, argb_visual.visualid );
 }
@@ -593,6 +597,7 @@ static BOOL process_attach(void)
     X11DRV_InitKeyboard( gdi_display );
     if (use_xim) use_xim = X11DRV_InitXIM( input_style );
 
+    X11DRV_DisplayDevices_Init(FALSE);
     return TRUE;
 }
 
