@@ -17,8 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <stdarg.h>
@@ -44,7 +42,6 @@
 #include "xinput.h"
 #include "xinput_private.h"
 
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(xinput);
 
@@ -190,7 +187,7 @@ static void build_private(struct hid_platform_private *private, PHIDP_PREPARSED_
     private->current_report = 0;
     private->reports[0] = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, private->report_length);
     private->reports[1] = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, private->report_length);
-    size = (strlenW(path) + 1) * sizeof(WCHAR);
+    size = (lstrlenW(path) + 1) * sizeof(WCHAR);
     private->device_path = HeapAlloc(GetProcessHeap(), 0, size);
     memcpy(private->device_path, path, size);
     private->enabled = TRUE;
@@ -238,13 +235,13 @@ void HID_find_gamepads(xinput_controller *devices)
                 &interface_data, data, sizeof(*data) + detail_size, NULL, NULL))
             continue;
 
-        if (!strstrW(data->DevicePath, ig))
+        if (!wcsstr(data->DevicePath, ig))
             continue;
 
         for (i = 0; i < XUSER_MAX_COUNT; i++)
         {
             struct hid_platform_private *private = devices[i].platform_private;
-            if (devices[i].connected && !strcmpW(data->DevicePath, private->device_path))
+            if (devices[i].connected && !wcscmp(data->DevicePath, private->device_path))
                 break;
         }
         if (i != XUSER_MAX_COUNT)
@@ -469,8 +466,8 @@ DWORD HID_set_state(xinput_controller* device, XINPUT_VIBRATION* state)
             report.report = 0;
             report.pad1[0] = 0x8;
             report.pad1[1] = 0x0;
-            report.left = (BYTE)(state->wLeftMotorSpeed / 255);
-            report.right = (BYTE)(state->wRightMotorSpeed / 255);
+            report.left = (BYTE)(state->wLeftMotorSpeed / 256);
+            report.right = (BYTE)(state->wRightMotorSpeed / 256);
             memset(&report.pad2, 0, sizeof(report.pad2));
 
             EnterCriticalSection(&private->crit);
