@@ -357,14 +357,12 @@ wchar_t* __cdecl getws(wchar_t*);
 wint_t   __cdecl putwc(wint_t,FILE*);
 wint_t   __cdecl putwchar(wint_t);
 int      __cdecl putws(const wchar_t*);
-int      WINAPIV swprintf(wchar_t*,const wchar_t*,...);
 int      WINAPIV swprintf_s(wchar_t*,size_t,const wchar_t*,...);
 int      WINAPIV swscanf(const wchar_t*,const wchar_t*,...);
 int      WINAPIV swscanf_s(const wchar_t*,const wchar_t*,...);
 wint_t   __cdecl ungetwc(wint_t,FILE*);
 int      __cdecl vfwprintf(FILE*,const wchar_t*,__ms_va_list);
 int      __cdecl vfwprintf_s(FILE*,const wchar_t*,__ms_va_list);
-int      __cdecl vswprintf(wchar_t*,const wchar_t*,__ms_va_list);
 int      __cdecl vswprintf_s(wchar_t*,size_t,const wchar_t*,__ms_va_list);
 int      __cdecl vwprintf(const wchar_t*,__ms_va_list);
 int      __cdecl vwprintf_s(const wchar_t*,__ms_va_list);
@@ -372,6 +370,23 @@ int      WINAPIV wprintf(const wchar_t*,...);
 int      WINAPIV wprintf_s(const wchar_t*,...);
 int      WINAPIV wscanf(const wchar_t*,...);
 int      WINAPIV wscanf_s(const wchar_t*,...);
+
+#ifdef _CRT_NON_CONFORMING_SWPRINTFS
+int WINAPIV swprintf(wchar_t*,const wchar_t*,...);
+int __cdecl vswprintf(wchar_t*,const wchar_t*,__ms_va_list);
+#else  /*  _CRT_NON_CONFORMING_SWPRINTFS */
+static inline int vswprintf(wchar_t *buffer, size_t size, const wchar_t *format, __ms_va_list args) { return _vsnwprintf(buffer,size,format,args); }
+static inline int WINAPIV swprintf(wchar_t *buffer, size_t size, const wchar_t *format, ...)
+{
+    int ret;
+    __ms_va_list args;
+
+    __ms_va_start(args, format);
+    ret = _vsnwprintf(buffer, size, format, args);
+    __ms_va_end(args);
+    return ret;
+}
+#endif  /*  _CRT_NON_CONFORMING_SWPRINTFS */
 #endif /* _WSTDIO_DEFINED */
 
 #ifndef _WSTDLIB_DEFINED
@@ -399,6 +414,7 @@ __msvcrt_long __cdecl _wtol(const wchar_t*);
 size_t        __cdecl mbstowcs(wchar_t*,const char*,size_t);
 errno_t       __cdecl mbstowcs_s(size_t*,wchar_t*,size_t,const char*,size_t);
 int           __cdecl mbtowc(wchar_t*,const char*,size_t);
+float         __cdecl wcstof(const wchar_t*,wchar_t**);
 double        __cdecl wcstod(const wchar_t*,wchar_t**);
 __msvcrt_long __cdecl wcstol(const wchar_t*,wchar_t**,int);
 size_t        __cdecl wcstombs(char*,const wchar_t*,size_t);
