@@ -36,15 +36,6 @@
 #include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_DIRECT_H
-# include <direct.h>
-#endif
-#ifdef HAVE_IO_H
-# include <io.h>
-#endif
-#ifdef HAVE_PROCESS_H
-# include <process.h>
-#endif
 #include <string.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -53,10 +44,41 @@
 
 
 /****************************************************************
- * Type definitions
+ * Hard-coded values for the Windows platform
  */
 
-#if !defined(_MSC_VER) && !defined(__int64)
+#ifdef _WIN32
+
+#include <direct.h>
+#include <io.h>
+#include <process.h>
+
+#define mkdir(path,mode) mkdir(path)
+
+#ifdef _MSC_VER
+
+#define ftruncate chsize
+#define isfinite(x) _finite(x)
+#define isinf(x) (!(_finite(x) || _isnan(x)))
+#define isnan(x) _isnan(x)
+#define popen _popen
+#define pclose _pclose
+#define snprintf _snprintf
+#define strtoll _strtoi64
+#define strtoull _strtoui64
+#define strncasecmp _strnicmp
+#define strcasecmp _stricmp
+
+typedef int mode_t;
+typedef long off_t;
+typedef int pid_t;
+typedef int ssize_t;
+
+#endif /* _MSC_VER */
+
+#else  /* _WIN32 */
+
+#ifndef __int64
 #  if defined(__x86_64__) || defined(__aarch64__) || defined(_WIN64)
 #    define __int64 long
 #  else
@@ -64,21 +86,36 @@
 #  endif
 #endif
 
-#ifndef HAVE_MODE_T
-typedef int mode_t;
+#ifndef HAVE_ISFINITE
+int isfinite(double x);
 #endif
-#ifndef HAVE_OFF_T
-typedef long off_t;
+
+#ifndef HAVE_ISINF
+int isinf(double x);
 #endif
-#ifndef HAVE_PID_T
-typedef int pid_t;
+
+#ifndef HAVE_ISNAN
+int isnan(double x);
 #endif
-#ifndef HAVE_SIZE_T
-typedef unsigned int size_t;
+
+/* Process creation flags */
+#ifndef _P_WAIT
+# define _P_WAIT    0
+# define _P_NOWAIT  1
+# define _P_OVERLAY 2
+# define _P_NOWAITO 3
+# define _P_DETACH  4
 #endif
-#ifndef HAVE_SSIZE_T
-typedef int ssize_t;
+#ifndef HAVE__SPAWNVP
+extern int _spawnvp(int mode, const char *cmdname, const char * const argv[]);
 #endif
+
+#endif  /* _WIN32 */
+
+/****************************************************************
+ * Type definitions
+ */
+
 #ifndef HAVE_FSBLKCNT_T
 typedef unsigned long fsblkcnt_t;
 #endif
@@ -116,42 +153,6 @@ struct statvfs
 #define RTLD_GLOBAL  0x100
 #endif
 
-#ifdef HAVE_ONE_ARG_MKDIR
-#define mkdir(path,mode) mkdir(path)
-#endif
-
-#if !defined(HAVE_FTRUNCATE) && defined(HAVE_CHSIZE)
-#define ftruncate chsize
-#endif
-
-#if !defined(HAVE_POPEN) && defined(HAVE__POPEN)
-#define popen _popen
-#endif
-
-#if !defined(HAVE_PCLOSE) && defined(HAVE__PCLOSE)
-#define pclose _pclose
-#endif
-
-#if !defined(HAVE_STRDUP) && defined(HAVE__STRDUP)
-#define strdup _strdup
-#endif
-
-#if !defined(HAVE_SNPRINTF) && defined(HAVE__SNPRINTF)
-#define snprintf _snprintf
-#endif
-
-#if !defined(HAVE_VSNPRINTF) && defined(HAVE__VSNPRINTF)
-#define vsnprintf _vsnprintf
-#endif
-
-#if !defined(HAVE_STRTOLL) && defined(HAVE__STRTOI64)
-#define strtoll _strtoi64
-#endif
-
-#if !defined(HAVE_STRTOULL) && defined(HAVE__STRTOUI64)
-#define strtoull _strtoui64
-#endif
-
 #ifndef S_ISLNK
 # define S_ISLNK(mod) (0)
 #endif
@@ -260,18 +261,6 @@ extern int getopt_long_only (int ___argc, char *const *___argv,
 int ffs( int x );
 #endif
 
-#ifndef HAVE_ISFINITE
-int isfinite(double x);
-#endif
-
-#ifndef HAVE_ISINF
-int isinf(double x);
-#endif
-
-#ifndef HAVE_ISNAN
-int isnan(double x);
-#endif
-
 #ifndef HAVE_LLRINT
 __int64 llrint(double x);
 #endif
@@ -292,10 +281,6 @@ long lrintf(float x);
 int lstat(const char *file_name, struct stat *buf);
 #endif /* HAVE_LSTAT */
 
-#ifndef HAVE_MEMMOVE
-void *memmove(void *dest, const void *src, size_t len);
-#endif /* !defined(HAVE_MEMMOVE) */
-
 #ifndef HAVE_POLL
 struct pollfd
 {
@@ -336,30 +321,10 @@ float rintf(float x);
 int statvfs( const char *path, struct statvfs *buf );
 #endif
 
-#ifndef HAVE_STRNCASECMP
-# ifndef HAVE__STRNICMP
-int strncasecmp(const char *str1, const char *str2, size_t n);
-# else
-# define strncasecmp _strnicmp
-# endif
-#endif /* !defined(HAVE_STRNCASECMP) */
-
 #ifndef HAVE_STRNLEN
 size_t strnlen( const char *str, size_t maxlen );
 #endif /* !defined(HAVE_STRNLEN) */
 
-#ifndef HAVE_STRERROR
-const char *strerror(int err);
-#endif /* !defined(HAVE_STRERROR) */
-
-#ifndef HAVE_STRCASECMP
-# ifndef HAVE__STRICMP
-int strcasecmp(const char *str1, const char *str2);
-# else
-# define strcasecmp _stricmp
-# endif
-#endif /* !defined(HAVE_STRCASECMP) */
-
 #ifndef HAVE_SYMLINK
 int symlink(const char *from, const char *to);
 #endif
@@ -368,29 +333,8 @@ int symlink(const char *from, const char *to);
 int usleep (unsigned int useconds);
 #endif /* !defined(HAVE_USLEEP) */
 
-#ifdef __i386__
-static inline void *memcpy_unaligned( void *dst, const void *src, size_t size )
-{
-    return memcpy( dst, src, size );
-}
-#else
-extern void *memcpy_unaligned( void *dst, const void *src, size_t size );
-#endif /* __i386__ */
-
 extern int mkstemps(char *template, int suffix_len);
 
-/* Process creation flags */
-#ifndef _P_WAIT
-# define _P_WAIT    0
-# define _P_NOWAIT  1
-# define _P_OVERLAY 2
-# define _P_NOWAITO 3
-# define _P_DETACH  4
-#endif
-#ifndef HAVE__SPAWNVP
-extern int _spawnvp(int mode, const char *cmdname, const char * const argv[]);
-#endif
-
 /* Interlocked functions */
 
 #if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__))
@@ -532,16 +476,9 @@ extern __int64 interlocked_cmpxchg64( __int64 *dest, __int64 xchg, __int64 compa
 #define interlocked_xchg_ptr    __WINE_NOT_PORTABLE(interlocked_xchg_ptr)
 #define interlocked_xchg_add    __WINE_NOT_PORTABLE(interlocked_xchg_add)
 #define lstat                   __WINE_NOT_PORTABLE(lstat)
-#define memcpy_unaligned        __WINE_NOT_PORTABLE(memcpy_unaligned)
-#undef memmove
-#define memmove                 __WINE_NOT_PORTABLE(memmove)
 #define pread                   __WINE_NOT_PORTABLE(pread)
 #define pwrite                  __WINE_NOT_PORTABLE(pwrite)
-#define spawnvp                 __WINE_NOT_PORTABLE(spawnvp)
 #define statvfs                 __WINE_NOT_PORTABLE(statvfs)
-#define strcasecmp              __WINE_NOT_PORTABLE(strcasecmp)
-#define strerror                __WINE_NOT_PORTABLE(strerror)
-#define strncasecmp             __WINE_NOT_PORTABLE(strncasecmp)
 #define strnlen                 __WINE_NOT_PORTABLE(strnlen)
 #define usleep                  __WINE_NOT_PORTABLE(usleep)
 
