@@ -51,30 +51,18 @@
  *   processed (requires translations from Unicode to Ansi).
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #define COBJMACROS
-#define WIN32_LEAN_AND_MEAN
 
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_GETOPT_H
-# include <getopt.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <sys/stat.h>
+#include <unistd.h>
 #include <windows.h>
 #include <winternl.h>
 #include <wine/svcctl.h>
-#include <wine/unicode.h>
-#include <wine/library.h>
+#include <wine/asm.h>
 #include <wine/debug.h>
 
 #include <shlobj.h>
@@ -90,43 +78,54 @@ extern BOOL shutdown_all_desktops( BOOL force );
 extern void kill_processes( BOOL kill_desktop );
 
 static WCHAR windowsdir[MAX_PATH];
+static const BOOL is_64bit = sizeof(void *) > sizeof(int);
 
-/* retrieve the (unix) path to the wine.inf file */
-static char *get_wine_inf_path(void)
+static const WCHAR winebuilddirW[] = {'W','I','N','E','B','U','I','L','D','D','I','R',0};
+static const WCHAR winedatadirW[] = {'W','I','N','E','D','A','T','A','D','I','R',0};
+static const WCHAR wineconfigdirW[] = {'W','I','N','E','C','O','N','F','I','G','D','I','R',0};
+
+/* retrieve the path to the wine.inf file */
+static WCHAR *get_wine_inf_path(void)
 {
-    const char *build_dir, *data_dir;
-    char *name = NULL;
+    static const WCHAR loaderW[] = {'\\','l','o','a','d','e','r',0};
+    static const WCHAR wine_infW[] = {'\\','w','i','n','e','.','i','n','f',0};
+    WCHAR *dir, *name = NULL;
 
-    if ((data_dir = wine_get_data_dir()))
+    if ((dir = _wgetenv( winebuilddirW )))
     {
-        if (!(name = HeapAlloc( GetProcessHeap(), 0, strlen(data_dir) + sizeof("/wine.inf") )))
+        if (!(name = HeapAlloc( GetProcessHeap(), 0,
+                                sizeof(loaderW) + sizeof(wine_infW) + lstrlenW(dir) * sizeof(WCHAR) )))
             return NULL;
-        strcpy( name, data_dir );
-        strcat( name, "/wine.inf" );
+        lstrcpyW( name, dir );
+        lstrcatW( name, loaderW );
     }
-    else if ((build_dir = wine_get_build_dir()))
+    else if ((dir = _wgetenv( winedatadirW )))
     {
-        if (!(name = HeapAlloc( GetProcessHeap(), 0, strlen(build_dir) + sizeof("/loader/wine.inf") )))
+        if (!(name = HeapAlloc( GetProcessHeap(), 0, sizeof(wine_infW) + lstrlenW(dir) * sizeof(WCHAR) )))
             return NULL;
-        strcpy( name, build_dir );
-        strcat( name, "/loader/wine.inf" );
+        lstrcpyW( name, dir );
     }
+    else return NULL;
+
+    lstrcatW( name, wine_infW );
+    name[1] = '\\';  /* change \??\ to \\?\ */
     return name;
 }
 
 /* update the timestamp if different from the reference time */
-static BOOL update_timestamp( const char *config_dir, unsigned long timestamp )
+static BOOL update_timestamp( const WCHAR *config_dir, unsigned long timestamp )
 {
+    static const WCHAR timestampW[] = {'\\','.','u','p','d','a','t','e','-','t','i','m','e','s','t','a','m','p',0};
     BOOL ret = FALSE;
     int fd, count;
     char buffer[100];
-    char *file = HeapAlloc( GetProcessHeap(), 0, strlen(config_dir) + sizeof("/.update-timestamp") );
+    WCHAR *file = HeapAlloc( GetProcessHeap(), 0, lstrlenW(config_dir) * sizeof(WCHAR) + sizeof(timestampW) );
 
     if (!file) return FALSE;
-    strcpy( file, config_dir );
-    strcat( file, "/.update-timestamp" );
+    lstrcpyW( file, config_dir );
+    lstrcatW( file, timestampW );
 
-    if ((fd = open( file, O_RDWR )) != -1)
+    if ((fd = _wopen( file, O_RDWR )) != -1)
     {
         if ((count = read( fd, buffer, sizeof(buffer) - 1 )) >= 0)
         {
@@ -135,19 +134,19 @@ static BOOL update_timestamp( const char *config_dir, unsigned long timestamp )
             if (timestamp == strtoul( buffer, NULL, 10 )) goto done;
         }
         lseek( fd, 0, SEEK_SET );
-        ftruncate( fd, 0 );
+        chsize( fd, 0 );
     }
     else
     {
         if (errno != ENOENT) goto done;
-        if ((fd = open( file, O_WRONLY | O_CREAT | O_TRUNC, 0666 )) == -1) goto done;
+        if ((fd = _wopen( file, O_WRONLY | O_CREAT | O_TRUNC, 0666 )) == -1) goto done;
     }
 
     count = sprintf( buffer, "%lu\n", timestamp );
     if (write( fd, buffer, count ) != count)
     {
-        WINE_WARN( "failed to update timestamp in %s\n", file );
-        ftruncate( fd, 0 );
+        WINE_WARN( "failed to update timestamp in %s\n", debugstr_w(file) );
+        chsize( fd, 0 );
     }
     else ret = TRUE;
 
@@ -157,17 +156,145 @@ done:
     return ret;
 }
 
+/* print the config directory in a more Unix-ish way */
+static const char *prettyprint_configdir(void)
+{
+    static char buffer[MAX_PATH];
+    WCHAR *path = _wgetenv( wineconfigdirW );
+    char *p;
+
+    if (!WideCharToMultiByte( CP_UNIXCP, 0, path, -1, buffer, ARRAY_SIZE(buffer), NULL, NULL ))
+        strcpy( buffer + ARRAY_SIZE(buffer) - 4, "..." );
+
+    if (!strncmp( buffer, "\\??\\unix\\", 9 ))
+    {
+        for (p = buffer + 9; *p; p++) if (*p == '\\') *p = '/';
+        return buffer + 9;
+    }
+    else if (!strncmp( buffer, "\\??\\Z:\\", 7 ))
+    {
+        for (p = buffer + 6; *p; p++) if (*p == '\\') *p = '/';
+        return buffer + 6;
+    }
+    else return buffer + 4;
+}
+
 /* wrapper for RegSetValueExW */
 static DWORD set_reg_value( HKEY hkey, const WCHAR *name, const WCHAR *value )
 {
-    return RegSetValueExW( hkey, name, 0, REG_SZ, (const BYTE *)value, (strlenW(value) + 1) * sizeof(WCHAR) );
+    return RegSetValueExW( hkey, name, 0, REG_SZ, (const BYTE *)value, (lstrlenW(value) + 1) * sizeof(WCHAR) );
+}
+
+#if defined(_MSC_VER)
+static void do_cpuid( unsigned int ax, unsigned int *p )
+{
+    __cpuid( p, ax );
+}
+#elif defined(__i386__)
+extern void __cdecl do_cpuid( unsigned int ax, unsigned int *p );
+__ASM_GLOBAL_FUNC( do_cpuid,
+                   "pushl %esi\n\t"
+                   "pushl %ebx\n\t"
+                   "movl 12(%esp),%eax\n\t"
+                   "movl 16(%esp),%esi\n\t"
+                   "cpuid\n\t"
+                   "movl %eax,(%esi)\n\t"
+                   "movl %ebx,4(%esi)\n\t"
+                   "movl %ecx,8(%esi)\n\t"
+                   "movl %edx,12(%esi)\n\t"
+                   "popl %ebx\n\t"
+                   "popl %esi\n\t"
+                   "ret" )
+#elif defined(__x86_64__)
+extern void __cdecl do_cpuid( unsigned int ax, unsigned int *p );
+__ASM_GLOBAL_FUNC( do_cpuid,
+                   "pushq %rsi\n\t"
+                   "pushq %rbx\n\t"
+                   "movq %rcx,%rax\n\t"
+                   "movq %rdx,%rsi\n\t"
+                   "cpuid\n\t"
+                   "movl %eax,(%rsi)\n\t"
+                   "movl %ebx,4(%rsi)\n\t"
+                   "movl %ecx,8(%rsi)\n\t"
+                   "movl %edx,12(%rsi)\n\t"
+                   "popq %rbx\n\t"
+                   "popq %rsi\n\t"
+                   "ret" )
+#else
+static void do_cpuid( unsigned int ax, unsigned int *p )
+{
+    FIXME("\n");
+}
+#endif
+
+static void regs_to_str( unsigned int *regs, unsigned int len, WCHAR *buffer )
+{
+    unsigned int i;
+    unsigned char *p = (unsigned char *)regs;
+
+    for (i = 0; i < len; i++) { buffer[i] = *p++; }
+    buffer[i] = 0;
+}
+
+static unsigned int get_model( unsigned int reg0, unsigned int *stepping, unsigned int *family )
+{
+    unsigned int model, family_id = (reg0 & (0x0f << 8)) >> 8;
+
+    model = (reg0 & (0x0f << 4)) >> 4;
+    if (family_id == 6 || family_id == 15) model |= (reg0 & (0x0f << 16)) >> 12;
+
+    *family = family_id;
+    if (family_id == 15) *family += (reg0 & (0xff << 20)) >> 20;
+
+    *stepping = reg0 & 0x0f;
+    return model;
+}
+
+static void get_identifier( WCHAR *buf, size_t size, const WCHAR *arch )
+{
+    static const WCHAR fmtW[] = {'%','s',' ','F','a','m','i','l','y',' ','%','u',' ','M','o','d','e','l',
+                                 ' ','%','u',' ','S','t','e','p','p','i','n','g',' ','%','u',0};
+    unsigned int regs[4] = {0, 0, 0, 0}, family, model, stepping;
+
+    do_cpuid( 1, regs );
+    model = get_model( regs[0], &stepping, &family );
+    swprintf( buf, size, fmtW, arch, family, model, stepping );
+}
+
+static void get_vendorid( WCHAR *buf )
+{
+    unsigned int tmp, regs[4] = {0, 0, 0, 0};
+
+    do_cpuid( 0, regs );
+    tmp = regs[2];      /* swap edx and ecx */
+    regs[2] = regs[3];
+    regs[3] = tmp;
+
+    regs_to_str( regs + 1, 12, buf );
+}
+
+static void get_namestring( WCHAR *buf )
+{
+    unsigned int regs[4] = {0, 0, 0, 0};
+    int i;
+
+    do_cpuid( 0x80000000, regs );
+    if (regs[0] >= 0x80000004)
+    {
+        do_cpuid( 0x80000002, regs );
+        regs_to_str( regs, 16, buf );
+        do_cpuid( 0x80000003, regs );
+        regs_to_str( regs, 16, buf + 16 );
+        do_cpuid( 0x80000004, regs );
+        regs_to_str( regs, 16, buf + 32 );
+    }
+    for (i = lstrlenW(buf) - 1; i >= 0 && buf[i] == ' '; i--) buf[i] = 0;
 }
 
 /* create the volatile hardware registry keys */
 static void create_hardware_registry_keys(void)
 {
-    static const WCHAR SystemW[] = {'H','a','r','d','w','a','r','e','\\',
-                                    'D','e','s','c','r','i','p','t','i','o','n','\\',
+    static const WCHAR SystemW[] = {'H','a','r','d','w','a','r','e','\\','D','e','s','c','r','i','p','t','i','o','n','\\',
                                     'S','y','s','t','e','m',0};
     static const WCHAR fpuW[] = {'F','l','o','a','t','i','n','g','P','o','i','n','t','P','r','o','c','e','s','s','o','r',0};
     static const WCHAR cpuW[] = {'C','e','n','t','r','a','l','P','r','o','c','e','s','s','o','r',0};
@@ -178,22 +305,22 @@ static void create_hardware_registry_keys(void)
     static const WCHAR ARMSysidW[] = {'A','R','M',' ','p','r','o','c','e','s','s','o','r',' ','f','a','m','i','l','y',0};
     static const WCHAR mhzKeyW[] = {'~','M','H','z',0};
     static const WCHAR VendorIdentifierW[] = {'V','e','n','d','o','r','I','d','e','n','t','i','f','i','e','r',0};
-    static const WCHAR VenidIntelW[] = {'G','e','n','u','i','n','e','I','n','t','e','l',0};
-    /* static const WCHAR VenidAMDW[] = {'A','u','t','h','e','n','t','i','c','A','M','D',0}; */
     static const WCHAR PercentDW[] = {'%','d',0};
-    static const WCHAR IntelCpuDescrW[] = {'x','8','6',' ','F','a','m','i','l','y',' ','%','d',' ','M','o','d','e','l',' ','%','d',
-                                           ' ','S','t','e','p','p','i','n','g',' ','%','d',0};
     static const WCHAR ARMCpuDescrW[]  = {'A','R','M',' ','F','a','m','i','l','y',' ','%','d',' ','M','o','d','e','l',' ','%','d',
-                                            ' ','R','e','v','i','s','i','o','n',' ','%','d',0};
-    static const WCHAR IntelCpuStringW[] = {'I','n','t','e','l','(','R',')',' ','P','e','n','t','i','u','m','(','R',')',' ','4',' ',
-                                            'C','P','U',' ','2','.','4','0','G','H','z',0};
+                                          ' ','R','e','v','i','s','i','o','n',' ','%','d',0};
+    static const WCHAR x86W[] = {'x','8','6',0};
+    static const WCHAR intel64W[] = {'I','n','t','e','l','6','4',0};
+    static const WCHAR amd64W[] = {'A','M','D','6','4',0};
+    static const WCHAR authenticamdW[] = {'A','u','t','h','e','n','t','i','c','A','M','D',0};
     unsigned int i;
     HKEY hkey, system_key, cpu_key, fpu_key;
     SYSTEM_CPU_INFORMATION sci;
     PROCESSOR_POWER_INFORMATION* power_info;
     ULONG sizeof_power_info = sizeof(PROCESSOR_POWER_INFORMATION) * NtCurrentTeb()->Peb->NumberOfProcessors;
-    WCHAR idW[60];
+    WCHAR id[60], namestr[49], vendorid[13];
 
+    get_namestring( namestr );
+    get_vendorid( vendorid );
     NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
 
     power_info = HeapAlloc( GetProcessHeap(), 0, sizeof_power_info );
@@ -202,16 +329,20 @@ static void create_hardware_registry_keys(void)
     if (NtPowerInformation( ProcessorInformation, NULL, 0, power_info, sizeof_power_info ))
         memset( power_info, 0, sizeof_power_info );
 
-    /*TODO: report 64bit processors properly*/
-    switch(sci.Architecture)
+    switch (sci.Architecture)
     {
     case PROCESSOR_ARCHITECTURE_ARM:
     case PROCESSOR_ARCHITECTURE_ARM64:
-        sprintfW( idW, ARMCpuDescrW, sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
+        swprintf( id, ARRAY_SIZE(id), ARMCpuDescrW, sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
         break;
-    default:
+
+    case PROCESSOR_ARCHITECTURE_AMD64:
+        get_identifier( id, ARRAY_SIZE(id), !wcscmp(vendorid, authenticamdW) ? amd64W : intel64W );
+        break;
+
     case PROCESSOR_ARCHITECTURE_INTEL:
-        sprintfW( idW, IntelCpuDescrW, sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
+    default:
+        get_identifier( id, ARRAY_SIZE(id), x86W );
         break;
     }
 
@@ -222,14 +353,16 @@ static void create_hardware_registry_keys(void)
         return;
     }
 
-    switch(sci.Architecture)
+    switch (sci.Architecture)
     {
     case PROCESSOR_ARCHITECTURE_ARM:
     case PROCESSOR_ARCHITECTURE_ARM64:
         set_reg_value( system_key, IdentifierW, ARMSysidW );
         break;
-    default:
+
     case PROCESSOR_ARCHITECTURE_INTEL:
+    case PROCESSOR_ARCHITECTURE_AMD64:
+    default:
         set_reg_value( system_key, IdentifierW, SysidW );
         break;
     }
@@ -247,15 +380,15 @@ static void create_hardware_registry_keys(void)
     {
         WCHAR numW[10];
 
-        sprintfW( numW, PercentDW, i );
+        swprintf( numW, ARRAY_SIZE(numW), PercentDW, i );
         if (!RegCreateKeyExW( cpu_key, numW, 0, NULL, REG_OPTION_VOLATILE,
                               KEY_ALL_ACCESS, NULL, &hkey, NULL ))
         {
             RegSetValueExW( hkey, FeatureSetW, 0, REG_DWORD, (BYTE *)&sci.FeatureSet, sizeof(DWORD) );
-            set_reg_value( hkey, IdentifierW, idW );
-            /*TODO; report ARM and AMD properly*/
-            set_reg_value( hkey, ProcessorNameStringW, IntelCpuStringW );
-            set_reg_value( hkey, VendorIdentifierW, VenidIntelW );
+            set_reg_value( hkey, IdentifierW, id );
+            /* TODO: report ARM properly */
+            set_reg_value( hkey, ProcessorNameStringW, namestr );
+            set_reg_value( hkey, VendorIdentifierW, vendorid );
             RegSetValueExW( hkey, mhzKeyW, 0, REG_DWORD, (BYTE *)&power_info[i].MaxMhz, sizeof(DWORD) );
             RegCloseKey( hkey );
         }
@@ -264,7 +397,7 @@ static void create_hardware_registry_keys(void)
             !RegCreateKeyExW( fpu_key, numW, 0, NULL, REG_OPTION_VOLATILE,
                               KEY_ALL_ACCESS, NULL, &hkey, NULL ))
         {
-            set_reg_value( hkey, IdentifierW, idW );
+            set_reg_value( hkey, IdentifierW, id );
             RegCloseKey( hkey );
         }
     }
@@ -301,59 +434,66 @@ static void create_environment_registry_keys( void )
     static const WCHAR NumProcW[]  = {'N','U','M','B','E','R','_','O','F','_','P','R','O','C','E','S','S','O','R','S',0};
     static const WCHAR ProcArchW[] = {'P','R','O','C','E','S','S','O','R','_','A','R','C','H','I','T','E','C','T','U','R','E',0};
     static const WCHAR x86W[]      = {'x','8','6',0};
-    static const WCHAR armW[]      = {'A','R','M',0};
-    static const WCHAR arm64W[]    = {'A','R','M','6','4',0};
-    static const WCHAR AMD64W[]    = {'A','M','D','6','4',0};
+    static const WCHAR intel64W[]  = {'I','n','t','e','l','6','4',0};
+    static const WCHAR amd64W[]    = {'A','M','D','6','4',0};
+    static const WCHAR authenticamdW[] = {'A','u','t','h','e','n','t','i','c','A','M','D',0};
+    static const WCHAR commaW[]    = {',',' ',0};
     static const WCHAR ProcIdW[]   = {'P','R','O','C','E','S','S','O','R','_','I','D','E','N','T','I','F','I','E','R',0};
     static const WCHAR ProcLvlW[]  = {'P','R','O','C','E','S','S','O','R','_','L','E','V','E','L',0};
     static const WCHAR ProcRevW[]  = {'P','R','O','C','E','S','S','O','R','_','R','E','V','I','S','I','O','N',0};
     static const WCHAR PercentDW[] = {'%','d',0};
     static const WCHAR Percent04XW[] = {'%','0','4','x',0};
-    static const WCHAR IntelCpuDescrW[]  = {'%','s',' ','F','a','m','i','l','y',' ','%','d',' ','M','o','d','e','l',' ','%','d',
-                                            ' ','S','t','e','p','p','i','n','g',' ','%','d',',',' ','G','e','n','u','i','n','e','I','n','t','e','l',0};
     static const WCHAR ARMCpuDescrW[]  = {'A','R','M',' ','F','a','m','i','l','y',' ','%','d',' ','M','o','d','e','l',' ','%','d',
-                                            ' ','R','e','v','i','s','i','o','n',' ','%','d',0};
-
+                                          ' ','R','e','v','i','s','i','o','n',' ','%','d',0};
     HKEY env_key;
     SYSTEM_CPU_INFORMATION sci;
-    WCHAR buffer[60];
-    const WCHAR *arch;
+    WCHAR buffer[60], vendorid[13];
+    const WCHAR *arch, *parch;
 
     if (RegCreateKeyW( HKEY_LOCAL_MACHINE, EnvironW, &env_key )) return;
 
+    get_vendorid( vendorid );
     NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
 
-    sprintfW( buffer, PercentDW, NtCurrentTeb()->Peb->NumberOfProcessors );
+    swprintf( buffer, ARRAY_SIZE(buffer), PercentDW, NtCurrentTeb()->Peb->NumberOfProcessors );
     set_reg_value( env_key, NumProcW, buffer );
 
-    switch(sci.Architecture)
+    switch (sci.Architecture)
     {
-    case PROCESSOR_ARCHITECTURE_AMD64: arch = AMD64W; break;
-    case PROCESSOR_ARCHITECTURE_ARM:   arch = armW; break;
-    case PROCESSOR_ARCHITECTURE_ARM64: arch = arm64W; break;
+    case PROCESSOR_ARCHITECTURE_AMD64:
+        arch = amd64W;
+        parch = !wcscmp(vendorid, authenticamdW) ? amd64W : intel64W;
+        break;
+
+    case PROCESSOR_ARCHITECTURE_INTEL:
     default:
-    case PROCESSOR_ARCHITECTURE_INTEL: arch = x86W; break;
+        arch = parch = x86W;
+        break;
     }
     set_reg_value( env_key, ProcArchW, arch );
 
-    switch(sci.Architecture)
+    switch (sci.Architecture)
     {
     case PROCESSOR_ARCHITECTURE_ARM:
     case PROCESSOR_ARCHITECTURE_ARM64:
-        sprintfW( buffer, ARMCpuDescrW, sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
+        swprintf( buffer, ARRAY_SIZE(buffer), ARMCpuDescrW,
+                  sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
         break;
-    default:
+
+    case PROCESSOR_ARCHITECTURE_AMD64:
     case PROCESSOR_ARCHITECTURE_INTEL:
-        sprintfW( buffer, IntelCpuDescrW, arch, sci.Level, HIBYTE(sci.Revision), LOBYTE(sci.Revision) );
+    default:
+        get_identifier( buffer, ARRAY_SIZE(buffer), parch );
+        lstrcatW( buffer, commaW );
+        lstrcatW( buffer, vendorid );
         break;
     }
     set_reg_value( env_key, ProcIdW, buffer );
 
-    sprintfW( buffer, PercentDW, sci.Level );
+    swprintf( buffer, ARRAY_SIZE(buffer), PercentDW, sci.Level );
     set_reg_value( env_key, ProcLvlW, buffer );
 
-    /* Properly report model/stepping */
-    sprintfW( buffer, Percent04XW, sci.Revision );
+    swprintf( buffer, ARRAY_SIZE(buffer), Percent04XW, sci.Revision );
     set_reg_value( env_key, ProcRevW, buffer );
 
     RegCloseKey( env_key );
@@ -445,12 +585,12 @@ static BOOL wininit(void)
         if (!(buffer = HeapAlloc( GetProcessHeap(), 0, size * sizeof(WCHAR) ))) return FALSE;
     }
 
-    for (str = buffer; *str; str += strlenW(str) + 1)
+    for (str = buffer; *str; str += lstrlenW(str) + 1)
     {
         WCHAR *value;
 
         if (*str == ';') continue;  /* comment */
-        if (!(value = strchrW( str, '=' ))) continue;
+        if (!(value = wcschr( str, '=' ))) continue;
 
         /* split the line into key and value */
         *value++ = 0;
@@ -652,6 +792,70 @@ static DWORD runCmd(LPWSTR cmdline, LPCWSTR dir, BOOL wait, BOOL minimized)
     return exit_code;
 }
 
+static void process_run_key( HKEY key, const WCHAR *keyname, BOOL delete, BOOL synchronous )
+{
+    HKEY runkey;
+    LONG res;
+    DWORD disp, i, max_cmdline = 0, max_value = 0;
+    WCHAR *cmdline = NULL, *value = NULL;
+
+    if (RegCreateKeyExW( key, keyname, 0, NULL, 0, delete ? KEY_ALL_ACCESS : KEY_READ, NULL, &runkey, &disp ))
+        return;
+
+    if (disp == REG_CREATED_NEW_KEY)
+        goto end;
+
+    if (RegQueryInfoKeyW( runkey, NULL, NULL, NULL, NULL, NULL, NULL, &i, &max_value, &max_cmdline, NULL, NULL ))
+        goto end;
+
+    if (!i)
+    {
+        WINE_TRACE( "No commands to execute.\n" );
+        goto end;
+    }
+    if (!(cmdline = HeapAlloc( GetProcessHeap(), 0, max_cmdline )))
+    {
+        WINE_ERR( "Couldn't allocate memory for the commands to be executed.\n" );
+        goto end;
+    }
+    if (!(value = HeapAlloc( GetProcessHeap(), 0, ++max_value * sizeof(*value) )))
+    {
+        WINE_ERR( "Couldn't allocate memory for the value names.\n" );
+        goto end;
+    }
+
+    while (i)
+    {
+        DWORD len = max_value, len_data = max_cmdline, type;
+
+        if ((res = RegEnumValueW( runkey, --i, value, &len, 0, &type, (BYTE *)cmdline, &len_data )))
+        {
+            WINE_ERR( "Couldn't read value %u (%d).\n", i, res );
+            continue;
+        }
+        if (delete && (res = RegDeleteValueW( runkey, value )))
+        {
+            WINE_ERR( "Couldn't delete value %u (%d). Running command anyways.\n", i, res );
+        }
+        if (type != REG_SZ)
+        {
+            WINE_ERR( "Incorrect type of value %u (%u).\n", i, type );
+            continue;
+        }
+        if (runCmd( cmdline, NULL, synchronous, FALSE ) == INVALID_RUNCMD_RETURN)
+        {
+            WINE_ERR( "Error running cmd %s (%u).\n", wine_dbgstr_w(cmdline), GetLastError() );
+        }
+        WINE_TRACE( "Done processing cmd %u.\n", i );
+    }
+
+end:
+    HeapFree( GetProcessHeap(), 0, value );
+    HeapFree( GetProcessHeap(), 0, cmdline );
+    RegCloseKey( runkey );
+    WINE_TRACE( "Done.\n" );
+}
+
 /*
  * Process a "Run" type registry key.
  * hkRoot is the HKEY from which "Software\Microsoft\Windows\CurrentVersion" is
@@ -661,110 +865,36 @@ static DWORD runCmd(LPWSTR cmdline, LPCWSTR dir, BOOL wait, BOOL minimized)
  * bSynchronous tells whether we should wait for the prog to complete before
  *      going on to the next prog.
  */
-static BOOL ProcessRunKeys( HKEY hkRoot, LPCWSTR szKeyName, BOOL bDelete,
-        BOOL bSynchronous )
+static void ProcessRunKeys( HKEY root, const WCHAR *keyname, BOOL delete, BOOL synchronous )
 {
-    static const WCHAR WINKEY_NAME[]={'S','o','f','t','w','a','r','e','\\',
-        'M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
-        'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
-    HKEY hkWin, hkRun;
-    DWORD res, dispos;
-    DWORD i, nMaxCmdLine=0, nMaxValue=0;
-    WCHAR *szCmdLine=NULL;
-    WCHAR *szValue=NULL;
+    static const WCHAR keypathW[] =
+        {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
+         'W','i','n','d','o','w','s','\\','C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
+    HKEY key;
 
-    if (hkRoot==HKEY_LOCAL_MACHINE)
-        WINE_TRACE("processing %s entries under HKLM\n",wine_dbgstr_w(szKeyName) );
+    if (root == HKEY_LOCAL_MACHINE)
+    {
+        WINE_TRACE( "Processing %s entries under HKLM.\n", wine_dbgstr_w(keyname) );
+        if (!RegCreateKeyExW( root, keypathW, 0, NULL, 0, KEY_READ, NULL, &key, NULL ))
+        {
+            process_run_key( key, keyname, delete, synchronous );
+            RegCloseKey( key );
+        }
+        if (is_64bit && !RegCreateKeyExW( root, keypathW, 0, NULL, 0, KEY_READ|KEY_WOW64_32KEY, NULL, &key, NULL ))
+        {
+            process_run_key( key, keyname, delete, synchronous );
+            RegCloseKey( key );
+        }
+    }
     else
-        WINE_TRACE("processing %s entries under HKCU\n",wine_dbgstr_w(szKeyName) );
-
-    if (RegCreateKeyExW( hkRoot, WINKEY_NAME, 0, NULL, 0, KEY_READ, NULL, &hkWin, NULL ) != ERROR_SUCCESS)
-        return TRUE;
-
-    if ((res = RegCreateKeyExW( hkWin, szKeyName, 0, NULL, 0, bDelete ? KEY_ALL_ACCESS : KEY_READ,
-                                NULL, &hkRun, &dispos )) != ERROR_SUCCESS)
     {
-        RegCloseKey( hkWin );
-        return TRUE;
-    }
-    RegCloseKey( hkWin );
-    if (dispos == REG_CREATED_NEW_KEY) goto end;
-
-    if( (res=RegQueryInfoKeyW( hkRun, NULL, NULL, NULL, NULL, NULL, NULL, &i, &nMaxValue,
-                    &nMaxCmdLine, NULL, NULL ))!=ERROR_SUCCESS )
-        goto end;
-
-    if( i==0 )
-    {
-        WINE_TRACE("No commands to execute.\n");
-
-        res=ERROR_SUCCESS;
-        goto end;
-    }
-    
-    if( (szCmdLine=HeapAlloc(GetProcessHeap(),0,nMaxCmdLine))==NULL )
-    {
-        WINE_ERR("Couldn't allocate memory for the commands to be executed\n");
-
-        res=ERROR_NOT_ENOUGH_MEMORY;
-        goto end;
-    }
-
-    if( (szValue=HeapAlloc(GetProcessHeap(),0,(++nMaxValue)*sizeof(*szValue)))==NULL )
-    {
-        WINE_ERR("Couldn't allocate memory for the value names\n");
-
-        res=ERROR_NOT_ENOUGH_MEMORY;
-        goto end;
-    }
-    
-    while( i>0 )
-    {
-        DWORD nValLength=nMaxValue, nDataLength=nMaxCmdLine;
-        DWORD type;
-
-        --i;
-
-        if( (res=RegEnumValueW( hkRun, i, szValue, &nValLength, 0, &type,
-                        (LPBYTE)szCmdLine, &nDataLength ))!=ERROR_SUCCESS )
+        WINE_TRACE( "Processing %s entries under HKCU.\n", wine_dbgstr_w(keyname) );
+        if (!RegCreateKeyExW( root, keypathW, 0, NULL, 0, KEY_READ, NULL, &key, NULL ))
         {
-            WINE_ERR("Couldn't read in value %d - %d\n", i, res );
-
-            continue;
+            process_run_key( key, keyname, delete, synchronous );
+            RegCloseKey( key );
         }
-
-        if( bDelete && (res=RegDeleteValueW( hkRun, szValue ))!=ERROR_SUCCESS )
-        {
-            WINE_ERR("Couldn't delete value - %d, %d. Running command anyways.\n", i, res );
-        }
-        
-        if( type!=REG_SZ )
-        {
-            WINE_ERR("Incorrect type of value #%d (%d)\n", i, type );
-
-            continue;
-        }
-
-        if( (res=runCmd(szCmdLine, NULL, bSynchronous, FALSE ))==INVALID_RUNCMD_RETURN )
-        {
-            WINE_ERR("Error running cmd %s (%d)\n", wine_dbgstr_w(szCmdLine), GetLastError() );
-        }
-
-        WINE_TRACE("Done processing cmd #%d\n", i);
     }
-
-    res=ERROR_SUCCESS;
-
-end:
-    HeapFree( GetProcessHeap(), 0, szValue );
-    HeapFree( GetProcessHeap(), 0, szCmdLine );
-
-    if( hkRun!=NULL )
-        RegCloseKey( hkRun );
-
-    WINE_TRACE("done\n");
-
-    return res==ERROR_SUCCESS;
 }
 
 /*
@@ -801,7 +931,7 @@ static int ProcessWindowsFileProtection(void)
             sz += sizeof(WCHAR);
             dllcache = HeapAlloc(GetProcessHeap(),0,sz + sizeof(wildcardW));
             RegQueryValueExW( hkey, cachedirW, 0, NULL, (LPBYTE)dllcache, &sz);
-            strcatW( dllcache, wildcardW );
+            lstrcatW( dllcache, wildcardW );
         }
     }
     RegCloseKey(hkey);
@@ -811,11 +941,11 @@ static int ProcessWindowsFileProtection(void)
         DWORD sz = GetSystemDirectoryW( NULL, 0 );
         dllcache = HeapAlloc( GetProcessHeap(), 0, sz * sizeof(WCHAR) + sizeof(dllcacheW));
         GetSystemDirectoryW( dllcache, sz );
-        strcatW( dllcache, dllcacheW );
+        lstrcatW( dllcache, dllcacheW );
     }
 
     find_handle = FindFirstFileW(dllcache,&finddata);
-    dllcache[ strlenW(dllcache) - 2] = 0; /* strip off wildcard */
+    dllcache[ lstrlenW(dllcache) - 2] = 0; /* strip off wildcard */
     find_rc = find_handle != INVALID_HANDLE_VALUE;
     while (find_rc)
     {
@@ -827,7 +957,7 @@ static int ProcessWindowsFileProtection(void)
         UINT sz2;
         WCHAR tempfile[MAX_PATH];
 
-        if (strcmpW(finddata.cFileName,dotW) == 0 || strcmpW(finddata.cFileName,dotdotW) == 0)
+        if (wcscmp(finddata.cFileName,dotW) == 0 || wcscmp(finddata.cFileName,dotdotW) == 0)
         {
             find_rc = FindNextFileW(find_handle,&finddata);
             continue;
@@ -848,7 +978,7 @@ static int ProcessWindowsFileProtection(void)
 
         /* now delete the source file so that we don't try to install it over and over again */
         lstrcpynW( targetpath, dllcache, MAX_PATH - 1 );
-        sz = strlenW( targetpath );
+        sz = lstrlenW( targetpath );
         targetpath[sz++] = '\\';
         lstrcpynW( targetpath + sz, finddata.cFileName, MAX_PATH - sz );
         if (!DeleteFileW( targetpath ))
@@ -870,9 +1000,9 @@ static BOOL start_services_process(void)
     HANDLE wait_handles[2];
     WCHAR path[MAX_PATH];
 
-    if (!GetSystemDirectoryW(path, MAX_PATH - strlenW(services)))
+    if (!GetSystemDirectoryW(path, MAX_PATH - lstrlenW(services)))
         return FALSE;
-    strcatW(path, services);
+    lstrcatW(path, services);
     ZeroMemory(&si, sizeof(si));
     si.cb = sizeof(si);
     if (!CreateProcessW(path, path, NULL, NULL, TRUE, DETACHED_PROCESS, NULL, NULL, &si, &pi))
@@ -907,13 +1037,15 @@ static INT_PTR CALLBACK wait_dlgproc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp
     {
     case WM_INITDIALOG:
         {
+            DWORD len;
             WCHAR *buffer, text[1024];
             const WCHAR *name = (WCHAR *)lp;
             HICON icon = LoadImageW( 0, (LPCWSTR)IDI_WINLOGO, IMAGE_ICON, 48, 48, LR_SHARED );
             SendDlgItemMessageW( hwnd, IDC_WAITICON, STM_SETICON, (WPARAM)icon, 0 );
             SendDlgItemMessageW( hwnd, IDC_WAITTEXT, WM_GETTEXT, 1024, (LPARAM)text );
-            buffer = HeapAlloc( GetProcessHeap(), 0, (strlenW(text) + strlenW(name) + 1) * sizeof(WCHAR) );
-            sprintfW( buffer, text, name );
+            len = lstrlenW(text) + lstrlenW(name) + 1;
+            buffer = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
+            swprintf( buffer, len, text, name );
             SendDlgItemMessageW( hwnd, IDC_WAITTEXT, WM_SETTEXT, 0, (LPARAM)buffer );
             HeapFree( GetProcessHeap(), 0, buffer );
         }
@@ -924,7 +1056,7 @@ static INT_PTR CALLBACK wait_dlgproc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp
 
 static HWND show_wait_window(void)
 {
-    const char *config_dir = wine_get_config_dir();
+    const char *config_dir = prettyprint_configdir();
     WCHAR *name;
     HWND hwnd;
     DWORD len;
@@ -939,20 +1071,20 @@ static HWND show_wait_window(void)
     return hwnd;
 }
 
-static HANDLE start_rundll32( const char *inf_path, BOOL wow64 )
+static HANDLE start_rundll32( const WCHAR *inf_path, BOOL wow64 )
 {
     static const WCHAR rundll[] = {'\\','r','u','n','d','l','l','3','2','.','e','x','e',0};
     static const WCHAR setupapi[] = {' ','s','e','t','u','p','a','p','i',',',
                                      'I','n','s','t','a','l','l','H','i','n','f','S','e','c','t','i','o','n',0};
     static const WCHAR definstall[] = {' ','D','e','f','a','u','l','t','I','n','s','t','a','l','l',0};
     static const WCHAR wowinstall[] = {' ','W','o','w','6','4','I','n','s','t','a','l','l',0};
-    static const WCHAR inf[] = {' ','1','2','8',' ','\\','\\','?','\\','u','n','i','x',0 };
+    static const WCHAR flags[] = {' ','1','2','8',' ',0};
 
     WCHAR app[MAX_PATH + ARRAY_SIZE(rundll)];
     STARTUPINFOW si;
     PROCESS_INFORMATION pi;
     WCHAR *buffer;
-    DWORD inf_len, cmd_len;
+    DWORD len;
 
     memset( &si, 0, sizeof(si) );
     si.cb = sizeof(si);
@@ -963,18 +1095,17 @@ static HANDLE start_rundll32( const char *inf_path, BOOL wow64 )
     }
     else GetSystemDirectoryW( app, MAX_PATH );
 
-    strcatW( app, rundll );
+    lstrcatW( app, rundll );
 
-    cmd_len = strlenW(app) * sizeof(WCHAR) + sizeof(setupapi) + sizeof(definstall) + sizeof(inf);
-    inf_len = MultiByteToWideChar( CP_UNIXCP, 0, inf_path, -1, NULL, 0 );
+    len = lstrlenW(app) + ARRAY_SIZE(setupapi) + ARRAY_SIZE(definstall) + ARRAY_SIZE(flags) + lstrlenW(inf_path);
 
-    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, cmd_len + inf_len * sizeof(WCHAR) ))) return 0;
+    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) ))) return 0;
 
-    strcpyW( buffer, app );
-    strcatW( buffer, setupapi );
-    strcatW( buffer, wow64 ? wowinstall : definstall );
-    strcatW( buffer, inf );
-    MultiByteToWideChar( CP_UNIXCP, 0, inf_path, -1, buffer + strlenW(buffer), inf_len );
+    lstrcpyW( buffer, app );
+    lstrcatW( buffer, setupapi );
+    lstrcatW( buffer, wow64 ? wowinstall : definstall );
+    lstrcatW( buffer, flags );
+    lstrcatW( buffer, inf_path );
 
     if (CreateProcessW( app, buffer, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi ))
         CloseHandle( pi.hThread );
@@ -988,20 +1119,20 @@ static HANDLE start_rundll32( const char *inf_path, BOOL wow64 )
 /* execute rundll32 on the wine.inf file if necessary */
 static void update_wineprefix( BOOL force )
 {
-    const char *config_dir = wine_get_config_dir();
-    char *inf_path = get_wine_inf_path();
+    const WCHAR *config_dir = _wgetenv( wineconfigdirW );
+    WCHAR *inf_path = get_wine_inf_path();
     int fd;
     struct stat st;
 
     if (!inf_path)
     {
-        WINE_MESSAGE( "wine: failed to update %s, wine.inf not found\n", config_dir );
+        WINE_MESSAGE( "wine: failed to update %s, wine.inf not found\n", prettyprint_configdir() );
         return;
     }
-    if ((fd = open( inf_path, O_RDONLY )) == -1)
+    if ((fd = _wopen( inf_path, O_RDONLY )) == -1)
     {
         WINE_MESSAGE( "wine: failed to update %s with %s: %s\n",
-                      config_dir, inf_path, strerror(errno) );
+                      prettyprint_configdir(), debugstr_w(inf_path), strerror(errno) );
         goto done;
     }
     fstat( fd, &st );
@@ -1028,7 +1159,7 @@ static void update_wineprefix( BOOL force )
             }
             DestroyWindow( hwnd );
         }
-        WINE_MESSAGE( "wine: configuration in '%s' has been updated.\n", config_dir );
+        WINE_MESSAGE( "wine: configuration in '%s' has been updated.\n", prettyprint_configdir() );
     }
 
 done:
@@ -1121,7 +1252,7 @@ done:
     return ret;
 }
 
-static void usage(void)
+static void usage( int status )
 {
     WINE_MESSAGE( "Usage: wineboot [options]\n" );
     WINE_MESSAGE( "Options;\n" );
@@ -1133,24 +1264,10 @@ static void usage(void)
     WINE_MESSAGE( "    -r,--restart      Restart only, don't do normal startup operations\n" );
     WINE_MESSAGE( "    -s,--shutdown     Shutdown only, don't reboot\n" );
     WINE_MESSAGE( "    -u,--update       Update the wineprefix directory\n" );
+    exit( status );
 }
 
-static const char short_options[] = "efhikrsu";
-
-static const struct option long_options[] =
-{
-    { "help",        0, 0, 'h' },
-    { "end-session", 0, 0, 'e' },
-    { "force",       0, 0, 'f' },
-    { "init" ,       0, 0, 'i' },
-    { "kill",        0, 0, 'k' },
-    { "restart",     0, 0, 'r' },
-    { "shutdown",    0, 0, 's' },
-    { "update",      0, 0, 'u' },
-    { NULL,          0, 0, 0 }
-};
-
-int main( int argc, char *argv[] )
+int __cdecl main( int argc, char *argv[] )
 {
     static const WCHAR RunW[] = {'R','u','n',0};
     static const WCHAR RunOnceW[] = {'R','u','n','O','n','c','e',0};
@@ -1159,7 +1276,7 @@ int main( int argc, char *argv[] )
     static const WCHAR wineboot_eventW[] = {'_','_','w','i','n','e','b','o','o','t','_','e','v','e','n','t',0};
 
     /* First, set the current directory to SystemRoot */
-    int optc;
+    int i, j;
     BOOL end_session, force, init, kill, restart, shutdown, update;
     HANDLE event;
     SECURITY_ATTRIBUTES sa;
@@ -1194,19 +1311,36 @@ int main( int argc, char *argv[] )
         Wow64RevertWow64FsRedirection( redir );
     }
 
-    while ((optc = getopt_long(argc, argv, short_options, long_options, NULL )) != -1)
+    for (i = 1; i < argc; i++)
     {
-        switch(optc)
+        if (argv[i][0] != '-') continue;
+        if (argv[i][1] == '-')
         {
-        case 'e': end_session = TRUE; break;
-        case 'f': force = TRUE; break;
-        case 'i': init = TRUE; break;
-        case 'k': kill = TRUE; break;
-        case 'r': restart = TRUE; break;
-        case 's': shutdown = TRUE; break;
-        case 'u': update = TRUE; break;
-        case 'h': usage(); return 0;
-        case '?': usage(); return 1;
+            if (!strcmp( argv[i], "--help" )) usage( 0 );
+            else if (!strcmp( argv[i], "--end-session" )) end_session = TRUE;
+            else if (!strcmp( argv[i], "--force" )) force = TRUE;
+            else if (!strcmp( argv[i], "--init" )) init = TRUE;
+            else if (!strcmp( argv[i], "--kill" )) kill = TRUE;
+            else if (!strcmp( argv[i], "--restart" )) restart = TRUE;
+            else if (!strcmp( argv[i], "--shutdown" )) shutdown = TRUE;
+            else if (!strcmp( argv[i], "--update" )) update = TRUE;
+            else usage( 1 );
+            continue;
+        }
+        for (j = 1; argv[i][j]; j++)
+        {
+            switch (argv[i][j])
+            {
+            case 'e': end_session = TRUE; break;
+            case 'f': force = TRUE; break;
+            case 'i': init = TRUE; break;
+            case 'k': kill = TRUE; break;
+            case 'r': restart = TRUE; break;
+            case 's': shutdown = TRUE; break;
+            case 'u': update = TRUE; break;
+            case 'h': usage(0); break;
+            default:  usage(1); break;
+            }
         }
     }
 
