@@ -516,22 +516,28 @@ void generate_startup_debug_events( struct process *process, client_ptr_t entry
     struct list *ptr;
     struct thread *thread, *first_thread = get_process_first_thread( process );
 
-    /* generate creation events */
-    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
-    {
-        if (thread == first_thread)
-            generate_debug_event( thread, CREATE_PROCESS_DEBUG_EVENT, &entry );
-        else
-            generate_debug_event( thread, CREATE_THREAD_DEBUG_EVENT, NULL );
-    }
+    generate_debug_event( first_thread, CREATE_PROCESS_DEBUG_EVENT, &entry );
+    ptr = list_head( &process->dlls ); /* skip main module reported in create process event */
 
-    /* generate dll events (in loading order, i.e. reverse list order) */
-    ptr = list_tail( &process->dlls );
-    while (ptr != list_head( &process->dlls ))
+    /* generate ntdll.dll load event */
+    if (ptr && (ptr = list_next( &process->dlls, ptr )))
+    {
+        struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
+        generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
+    }
+
+    /* generate creation events */
+    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
+    {
+        if (thread != first_thread)
+            generate_debug_event( thread, CREATE_THREAD_DEBUG_EVENT, NULL );
+    }
+
+    /* generate dll events (in loading order) */
+    while (ptr && (ptr = list_next( &process->dlls, ptr )))
     {
         struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
         generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
-        ptr = list_prev( &process->dlls, ptr );
     }
 }
 
@@ -632,7 +638,6 @@ DECL_HANDLER(debug_process)
     else if (debugger_attach( process, current ))
     {
         generate_startup_debug_events( process, 0 );
-        break_process( process );
         resume_process( process );
     }
     release_object( process );
@@ -708,20 +713,6 @@ DECL_HANDLER(get_exception_status)
     }
 }
 
-/* simulate a breakpoint in a process */
-DECL_HANDLER(debug_break)
-{
-    struct process *process;
-
-    reply->self = 0;
-    if ((process = get_process_from_handle( req->handle, PROCESS_SET_INFORMATION /*FIXME*/ )))
-    {
-        if (process != current->process) break_process( process );
-        else reply->self = 1;
-        release_object( process );
-    }
-}
-
 /* set debugger kill on exit flag */
 DECL_HANDLER(set_debugger_kill_on_exit)
 {
