@@ -58,7 +58,6 @@ static inline int needs_relay( const ORDDEF *odp )
     {
     case TYPE_STDCALL:
     case TYPE_CDECL:
-    case TYPE_THISCALL:
         break;
     case TYPE_STUB:
         if (odp->u.func.nb_args != -1) break;
@@ -68,8 +67,6 @@ static inline int needs_relay( const ORDDEF *odp )
     }
     /* skip norelay and forward entry points */
     if (odp->flags & (FLAG_NORELAY|FLAG_FORWARD)) return 0;
-    /* skip register entry points on x86_64 */
-    if (target_cpu == CPU_x86_64 && (odp->flags & FLAG_REGISTER)) return 0;
     return 1;
 }
 
@@ -96,6 +93,104 @@ static int has_relays( DLLSPEC *spec )
     return 0;
 }
 
+static int cmp_func_args( const void *p1, const void *p2 )
+{
+    const ORDDEF *odp1 = *(const ORDDEF **)p1;
+    const ORDDEF *odp2 = *(const ORDDEF **)p2;
+
+    return odp2->u.func.nb_args - odp1->u.func.nb_args;
+}
+
+static void get_arg_string( ORDDEF *odp, char str[MAX_ARGUMENTS + 1] )
+{
+    int i;
+
+    for (i = 0; i < odp->u.func.nb_args; i++)
+    {
+        switch (odp->u.func.args[i])
+        {
+        case ARG_STR: str[i] = 's'; break;
+        case ARG_WSTR: str[i] = 'w'; break;
+        case ARG_FLOAT: str[i] = 'f'; break;
+        case ARG_DOUBLE: str[i] = 'd'; break;
+        case ARG_INT64:
+        case ARG_INT128:
+            if (get_ptr_size() == 4)
+            {
+                str[i] = (odp->u.func.args[i] == ARG_INT64) ? 'j' : 'k';
+                break;
+            }
+            /* fall through */
+        case ARG_LONG:
+        case ARG_PTR:
+        default:
+            str[i] = 'i';
+            break;
+        }
+    }
+    if (odp->flags & (FLAG_THISCALL | FLAG_FASTCALL)) str[0] = 't';
+    if ((odp->flags & FLAG_FASTCALL) && odp->u.func.nb_args > 1) str[1] = 't';
+
+    /* append return value */
+    if (get_ptr_size() == 4 && (odp->flags & FLAG_RET64))
+        strcpy( str + i, "J" );
+    else
+        strcpy( str + i, "I" );
+}
+
+static void output_data_directories( const char *names[16] )
+{
+    int i;
+
+    for (i = 0; i < 16; i++)
+    {
+        if (names[i])
+        {
+            output_rva( "%s", names[i] );
+            output( "\t.long %s_end - %s\n", names[i], names[i] );
+        }
+        else output( "\t.long 0,0\n" );
+    }
+}
+
+/*******************************************************************
+ *         build_args_string
+ */
+static char *build_args_string( DLLSPEC *spec )
+{
+    int i, count = 0, len = 1;
+    char *p, *buffer;
+    char str[MAX_ARGUMENTS + 2];
+    ORDDEF **funcs;
+
+    funcs = xmalloc( (spec->limit + 1 - spec->base) * sizeof(*funcs) );
+    for (i = spec->base; i <= spec->limit; i++)
+    {
+        ORDDEF *odp = spec->ordinals[i];
+
+        if (!needs_relay( odp )) continue;
+        funcs[count++] = odp;
+        len += odp->u.func.nb_args + 1;
+    }
+    /* sort functions by decreasing number of arguments */
+    qsort( funcs, count, sizeof(*funcs), cmp_func_args );
+    buffer = xmalloc( len );
+    buffer[0] = 0;
+    /* build the arguments string, reusing substrings where possible */
+    for (i = 0; i < count; i++)
+    {
+        get_arg_string( funcs[i], str );
+        if (!(p = strstr( buffer, str )))
+        {
+            p = buffer + strlen( buffer );
+            strcpy( p, str );
+        }
+        funcs[i]->u.func.args_str_offset = p - buffer;
+    }
+    free( funcs );
+    return buffer;
+}
+
 /*******************************************************************
  *         output_relay_debug
  *
@@ -103,8 +198,7 @@ static int has_relays( DLLSPEC *spec )
  */
 static void output_relay_debug( DLLSPEC *spec )
 {
-    int i, j;
-    unsigned int pos, args, flags;
+    int i;
 
     /* first the table of entry point offsets */
 
@@ -122,33 +216,10 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\t.long 0\n" );
     }
 
-    /* then the table of argument types */
+    /* then the strings of argument types */
 
-    output( "\t.align %d\n", get_alignment(4) );
-    output( ".L__wine_spec_relay_arg_types:\n" );
-
-    for (i = spec->base; i <= spec->limit; i++)
-    {
-        ORDDEF *odp = spec->ordinals[i];
-        unsigned int mask = 0;
-
-        if (needs_relay( odp ))
-        {
-            for (j = pos = 0; pos < 16 && j < odp->u.func.nb_args; j++)
-            {
-                switch (odp->u.func.args[j])
-                {
-                case ARG_STR:    mask |= 1 << (2 * pos++); break;
-                case ARG_WSTR:   mask |= 2 << (2 * pos++); break;
-                case ARG_INT64:
-                case ARG_DOUBLE: pos += 8 / get_ptr_size(); break;
-                case ARG_INT128: pos += (target_cpu == CPU_x86) ? 4 : 1; break;
-                default:         pos++; break;
-                }
-            }
-        }
-        output( "\t.long 0x%08x\n", mask );
-    }
+    output( ".L__wine_spec_relay_args_string:\n" );
+    output( "\t%s \"%s\"\n", get_asm_string_keyword(), build_args_string( spec ));
 
     /* then the relay thunks */
 
@@ -162,31 +233,22 @@ static void output_relay_debug( DLLSPEC *spec )
 
         if (!needs_relay( odp )) continue;
 
-        output( "\t.align %d\n", get_alignment(4) );
-        output( ".L__wine_spec_relay_entry_point_%d:\n", i );
-        output_cfi( ".cfi_startproc" );
-
-        args = get_args_size(odp) / get_ptr_size();
-        flags = 0;
-
         switch (target_cpu)
         {
         case CPU_x86:
-            if (odp->type == TYPE_THISCALL)  /* add the this pointer */
+            output( "\t.align %d\n", get_alignment(4) );
+            output( "\t.long 0x90909090,0x90909090\n" );
+            output( ".L__wine_spec_relay_entry_point_%d:\n", i );
+            output_cfi( ".cfi_startproc" );
+            output( "\t.byte 0x8b,0xff,0x55,0x8b,0xec,0x5d\n" );  /* hotpatch prolog */
+            if (odp->flags & (FLAG_THISCALL | FLAG_FASTCALL))  /* add the register arguments */
             {
                 output( "\tpopl %%eax\n" );
+                if ((odp->flags & FLAG_FASTCALL) && get_args_size( odp ) > 4) output( "\tpushl %%edx\n" );
                 output( "\tpushl %%ecx\n" );
                 output( "\tpushl %%eax\n" );
-                flags |= 2;
             }
-            if (odp->flags & FLAG_REGISTER)
-                output( "\tpushl %%eax\n" );
-            else
-                output( "\tpushl %%esp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset 4" );
-
-            if (odp->flags & FLAG_RET64) flags |= 1;
-            output( "\tpushl $%u\n", (flags << 24) | (args << 16) | (i - spec->base) );
+            output( "\tpushl $%u\n", (odp->u.func.args_str_offset << 16) | (i - spec->base) );
             output_cfi( ".cfi_adjust_cfa_offset 4" );
 
             if (UsePIC)
@@ -199,36 +261,32 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\tpushl %%eax\n" );
             output_cfi( ".cfi_adjust_cfa_offset 4" );
 
-            if (odp->flags & FLAG_REGISTER)
-            {
-                output( "\tcall *8(%%eax)\n" );
-            }
+            output( "\tcall *4(%%eax)\n" );
+            output_cfi( ".cfi_adjust_cfa_offset -8" );
+            if (odp->type == TYPE_STDCALL)
+                output( "\tret $%u\n", get_args_size( odp ));
             else
-            {
-                output( "\tcall *4(%%eax)\n" );
-                output_cfi( ".cfi_adjust_cfa_offset -12" );
-                if (odp->type == TYPE_STDCALL || odp->type == TYPE_THISCALL)
-                    output( "\tret $%u\n", args * get_ptr_size() );
-                else
-                    output( "\tret\n" );
-            }
+                output( "\tret\n" );
+            output_cfi( ".cfi_endproc" );
             break;
 
         case CPU_ARM:
         {
             unsigned int mask, val, count = 0;
-            unsigned int stack_size = min( 16, (args * 4 + 7) & ~7 );
+            int j, has_float = 0;
 
-            if (odp->flags & FLAG_RET64) flags |= 1;
-            val = (flags << 24) | (args << 16) | (i - spec->base);
-            switch (stack_size)
-            {
-            case 16: output( "\tpush {r0-r3}\n" ); break;
-            case 8:  output( "\tpush {r0-r1}\n" ); break;
-            case 0:  break;
-            }
-            output( "\tpush {LR}\n" );
+            if (strcmp( float_abi_option, "soft" ))
+                for (j = 0; j < odp->u.func.nb_args && !has_float; j++)
+                    has_float = is_float_arg( odp, j );
+
+            val = (odp->u.func.args_str_offset << 16) | (i - spec->base);
+            output( "\t.align %d\n", get_alignment(4) );
+            output( ".L__wine_spec_relay_entry_point_%d:\n", i );
+            output_cfi( ".cfi_startproc" );
+            output( "\tpush {r0-r3}\n" );
             output( "\tmov r2, SP\n");
+            if (has_float) output( "\tvpush {s0-s15}\n" );
+            output( "\tpush {LR}\n" );
             output( "\tsub SP, #4\n");
             for (mask = 0xff; mask; mask <<= 8)
                 if (val & mask) output( "\t%s r1,#%u\n", count++ ? "add" : "mov", val & mask );
@@ -238,14 +296,18 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\tldr IP, [r0, #4]\n");
             output( "1:\tblx IP\n");
             output( "\tldr IP, [SP, #4]\n" );
-            output( "\tadd SP, #%u\n", stack_size + 8 );
+            output( "\tadd SP, #%u\n", 24 + (has_float ? 64 : 0) );
             output( "\tbx IP\n");
             output( "2:\t.long .L__wine_spec_relay_descr-1b\n" );
+            output_cfi( ".cfi_endproc" );
             break;
         }
 
         case CPU_ARM64:
-            switch (args)
+            output( "\t.align %d\n", get_alignment(4) );
+            output( ".L__wine_spec_relay_entry_point_%d:\n", i );
+            output_cfi( ".cfi_startproc" );
+            switch (odp->u.func.nb_args)
             {
             default:
             case 8:
@@ -262,51 +324,50 @@ static void output_relay_debug( DLLSPEC *spec )
             /* fall through */
             case 0:  break;
             }
+            output( "\tmov x2, SP\n");
             output( "\tstp x29, x30, [SP,#-16]!\n" );
             output( "\tstp x8, x9, [SP,#-16]!\n" );
-            output( "\tmov x2, SP\n");
-            if (odp->flags & FLAG_RET64) flags |= 1;
-            output( "\tmov w1, #%u\n", (flags << 24) );
-            if (args) output( "\tadd w1, w1, #%u\n", (args << 16) );
+            output( "\tmov w1, #%u\n", odp->u.func.args_str_offset << 16 );
             if (i - spec->base) output( "\tadd w1, w1, #%u\n", i - spec->base );
-            output( "\tldr x0, 1f\n");
+            output( "\tadrp x0, .L__wine_spec_relay_descr\n");
+            output( "\tadd x0, x0, #:lo12:.L__wine_spec_relay_descr\n");
             output( "\tldr x3, [x0, #8]\n");
             output( "\tblr x3\n");
             output( "\tadd SP, SP, #16\n" );
             output( "\tldp x29, x30, [SP], #16\n" );
-            if (args) output( "\tadd SP, SP, #%u\n", 8 * ((min(args, 8) + 1) & 0xe) );
+            if (odp->u.func.nb_args)
+                output( "\tadd SP, SP, #%u\n", 8 * ((min(odp->u.func.nb_args, 8) + 1) & ~1) );
             output( "\tret\n");
-            output( "\t1: .quad .L__wine_spec_relay_descr\n");
+            output_cfi( ".cfi_endproc" );
             break;
 
         case CPU_x86_64:
-            output( "\tsubq $40,%%rsp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset 40" );
-            switch (args)
+            output( "\t.align %d\n", get_alignment(4) );
+            output( "\t.long 0x90909090,0x90909090\n" );
+            output( ".L__wine_spec_relay_entry_point_%d:\n", i );
+            output_cfi( ".cfi_startproc" );
+            switch (odp->u.func.nb_args)
             {
-            default: output( "\tmovq %%%s,72(%%rsp)\n", is_float_arg( odp, 3 ) ? "xmm3" : "r9" );
+            default: output( "\tmovq %%%s,32(%%rsp)\n", is_float_arg( odp, 3 ) ? "xmm3" : "r9" );
             /* fall through */
-            case 3:  output( "\tmovq %%%s,64(%%rsp)\n", is_float_arg( odp, 2 ) ? "xmm2" : "r8" );
+            case 3:  output( "\tmovq %%%s,24(%%rsp)\n", is_float_arg( odp, 2 ) ? "xmm2" : "r8" );
             /* fall through */
-            case 2:  output( "\tmovq %%%s,56(%%rsp)\n", is_float_arg( odp, 1 ) ? "xmm1" : "rdx" );
+            case 2:  output( "\tmovq %%%s,16(%%rsp)\n", is_float_arg( odp, 1 ) ? "xmm1" : "rdx" );
             /* fall through */
-            case 1:  output( "\tmovq %%%s,48(%%rsp)\n", is_float_arg( odp, 0 ) ? "xmm0" : "rcx" );
+            case 1:  output( "\tmovq %%%s,8(%%rsp)\n", is_float_arg( odp, 0 ) ? "xmm0" : "rcx" );
             /* fall through */
             case 0:  break;
             }
-            output( "\tleaq 40(%%rsp),%%r8\n" );
-            output( "\tmovq $%u,%%rdx\n", (flags << 24) | (args << 16) | (i - spec->base) );
+            output( "\tmovl $%u,%%edx\n", (odp->u.func.args_str_offset << 16) | (i - spec->base) );
             output( "\tleaq .L__wine_spec_relay_descr(%%rip),%%rcx\n" );
             output( "\tcallq *8(%%rcx)\n" );
-            output( "\taddq $40,%%rsp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset -40" );
             output( "\tret\n" );
+            output_cfi( ".cfi_endproc" );
             break;
 
         default:
             assert(0);
         }
-        output_cfi( ".cfi_endproc" );
     }
 }
 
@@ -318,12 +379,16 @@ static void output_relay_debug( DLLSPEC *spec )
 void output_exports( DLLSPEC *spec )
 {
     int i, fwd_size = 0;
+    int needs_imports = 0;
+    int needs_relay = has_relays( spec );
     int nr_exports = spec->base <= spec->limit ? spec->limit - spec->base + 1 : 0;
+    const char *func_ptr = (target_platform == PLATFORM_WINDOWS) ? ".rva" : get_asm_ptr_keyword();
+    const char *name;
 
     if (!nr_exports) return;
 
     output( "\n/* export table */\n\n" );
-    output( "\t.data\n" );
+    output( "\t%s\n", get_asm_export_section() );
     output( "\t.align %d\n", get_alignment(4) );
     output( ".L__wine_spec_exports:\n" );
 
@@ -332,15 +397,15 @@ void output_exports( DLLSPEC *spec )
     output( "\t.long 0\n" );                       /* Characteristics */
     output( "\t.long 0\n" );                       /* TimeDateStamp */
     output( "\t.long 0\n" );                       /* MajorVersion/MinorVersion */
-    output( "\t.long .L__wine_spec_exp_names-.L__wine_spec_rva_base\n" ); /* Name */
+    output_rva( ".L__wine_spec_exp_names" );       /* Name */
     output( "\t.long %u\n", spec->base );          /* Base */
     output( "\t.long %u\n", nr_exports );          /* NumberOfFunctions */
     output( "\t.long %u\n", spec->nb_names );      /* NumberOfNames */
-    output( "\t.long .L__wine_spec_exports_funcs-.L__wine_spec_rva_base\n" ); /* AddressOfFunctions */
+    output_rva( ".L__wine_spec_exports_funcs " );  /* AddressOfFunctions */
     if (spec->nb_names)
     {
-        output( "\t.long .L__wine_spec_exp_name_ptrs-.L__wine_spec_rva_base\n" ); /* AddressOfNames */
-        output( "\t.long .L__wine_spec_exp_ordinals-.L__wine_spec_rva_base\n" );  /* AddressOfNameOrdinals */
+        output_rva( ".L__wine_spec_exp_name_ptrs" ); /* AddressOfNames */
+        output_rva( ".L__wine_spec_exp_ordinals" );  /* AddressOfNameOrdinals */
     }
     else
     {
@@ -354,32 +419,37 @@ void output_exports( DLLSPEC *spec )
     for (i = spec->base; i <= spec->limit; i++)
     {
         ORDDEF *odp = spec->ordinals[i];
-        if (!odp) output( "\t%s 0\n", get_asm_ptr_keyword() );
+        if (!odp) output( "\t%s 0\n",
+                          (target_platform == PLATFORM_WINDOWS) ? ".long" : get_asm_ptr_keyword() );
         else switch(odp->type)
         {
         case TYPE_EXTERN:
         case TYPE_STDCALL:
         case TYPE_VARARGS:
         case TYPE_CDECL:
-        case TYPE_THISCALL:
             if (odp->flags & FLAG_FORWARD)
             {
-                output( "\t%s .L__wine_spec_forwards+%u\n", get_asm_ptr_keyword(), fwd_size );
+                output( "\t%s .L__wine_spec_forwards+%u\n", func_ptr, fwd_size );
                 fwd_size += strlen(odp->link_name) + 1;
             }
+            else if ((odp->flags & FLAG_IMPORT) && (target_cpu == CPU_x86 || target_cpu == CPU_x86_64))
+            {
+                name = odp->name ? odp->name : odp->export_name;
+                if (name) output( "\t%s %s_%s\n", func_ptr, asm_name("__wine_spec_imp"), name );
+                else output( "\t%s %s_%u\n", func_ptr, asm_name("__wine_spec_imp"), i );
+                needs_imports = 1;
+            }
             else if (odp->flags & FLAG_EXT_LINK)
             {
-                output( "\t%s %s_%s\n",
-                         get_asm_ptr_keyword(), asm_name("__wine_spec_ext_link"), odp->link_name );
+                output( "\t%s %s_%s\n", func_ptr, asm_name("__wine_spec_ext_link"), odp->link_name );
             }
             else
             {
-                output( "\t%s %s\n", get_asm_ptr_keyword(), asm_name(odp->link_name) );
+                output( "\t%s %s\n", func_ptr, asm_name( get_link_name( odp )));
             }
             break;
         case TYPE_STUB:
-            output( "\t%s %s\n", get_asm_ptr_keyword(),
-                     asm_name( get_stub_name( odp, spec )) );
+            output( "\t%s %s\n", func_ptr, asm_name( get_stub_name( odp, spec )) );
             break;
         default:
             assert(0);
@@ -395,7 +465,7 @@ void output_exports( DLLSPEC *spec )
         output( "\n.L__wine_spec_exp_name_ptrs:\n" );
         for (i = 0; i < spec->nb_names; i++)
         {
-            output( "\t.long .L__wine_spec_exp_names+%u-.L__wine_spec_rva_base\n", namepos );
+            output_rva( ".L__wine_spec_exp_names + %u", namepos );
             namepos += strlen(spec->names[i]->name) + 1;
         }
 
@@ -412,6 +482,13 @@ void output_exports( DLLSPEC *spec )
         }
     }
 
+    if (needs_relay)
+    {
+        output( "\t.long 0xdeb90002\n" );  /* magic */
+        if (target_platform == PLATFORM_WINDOWS) output_rva( ".L__wine_spec_relay_descr" );
+        else output( "\t.long 0\n" );
+    }
+
     /* output the export name strings */
 
     output( "\n.L__wine_spec_exp_names:\n" );
@@ -432,26 +509,78 @@ void output_exports( DLLSPEC *spec )
                 output( "\t%s \"%s\"\n", get_asm_string_keyword(), odp->link_name );
         }
     }
-    output( "\t.align %d\n", get_alignment(get_ptr_size()) );
-    output( ".L__wine_spec_exports_end:\n" );
 
     /* output relays */
 
-    if (!has_relays( spec ))
+    if (needs_relay)
     {
-        output( "\t%s 0\n", get_asm_ptr_keyword() );
-        return;
+        if (target_platform == PLATFORM_WINDOWS)
+        {
+            output( "\t.data\n" );
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+        }
+        else
+        {
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( ".L__wine_spec_exports_end:\n" );
+        }
+
+        output( ".L__wine_spec_relay_descr:\n" );
+        output( "\t%s 0xdeb90002\n", get_asm_ptr_keyword() );  /* magic */
+        output( "\t%s 0\n", get_asm_ptr_keyword() );           /* relay func */
+        output( "\t%s 0\n", get_asm_ptr_keyword() );           /* private data */
+        output( "\t%s __wine_spec_relay_entry_points\n", get_asm_ptr_keyword() );
+        output( "\t%s .L__wine_spec_relay_entry_point_offsets\n", get_asm_ptr_keyword() );
+        output( "\t%s .L__wine_spec_relay_args_string\n", get_asm_ptr_keyword() );
+
+        output_relay_debug( spec );
+    }
+    else if (target_platform != PLATFORM_WINDOWS)
+    {
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( ".L__wine_spec_exports_end:\n" );
+            output( "\t%s 0\n", get_asm_ptr_keyword() );
     }
 
-    output( ".L__wine_spec_relay_descr:\n" );
-    output( "\t%s 0xdeb90001\n", get_asm_ptr_keyword() );  /* magic */
-    output( "\t%s 0,0\n", get_asm_ptr_keyword() );         /* relay funcs */
-    output( "\t%s 0\n", get_asm_ptr_keyword() );           /* private data */
-    output( "\t%s __wine_spec_relay_entry_points\n", get_asm_ptr_keyword() );
-    output( "\t%s .L__wine_spec_relay_entry_point_offsets\n", get_asm_ptr_keyword() );
-    output( "\t%s .L__wine_spec_relay_arg_types\n", get_asm_ptr_keyword() );
+    /* output import thunks */
 
-    output_relay_debug( spec );
+    if (!needs_imports) return;
+    output( "\t.text\n" );
+    for (i = spec->base; i <= spec->limit; i++)
+    {
+        ORDDEF *odp = spec->ordinals[i];
+        if (!odp) continue;
+        if (!(odp->flags & FLAG_IMPORT)) continue;
+
+        name = odp->name ? odp->name : odp->export_name;
+
+        output( "\t.align %d\n", get_alignment(4) );
+        output( "\t.long 0x90909090,0x90909090\n" );
+        if (name) output( "%s_%s:\n", asm_name("__wine_spec_imp"), name );
+        else output( "%s_%u:\n", asm_name("__wine_spec_imp"), i );
+        output_cfi( ".cfi_startproc" );
+
+        switch (target_cpu)
+        {
+        case CPU_x86:
+            output( "\t.byte 0x8b,0xff,0x55,0x8b,0xec,0x5d\n" );  /* hotpatch prolog */
+            if (UsePIC)
+            {
+                output( "\tcall %s\n", asm_name("__wine_spec_get_pc_thunk_eax") );
+                output( "1:\tjmp *__imp_%s-1b(%%eax)\n", asm_name( get_link_name( odp )));
+                needs_get_pc_thunk = 1;
+            }
+            else output( "\tjmp *__imp_%s\n", asm_name( get_link_name( odp )));
+            break;
+        case CPU_x86_64:
+            output( "\t.byte 0x48\n" );  /* hotpatch prolog */
+            output( "\tjmp *__imp_%s(%%rip)\n", asm_name( get_link_name( odp )));
+            break;
+        default:
+            assert(0);
+        }
+        output_cfi( ".cfi_endproc" );
+    }
 }
 
 
@@ -501,11 +630,14 @@ void output_module( DLLSPEC *spec )
 {
     int machine = 0;
     unsigned int page_size = get_page_size();
+    const char *data_dirs[16] = { NULL };
 
     /* Reserve some space for the PE header */
 
     switch (target_platform)
     {
+    case PLATFORM_WINDOWS:
+        return;  /* nothing to do */
     case PLATFORM_APPLE:
         output( "\t.text\n" );
         output( "\t.align %d\n", get_alignment(page_size) );
@@ -591,8 +723,7 @@ void output_module( DLLSPEC *spec )
     output( "\t.short %u,%u\n",           /* Major/MinorSubsystemVersion */
              spec->subsystem_major, spec->subsystem_minor );
     output( "\t.long 0\n" );                          /* Win32VersionValue */
-    output( "\t.long %s-.L__wine_spec_rva_base\n",    /* SizeOfImage */
-             asm_name("_end") );
+    output_rva( "%s", asm_name("_end") ); /* SizeOfImage */
     output( "\t.long %u\n", page_size );  /* SizeOfHeaders */
     output( "\t.long 0\n" );              /* CheckSum */
     output( "\t.short 0x%04x\n",          /* Subsystem */
@@ -606,41 +737,17 @@ void output_module( DLLSPEC *spec )
     output( "\t.long 0\n" );              /* LoaderFlags */
     output( "\t.long 16\n" );             /* NumberOfRvaAndSizes */
 
-    if (spec->base <= spec->limit)   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT] */
-        output( "\t.long .L__wine_spec_exports-.L__wine_spec_rva_base,"
-                 ".L__wine_spec_exports_end-.L__wine_spec_exports\n" );
-    else
-        output( "\t.long 0,0\n" );
+    if (spec->base <= spec->limit)
+        data_dirs[0] = ".L__wine_spec_exports";   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT] */
+    if (has_imports())
+        data_dirs[1] = ".L__wine_spec_imports";   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT] */
+    if (spec->nb_resources)
+        data_dirs[2] = ".L__wine_spec_resources"; /* DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE] */
 
-    if (has_imports())   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT] */
-        output( "\t.long .L__wine_spec_imports-.L__wine_spec_rva_base,"
-                 ".L__wine_spec_imports_end-.L__wine_spec_imports\n" );
-    else
-        output( "\t.long 0,0\n" );
-
-    if (spec->nb_resources)   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE] */
-        output( "\t.long .L__wine_spec_resources-.L__wine_spec_rva_base,"
-                 ".L__wine_spec_resources_end-.L__wine_spec_resources\n" );
-    else
-        output( "\t.long 0,0\n" );
-
-    output( "\t.long 0,0\n" );  /* DataDirectory[3] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[4] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[5] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[6] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[7] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[8] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[9] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[10] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[11] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[12] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[13] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[14] */
-    output( "\t.long 0,0\n" );  /* DataDirectory[15] */
+    output_data_directories( data_dirs );
 
     output( "\n\t%s\n", get_asm_string_section() );
     output( "%s\n", asm_globl("__wine_spec_file_name") );
-    output( ".L__wine_spec_file_name:\n" );
     output( "\t%s \"%s\"\n", get_asm_string_keyword(), spec->file_name );
     if (target_platform == PLATFORM_APPLE)
         output( "\t.lcomm %s,4\n", asm_name("_end") );
@@ -650,23 +757,23 @@ void output_module( DLLSPEC *spec )
 
 
 /*******************************************************************
- *         BuildSpec32File
+ *         output_spec32_file
  *
  * Build a Win32 C file from a spec file.
  */
-void BuildSpec32File( DLLSPEC *spec )
+void output_spec32_file( DLLSPEC *spec )
 {
     needs_get_pc_thunk = 0;
-    resolve_imports( spec );
+    open_output_file();
     output_standard_file_header();
     output_module( spec );
     output_stubs( spec );
     output_exports( spec );
     output_imports( spec );
-    if (is_undefined( "__wine_call_from_regs" )) output_asm_relays();
     if (needs_get_pc_thunk) output_get_pc_thunk();
     output_resources( spec );
     output_gnu_stack_note();
+    close_output_file();
 }
 
 
@@ -876,7 +983,7 @@ void output_fake_module( DLLSPEC *spec )
  *
  * Build a Win32 def file from a spec file.
  */
-void output_def_file( DLLSPEC *spec, int include_private )
+void output_def_file( DLLSPEC *spec, int include_stubs )
 {
     DLLSPEC *spec32 = NULL;
     const char *name;
@@ -903,16 +1010,17 @@ void output_def_file( DLLSPEC *spec, int include_private )
     for (i = total = 0; i < spec->nb_entry_points; i++)
     {
         const ORDDEF *odp = &spec->entry_points[i];
-        int is_data = 0;
+        int is_data = 0, is_private = odp->flags & FLAG_PRIVATE;
 
         if (odp->name) name = odp->name;
         else if (odp->export_name) name = odp->export_name;
         else continue;
 
-        if (!(odp->flags & FLAG_PRIVATE)) total++;
-        else if (!include_private) continue;
+        if (!is_private) total++;
+        if (!include_stubs && odp->type == TYPE_STUB) continue;
 
-        if (odp->type == TYPE_STUB) continue;
+        if ((odp->flags & FLAG_FASTCALL) && target_platform == PLATFORM_WINDOWS)
+            name = strmake( "@%s", name );
 
         output( "  %s", name );
 
@@ -923,7 +1031,6 @@ void output_def_file( DLLSPEC *spec, int include_private )
             /* fall through */
         case TYPE_VARARGS:
         case TYPE_CDECL:
-        case TYPE_THISCALL:
             /* try to reduce output */
             if(strcmp(name, odp->link_name) || (odp->flags & FLAG_FORWARD))
                 output( "=%s", odp->link_name );
@@ -933,23 +1040,22 @@ void output_def_file( DLLSPEC *spec, int include_private )
             int at_param = get_args_size( odp );
             if (!kill_at && target_cpu == CPU_x86) output( "@%d", at_param );
             if  (odp->flags & FLAG_FORWARD)
-            {
                 output( "=%s", odp->link_name );
-            }
             else if (strcmp(name, odp->link_name)) /* try to reduce output */
-            {
-                output( "=%s", odp->link_name );
-                if (!kill_at && target_cpu == CPU_x86) output( "@%d", at_param );
-            }
+                output( "=%s", get_link_name( odp ));
             break;
         }
+        case TYPE_STUB:
+            if (!kill_at && target_cpu == CPU_x86) output( "@%d", get_args_size( odp ));
+            is_private = 1;
+            break;
         default:
             assert(0);
         }
         output( " @%d", odp->ordinal );
         if (!odp->name || (odp->flags & FLAG_ORDINAL)) output( " NONAME" );
         if (is_data) output( " DATA" );
-        if (odp->flags & FLAG_PRIVATE) output( " PRIVATE" );
+        if (is_private) output( " PRIVATE" );
         output( "\n" );
     }
     if (!total) warning( "%s: Import library doesn't export anything\n", spec->file_name );
