@@ -462,6 +462,10 @@ struct strarray get_ld_command(void)
             break;
         }
     }
+
+    if (target_cpu == CPU_ARM && target_platform != PLATFORM_WINDOWS)
+        strarray_add( &args, "--no-wchar-size-warning", NULL );
+
     return args;
 }
 
@@ -893,15 +897,45 @@ const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec )
 const char *get_link_name( const ORDDEF *odp )
 {
     static char *buffer;
+    char *ret;
 
-    if (!kill_at && target_platform == PLATFORM_WINDOWS && target_cpu == CPU_x86 &&
-        odp->type == TYPE_STDCALL && !(odp->flags & FLAG_THISCALL))
+    if (target_cpu != CPU_x86) return odp->link_name;
+
+    switch (odp->type)
     {
-        free( buffer );
-        buffer = strmake( "%s@%u", odp->link_name, get_args_size( odp ));
-        return buffer;
+    case TYPE_STDCALL:
+        if (target_platform == PLATFORM_WINDOWS)
+        {
+            if (odp->flags & FLAG_THISCALL) return odp->link_name;
+            if (odp->flags & FLAG_FASTCALL) ret = strmake( "@%s@%u", odp->link_name, get_args_size( odp ));
+            else if (!kill_at) ret = strmake( "%s@%u", odp->link_name, get_args_size( odp ));
+            else return odp->link_name;
+        }
+        else
+        {
+            if (odp->flags & FLAG_THISCALL) ret = strmake( "__thiscall_%s", odp->link_name );
+            else if (odp->flags & FLAG_FASTCALL) ret = strmake( "__fastcall_%s", odp->link_name );
+            else return odp->link_name;
+        }
+        break;
+
+    case TYPE_PASCAL:
+        if (target_platform == PLATFORM_WINDOWS && !kill_at)
+        {
+            int args = get_args_size( odp );
+            if (odp->flags & FLAG_REGISTER) args += get_ptr_size();  /* context argument */
+            ret = strmake( "%s@%u", odp->link_name, args );
+        }
+        else return odp->link_name;
+        break;
+
+    default:
+        return odp->link_name;
     }
-    return odp->link_name;
+
+    free( buffer );
+    buffer = ret;
+    return ret;
 }
 
 /* parse a cpu name and return the corresponding value */
@@ -909,7 +943,7 @@ int get_cpu_from_name( const char *name )
 {
     unsigned int i;
 
-    for (i = 0; i < sizeof(cpu_names)/sizeof(cpu_names[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(cpu_names); i++)
         if (!strcmp( cpu_names[i].name, name )) return cpu_names[i].cpu;
     return -1;
 }
@@ -1036,6 +1070,7 @@ const char *asm_name( const char *sym )
     {
     case PLATFORM_WINDOWS:
         if (target_cpu != CPU_x86) return sym;
+        if (sym[0] == '@') return sym;  /* fastcall */
         /* fall through */
     case PLATFORM_APPLE:
         if (sym[0] == '.' && sym[1] == 'L') return sym;
